/* arp.h
 *
 * Copyright 2003-2008 by Praemio Inc. All rights reserved.
 * Author: John Bartas - jbartas@praemio.com
 * This software is released under the terms of the LGPL.
 * See LGPL Notice at end of this file; Full license in ../license.txt
 *
 * ARP protocol definitions. This includes support for Buster to 
 * send gratuitous arps. 
 *
 */


#ifndef _ARP_H_
#define  _ARP_H_   1

#define     ET_ARP         htons(0x0806)  /* net endian 0x0806 */
#define     MACADDSIZ      6              /* biggest mac address we use */
#define     ARP_ETHTYPE    htons(1)
#define     ARP_REQCODE    htons(1)
#define     ARP_REPCODE    htons(2)
#define     ARP_IPTYPE     htons(0x0800)  /* IP type in net endian */


struct arpentry
{
   struct arpentry * next;         /* hash bucket link */
   ip_addr          ipaddr;
   unsigned char    macaddr[6];
   NET              net;            /* iface for this entries MAC */
   PACKET           pending;        /* send IP packet pending on reply */
   u_long           createtime;     /* ctick entry was created */
   u_long           lasttime;       /* ctick entry was last referenced */
};

#define  MAXARPS    32          /* maximum mumber of arp table entries */
extern   struct arpentry * arp_table[MAXARPS];  /* the actual table */


int   arp_rcv (PACKET);                         /* arp receive callback */
int   arp_send(PACKET pkt, ip_addr dest_ip);    /* send packet via arp system */
int   send_via_arp(PACKET pkt, ip_addr nexthop);
int   arp_scan(int iface);
void  arp_cleanup(void);
struct arpentry *   send_arp_net(NET net, ip_addr dest_ip);
struct arpentry *   arp_lru(ip_addr dest_ip);
struct arpentry *   arp_mkentry(ip_addr dest_ip, NET net);

extern   int   arp_scanning[MAXNETS];   /* flags/IPs for arp_scan() */

struct arpstat
{
    u_long    reqsIn;     /* requests received */
    u_long    reqsOut;    /* requests sent */
    u_long    repsIn;     /* replys received */
    u_long    repsOut;    /* replys sent */
};
extern struct arpstat arpstats;


/* the ARP header */
struct arp_hdr   
{
   unshort     ar_hd;      /* hardware type */
   unshort     ar_pro;     /* protcol type */
   char        ar_hln;     /* hardware addr length */
   char        ar_pln;     /* protocol header length */
   unshort     ar_op;      /* opcode 1=request, 2=reply */
   char        data[20];   /* send IP, send eth, target IP, target eth */
};

/* offsets to fields in arp_hdr->data[] */
#define  ARP_SRCMACHA   0
#define  ARP_SRCIP      6
#define  ARP_DESTMAC    10
#define  ARP_DESTIP     16

#endif   /* _ARP_H_ */

/*
Copyright (C) 2003-2008 by Praemio Inc. All rights reserved.

Author: John Bartas - jbartas@praemio.com

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


