/* bralert.c
 *
 * Buster alert support.
 *
 * ROUTINES: 
 * br_getnametext(), alarm_general(), alert_check(), 
 * alert_init(), br_logentry(), br_log(), 
 *
 * Copyright 2003-2008 by Praemio Inc. All rights reserved.
 * Author: John Bartas - jbartas@praemio.com
 * This software is released under the terms of the LGPL.
 * See LGPL Notice at end of this file; Full license in ../license.txt
 */


#include "busterport.h"
#include "brnet.h"
#include "ip.h"
#include "brdb.h"


brqueue alarms_free;      /* set of available alarm data structures */
brqueue alarms_queued;    /* Alarm data structures queued for alarm task */

extern   void  br_flash(int itterate, int level);

extern int br_system_shutdown;

char * logfilename = "buster.log";
char * oldlogfilename = "busterbk.log";

/* predecl */
void  br_logentry(struct alert_event * evt);

/* Create a thread to handle alerts */
TH_OBJECT(to_alert);
TH_PROTO(te_alert);

struct br_thread alert_thread = 
{
    &to_alert,
    "Alerts",
    te_alert,
    APP_PRIORITY,
    APP_STACK_SIZE,
};


/* Buster alarm definitions and default options. */
struct alarm_type newmac = {
   "newmac", 
   "New computer on network", 
   "A new computer has appeared on the local network", 
   AL_EMAIL | AL_AUDIO | AL_POPUP | AL_CONSOLE,
   (TPS * 10),    /* backoff internval seed */
   "www.praemio.info/alerts/newhost.html",
   NULL,
};

struct alarm_type conn_limit = {
   "conns",
   "Too many connections",
   "Suspiciously large number of connections to/from computer %s",
   AL_EMAIL | AL_AUDIO | AL_POPUP | AL_CONSOLE,
   (TPS * 10),    /* backoff internval seed */
   "www.praemio.info/alerts/cnlimit.html",
   NULL,
};

struct alarm_type blocked_service = {
   "svc",
   "Accessing restricted service",
   "Connections attempted to restricted service \"%s\" \r\nfrom local computer:",
   AL_EMAIL | AL_AUDIO | AL_POPUP | AL_CONSOLE,
   (TPS * 10),    /* backoff internval seed */
   "www.praemio.info/alerts/blocksvc.html",
   NULL,
};


struct alarm_type blocked_ip = {
   "ip",
   "Accessing restricted site",
   "Connections attempted to restricted server \"%s\" \r\nfrom local computer:",
   AL_EMAIL | AL_AUDIO | AL_POPUP | AL_CONSOLE,
   (TPS * 10),    /* backoff internval seed */
   "www.praemio.info/alerts/blockip.html",
   NULL,
};

struct alarm_type virus_detected = {
   "virus", 
   "Virus detected",
   "Virus \"%s\" detected on connection ",
   AL_EMAIL | AL_AUDIO | AL_POPUP | AL_CONSOLE,
   0,          /* backoff internval seed (0 == no interval) */
   "www.praemio.info/alerts/virus.html",
   NULL,
};

struct alarm_type bad_app = {
   "prog",
   "Detected restriced application",
   "Restricted application %s tried to access the network",
   AL_EMAIL | AL_AUDIO | AL_POPUP | AL_CONSOLE,
   (TPS * 10),    /* backoff internval seed */
   "www.praemio.info/alerts/badapp.html",
   NULL,
};

struct alarm_type syn_flood = {
   "flood",
   "Detected \"SYN flood\" attack",
   "Apparent \"SYN\" flood detected from %s",
   AL_EMAIL | AL_AUDIO | AL_POPUP | AL_CONSOLE,
   (TPS * 10),    /* backoff internval seed */
   "www.praemio.info/alerts/synflood.html",
   NULL,
};


/* Alarm for remote computers doing port/Ip scans */

struct alarm_type remote_scan = {
   "scan",
   "Detected IP or port scan",
   "IP or port scan detected from %s",
   AL_EMAIL | AL_AUDIO | AL_POPUP | AL_CONSOLE,
   0,          /* backoff internval seed */
   "www.praemio.info/alerts/rscan.html",
   NULL,
};

/* Alarm to notify user if Buster can't get an IP address. */
struct alarm_type no_ipaddr = 
{
   "no_ipaddr",
   "Network has no IP address",
   "Viewable or active network %s has no IP address",
   AL_EMAIL | AL_AUDIO | AL_POPUP | AL_CONSOLE,
   (TPS * 10),    /* backoff internval seed */
   "www.praemio.info/alerts/noip.html",
   NULL,
};


/* Alarm for simple text "FYI" messages */

struct alarm_type alarm_text = {
   "text",
   "Other minor alarms",
   "%s",
   AL_EMAIL | AL_AUDIO | AL_POPUP | AL_CONSOLE,
   (MINUTE),          /* backoff internval seed */
   "www.praemio.info/alerts/text.html",
   NULL,
};


struct alarm_type dup_ipaddr = {
   "dup_ip",
   "Duplicate IP address",
   "%s",
   AL_EMAIL | AL_AUDIO | AL_POPUP | AL_CONSOLE,
   (MINUTE),          /* backoff internval seed */
   "www.praemio.info/alerts/dupip.html",
   NULL,
};

struct alarm_type comm_file = {
   "cmfile",
   "Community DB file problem",
   "%s",
   AL_EMAIL | AL_AUDIO | AL_POPUP | AL_CONSOLE,
   (SECOND * 2),          /* backoff internval seed */
   "www.praemio.info/alerts/cmfile.html",
   NULL,
};

struct alarm_type comm_update = {
   "cmupdate",
   "Community DB update problem",
   "%s",
   AL_EMAIL | AL_AUDIO | AL_POPUP | AL_CONSOLE,
   (MINUTE),          /* backoff internval seed */
   "www.praemio.info/alerts/cmupdate.html",
   NULL,
};

struct alarm_type bad_dhcp = {
   "dhcp",
   "Bad DHCP packet on local net",
   "%s",
   AL_EMAIL | AL_AUDIO | AL_POPUP | AL_CONSOLE,
   (MINUTE),          /* backoff internval seed */
   "www.praemio.info/alerts/dhcp.html",
   NULL,
};


struct alarm_type dead_iface = {
   "interface",
   "A network Interface has been disabled",
   "Network Interface %s has been disabled",
   AL_EMAIL | AL_AUDIO | AL_POPUP | AL_CONSOLE,
   (MINUTE),          /* backoff internval seed */
   "www.praemio.info/alerts/dead_if.html",
   NULL,
};

struct alarm_type no_iface = {
   "interface",
   "No working network interfaces",
   "The system has no working LANs",
   AL_EMAIL | AL_AUDIO | AL_POPUP | AL_CONSOLE,
   (MINUTE),          /* backoff internval seed */
   "www.praemio.info/alerts/no_ifs.html",
   NULL,
};

struct alarm_type   forged_tcp = {    /* Forged TCP control packet */
   "forgery",
   "Detected forged TCP Packet",
   "%s",
   AL_EMAIL | AL_AUDIO | AL_POPUP | AL_CONSOLE,
   (MINUTE),          /* backoff internval seed */
   "www.praemio.info/alerts/forged_tcp.html",
   NULL,
};


/* list of alarms for batch processing (disabled action bits). 
 * A pointer to each alarm should be maintained here. 
 */

struct alarm_type * alarms[] = 
{
   &newmac,
   &conn_limit,
   &blocked_service,
   &blocked_ip,
   &virus_detected,
   &bad_app,
   &syn_flood,
   &remote_scan,
   &alarm_text,
   &no_ipaddr,
   &dup_ipaddr,
   &comm_file,
   &comm_update,
   &bad_dhcp,
   &dead_iface,
   &no_iface,
   &forged_tcp,
};

int alertcount = (sizeof(alarms)/sizeof(struct alarm_type *));

/* Tables of event info th support GUI logic in ..\busthtml\cfgcgi.c */

/* base method strings to build alert control names, along with associated bits */

struct a_method a_methods[] =
{
    {   "email", AL_EMAIL},
    {   "popup", AL_POPUP},
    {   "cons",  AL_CONSOLE},
    {   "audio", AL_AUDIO}
};

int   alert_methodct = (sizeof(a_methods)/sizeof(struct a_method));

/* instances for global alarm types */
struct alarm_inst gen_cflood;
struct alarm_inst dupipaddr;
struct alarm_inst baddhcp;


#define ABSIZE 256


char getnamebuf[MAXNAMELEN];

/* br_getnametext() - get the best printable name for an IP address */

char * 
br_getnametext(ip_addr ipaddr)
{
   char * cp;
   struct ip_hostentry * iphost;

   iphost = br_iplookup(br_ifview, ipaddr);
   if((!iphost) || (*(cp = br_getname(iphost)) == '('))
   {
      sprintf(getnamebuf, "%u.%u.%u.%u", PASS_IPADDR(ipaddr));
      return getnamebuf;
   }
   return cp;
}

/* br_prtmac() - Pretty-print a MAC address for display. 
 * 
 * Returns a pointer to a static buffer containing the MAC address 
 * in formatted hex. Since two MAC addresses are often pronted in 
 * a single printf(), theis routine alternates between two buffers.
 */

static char    macbuf[2][24];
static int     lastmac;      /* alternates between 0 and 1 */

char *
br_prtmac(u_char * macaddr)
{
   char * cp;

   /* If passed pointer is NULL or absurdly low value (i.e. derived 
    * from offset info pointer to null struct) The dtrap and return 
    * empty string.
    */
   if(macaddr < (u_char*)(0x100))
   {
      dtrap();
      return "";
   }

   lastmac = (~lastmac) & 1;  /* alternate buffers */
   cp = &macbuf[lastmac][0];  /* get buffer to use */

   sprintf( cp, "%02X %02X %02X - %02X %02X %02X",
      *(macaddr),
      *(macaddr + 1),
      *(macaddr + 2),
      *(macaddr + 3),
      *(macaddr + 4),
      *(macaddr + 5));

   return cp;
}


/************************ alarm/alert/log routines ************* 
 * Routines to generate user alarms. The first of the two routines 
 * below, alarm_general(), gets an alarm event struct, copies in 
 * the alarm parameters, queues the struct for later handleing by 
 * a lower priority "handler" thread, and then wakes the handler 
 * thread. This routine is called from the Buster receive handler 
 * and thus must be fast.
 *
 *    The second routine below, alert_check(), 
 * is called by the handler thread's core loop
 * when it awakens. This handler routine does the 
 * potentially slow work of posting the alarm.
 */


/* Return number of cticks since last resume or coldstart */
u_long br_resumeticks()
{
   if(br_lastresume)    /* this is a resume */
       return ( (br_gettime() - br_lastresume) * TPS );
   else     /* this is a cold start */
      return cticks;
}


void 
alarm_general(struct alarm_type * type,      /* pointer to generic alert */
              struct alarm_inst * instance,  /* instance of alert */
              void * parm)                   /* misc */
{
   struct alert_event * evt;

   /* if this is not first alarm, see if it's time to send another */
   if(instance)      /* make sure there's an instance timer */
   {
      if(instance->alarmcount != 0)
      {
         u_long nextalarm;
         nextalarm = instance->last_ctick + 
                  (type->interval << instance->alarmcount);
         if(nextalarm > cticks)     /* not time for next alarm yet? */
            return;
      }
      instance->last_ctick = cticks;
      instance->alarmcount++;
   }

   /* Don't worry about new hosts in the first few minutes of operation */
   if(type == &newmac)
   {
      if(br_resumeticks() < (5 * MINUTE))
         return;
   }

   /* Dont report IP address of zero in first few minutes after start or resume */
   if(type == &no_ipaddr)
   {
      if(br_resumeticks() < (3 * MINUTE) )
         return;
   }

   /* get a queue-able alarm event and fill it in */
   evt = (struct alert_event * )getq(&alarms_free);
   if(!evt)
   {
      dprintf("warning - out of free alarm events\n");
      /* spin the output task to try and free some events */
      TH_WAKE( (long)(&to_alert) );
      return;
   }
   evt->data.alarm.type = type;
   evt->data.alarm.instance = instance;
   evt->data.alarm.parm = parm;

   putq(&alarms_queued, evt);
   TH_WAKE( (long)(&to_alert) );
   return;
}

static    char alertbuf[ABSIZE];

void
alert_check()
{
   struct host_entry * host;     /* host which triggered alert */
   ip_addr     ipaddr;           /* address of host */
   char *      cp;
   struct alarm_type * type;
   struct alert_event * evt;
   struct alarm_inst * instance;  /* instance of alert */
   void * parm;


   /* Get an alarm event from the queue, copy parameters into local
    * variables, and return event to event free queue.
    */
   evt = getq(&alarms_queued);
   if(!evt)
      return;

#ifdef BLOCKING_LOG
   /* Send blocking entries to block log */
   if(evt->flags & AF_BLOCK)
      blklog_log(evt);
#endif /* BLOCKING_LOG */

   /* If it's a log entry, handle it and return */
   if(evt->flags & AF_LOGENTRY)
   {
      br_logentry(evt);
      evt->flags = 0;
      putq(&alarms_free, evt);
      return;
   }

   /* process an alarm */
   type = evt->data.alarm.type;
   instance = evt->data.alarm.instance;
   parm = evt->data.alarm.parm;
   putq(&alarms_free, (qp)evt);

   /* fill in the text about the alert event. This should be a "switch" 
    * statement, but the Microsnot compiler complains (incorrectly) that 
    * "addresses are not constant".
    */
   if(type == &newmac)
   {
      int   i;
      u_char tmp[6];

      host = (struct host_entry *)parm;   /* parm is new MAC host */
      sprintf(alertbuf, type->message);
      strcat(alertbuf, "\n MAC address: ");
      cp = &alertbuf[strlen(alertbuf)];

      /* skip hosts which may have been deleted since they were passed */
      if((host->iface >= 0) &&
         (host->iface < (int)ifNumber) &&
         ((host->macaddr[0] & 1) == 0))    /* skip bcast/mcast */
      {
         char * name;

         MEMCPY(tmp, host->macaddr, 6);
         for(i = 0; i < 6; i++)
         {
            sprintf(cp, "%02x ", tmp[i] );
            cp+= 3;
         }
         if(host->locname[0])
            name = host->locname;
         else if(host->iphost && host->iphost->dnsname[0])
            name = host->iphost->dnsname;
         else 
            name = NULL;

         if(host->iphost)
         {
            ipaddr = host->iphost->ipaddr;
            cp = &alertbuf[strlen(alertbuf)];

            if(name)
            {
               sprintf(cp, "\n name: %s, ", name);
               cp = &alertbuf[strlen(alertbuf)];
            }

            if(ipaddr)
            {
               sprintf(cp, "\n IP address: %u.%u.%u.%u\n", 
                  PASS_IPADDR(ipaddr));
            }
            else
            {
               sprintf(cp, "No IP address (0.0.0.0)\n");
            }

            if(host->disctype == H_DHCP)
               sprintf(cp, "- DHCP request\n");
            else if(host->disctype == H_NETBEUI)
               sprintf(cp, "- NetBios packet\n");
         }
      }
   }
   else if((type == &conn_limit) || /* too many IP connections */
      (type == &syn_flood) ||       /* Syn flood from IP */
      (type == &remote_scan))       /* IP or port scan */
   {
      struct ip_hostentry * iphost = (struct ip_hostentry *)parm;
      if(iphost == NULL)
      {
         /* parm can be null for network-wide connection limit exceeded; but
          * for per-host connection limit exceeded, or other alarms, it
          * should be the IP host entry for the offender 
          */
         if(instance != &gen_cflood)
         {
            strcat(alertbuf, "Too many connections, exceeding system limit");
         }
         else
            strcat(alertbuf, type->message);
      }
      else
      {
         cp = br_getname(iphost);      /* try for text name */
         if(!cp || (cp == noname))     /* if no name... */
            cp = print_ipad(iphost->ipaddr);    /* format dot notation */
         sprintf(alertbuf, type->message, cp);
      }
   }
   else if((type ==  &blocked_service) ||
      (type == &bad_app) ||
      (type == &blocked_ip))
   {
      dtrap();    /* check parameter */
      sprintf(alertbuf, type->message);
   }
   else if(type == &virus_detected)
   {
      char * vdata;
      struct   pattern_entry * pep;
      struct   virussig * vp;
      struct conn_entry * conn;

      pep = (struct pattern_entry *)parm;
      vp = (struct virussig *)pep->data;
      vdata = vp->name;      /* get pointer to virus name */
      
      sprintf(alertbuf, type->message, vdata);
      strcat(alertbuf, "Connection ");
      conn = (struct conn_entry *)type->parm;
      strcat(alertbuf, br_getnametext(conn->rem.ce_addr));
      strcat(alertbuf, "<->");
      strcat(alertbuf, br_getnametext(conn->loc.ce_addr));
      strcat(alertbuf, ", ports: ");
      cp = &alertbuf[strlen(alertbuf)];
      sprintf(cp, "%u", conn->rem.ce_port);
      strcat(alertbuf, "<->");
      cp = &alertbuf[strlen(alertbuf)];
      sprintf(cp, "%u", conn->loc.ce_port);
      strcat(alertbuf, " \n");
   }
   else if(type == &forged_tcp)
   {
      struct conn_entry * conn;

      conn = (struct conn_entry *)parm;
      sprintf(alertbuf, "Suspicious %s packet from remote;\n", 
         (conn->ce_flags & CEF_FAKERESET)?"RESET":"FIN");
      cp = &alertbuf[strlen(alertbuf)];
      sprintf(cp, "remote: %u.%u.%u.%u:%u\n", PASS_IPADDR(conn->rem.ce_iphost->ipaddr), 
         conn->rem.ce_port);
      cp = &alertbuf[strlen(alertbuf)];
      sprintf(cp, "local: %u.%u.%u.%u:%u\n", PASS_IPADDR(conn->loc.ce_iphost->ipaddr), 
         conn->loc.ce_port);

   }
   else if(type == &bad_dhcp)
   {
      struct ip * pip;
      struct udp * pup;
      char * name;
      struct ip_hostentry * iphost;

      pip = (struct ip *)type->parm;
      pup = (struct udp *)(ip_data(pip));

      sprintf(alertbuf, type->message, parm);
      cp = &alertbuf[strlen(alertbuf)];
      sprintf(cp, "Sender IP:%u.%u.%u.%u \n", PASS_IPADDR(pip->ip_src));

      iphost = br_iplookup(br_ifview, pip->ip_src);
      name = br_getname(iphost);
      if(name != noname)
      {
         strcat(cp, "Sender Name: ");
         strcat(cp, name);
         strcat(cp, " \n");
      }
   }
   else 
   {
      sprintf(alertbuf, type->message, parm);
   }

   /* always invoke the "flash" indicator. This should flash an LED
    * or GUI icon.
    */
   br_flash(10, 1);

   /* invoke the other alert methods as indicated by user settings */
   if(type->actions & AL_CONSOLE)
      dprintf("ALERT: %s\n", alertbuf);

   if(type->actions & AL_AUDIO)
      br_beep();

   if(type->actions & AL_POPUP)
      br_popup( type, alertbuf );

#ifdef SMTP_ALERTS
   if(type->actions & AL_EMAIL)
      smtpalert( alertbuf, strlen(alertbuf), NULL );
#endif

   return;
}

/* alert_init(void)
 * 
 *    Called from the Buster port's initialization logic to
 * set up the event free queue for the routines above.
 *
 * RETURNS: 0 if OK, or negative ENP_ error code. 
 */

int  
alert_init(void)
{
   int   i;
   struct alert_event * evt;


   for(i = 0; i < ALERT_NUM_EVENTS; i++)
   {
      evt = (struct alert_event * )AL_ALLOC(sizeof(struct alert_event));
      if(!evt)
         return ENP_NOMEM;
      putq(&alarms_free, evt);
   }

   i = TH_NEWTASK(&alert_thread);

   return i;     /* return error */
}


/* Buster requires one thread to Handle 
 * alerts and log events. 
 */

TH_ENTRY(te_alert)
{
   for(;;)
   {
      /* Is TH_BLOCK/TH_WAKE broken? If doesn't seem 
       * to work here. -JB-
       */
//      TH_BLOCK();
      TH_SLEEP(2);

      if(br_system_shutdown)
          TH_RETURN_OK();

      /* handle alerts and log entries as they roll in */
      while(alarms_queued.q_len)
         alert_check();
   }
   TH_RETURN_OK();
}
TH_EXIT("te_alert")




/* br_log(char * format, ...) - printf-like logging 
 * function for Buster.
 *
 * This loads the passed parametwers into an alert structure 
 * and passes it to the thread which will do the actual file 
 * write. We don't do the write here for speed reasons - 
 * this is usually called form the high-priority Buster 
 * rx_packet thread, which must not be blocked by a file 
 * write.
 */

FILE *   br_setlogfile(void);    /* open & timestamp log file */

int br_inlog = 0;

extern char * prompt;

void
br_log(int flags, char * format, ...)
{
   va_list  va;
   struct alert_event * evt;
   int      i;
   int      len;
   char *   cp;

   /* Don't echo simple prompts. */
   if(strcmp(format, prompt) == 0)
      return;

   /* don't get into this routine recursivly */
   if(br_inlog++)
   {
//       dtrap();
      br_inlog--;
      return;
   }

   /* Get a queue-able alarm event and fill it in. */
   evt = (struct alert_event * )getq(&alarms_free);
   if(!evt)
   {
      dprintf("warning - out of free log/alert events\n");
      br_inlog--;
      /* spin the output task to try and free some events */
      TH_WAKE( (long)(&to_alert) );
      return;
   }
   /* Send a copy of the format string rather than the actual
    * string itself. This allows dprintf() to queue several log
    * entries with them all looking like the last one queued.
    */
   len = strlen(format) + 1;    /* allow a char for null term. */
   cp = LM_ALLOC(len);          /* Allocate a log message buffer */
   if(!cp)
   {
       dtrap();
      br_inlog--;
      return;
   }
   MEMCPY(cp, format, len);
   evt->data.log.format = cp;

   /* copy the rest of the args into the event structure */
   va_start(va, format);
   for(i = 0; i < BRLOG_PARMS; i++)
      evt->data.log.va_parms[i] = va_arg(va, long);
   va_end(va);

   evt->flags = flags;
   evt->flags |= AF_LOGENTRY;    /* mark event as a log entry */

   putq(&alarms_queued, evt);
   TH_WAKE( (long)(&to_alert) );
   br_inlog--;
   return;
}


char     lastlogday[DATEBUFLEN];  /* Date string for last log entry */

/* lasttime - Time of last write to log file. This is used to 
 * decide if br_setlogfile() should timestamp the current line 
 */
static   u_long   lasttime = 0xFFFFFFFF;  /* force stamp on 1st entry */

void
br_logentry(struct alert_event * evt)
{
   FILE * brlog;

   brlog = br_setlogfile();
   if(brlog == NULL)
      return;

   fprintf(brlog, evt->data.log.format, 
      evt->data.log.va_parms[0], evt->data.log.va_parms[1],
      evt->data.log.va_parms[2], evt->data.log.va_parms[3],
      evt->data.log.va_parms[4], evt->data.log.va_parms[5],
      evt->data.log.va_parms[6], evt->data.log.va_parms[7],
      evt->data.log.va_parms[8], evt->data.log.va_parms[9]
      );

   /* free the buffer allocaled for format string in br_log() */
   LM_FREE(evt->data.log.format);
   fclose(brlog);
   br_flash(1, 0);
   return;
}

FILE *
br_setlogfile()
{
   FILE *   brlog;
   long     size;
   char *   datestamp;
   char *   timestamp;
   int      loops = 0;

set_again:
   if(loops > 3)
   {
      dprintf("br_setlogfile: Can't swap out log\n");
      dtrap();
      return NULL;
   }
   /* open log, create if it doesn't exist */
   brlog = fopen(logfilename, "a");
   if(!brlog)
   {
      dprintf("unable to open log file %s\n", logfilename);
      return NULL;
   }

   fseek(brlog, 0L, SEEK_END);    /* seek to end of file */
   size = ftell(brlog);
   fseek(brlog, 0L, SEEK_SET);    /* restore file pointer to beginning */

   if(size > 100000)
   {
      /* Size of log file has reached the limit. We rename the 
       * Buster log file to the oldlogfile name and start a new
       * log file. If there is already an old log file, we 
       * erase it.
       */
      fclose(brlog);
      unlink(oldlogfilename);    /* get rid of old log file */
      rename(logfilename, oldlogfilename);      /* rename current log file */
      loops++;
      goto set_again;            /* setup the new log file */
   }
   datestamp = br_getlogdate();     /* get string with current date */

   if(strcmp(lastlogday, datestamp))   /* did day change? */
   {
      /* put new date */
      fprintf(brlog, "%s\n", datestamp);
      strcpy(lastlogday, datestamp);
   }

   /* If this is a new line, start it with a timestamp */
   if(lasttime != cticks)     /* last line ended with linefeed */
   {
      timestamp = br_getlogtime();     /* get time of day string */
      fprintf(brlog, timestamp);
      lasttime = cticks;
   }
   else     /* just indent a few spaces */
   {
      fprintf(brlog, "   ");
   }

   return brlog;
}

/*
Copyright (C) 2003-2008 by Praemio Inc. All rights reserved.

Author: John Bartas - jbartas@praemio.com

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


