/* brapp.c
 *
 * Buster code to handle events from built-in applications (e.g. 
 * Interniche Web server)
 *
 * Copyright 2003-2008 by Praemio Inc. All rights reserved.
 * Author: John Bartas - jbartas@praemio.com
 * This software is released under the terms of the LGPL.
 * See LGPL Notice at end of this file; Full license in ../license.txt
 */


#include "busterport.h"
#include "brnet.h"
#include "ip.h"
#include "in_utils.h"

// Include webio headers here



/* hooks to pass suspicious events from http code to Buster. */

int
br_htproblem(int opcode, void * hp_void)
{
   ip_addr        ipaddr;
   struct httpd * hp;

   hp = (void *)hp_void;      /* convert parameter back to HT struct */
   ipaddr = 0;                /* fix this later? -JB- */

   switch(opcode)
   {
   case HT_BADOPCODE:   /* bad HTTP opcode from browser */
      br_log(0, "Bad HTTP opcode (%d) from IP %u.%u.%u.%u (attack?)\n",
         hp->op, ipaddr);
      break;
   default:
      dtrap();
      break;
   }

	return 0;
}

/* br_filecheck() - check non-vfs web transaction for security.
 *
 * returns 0 if OK, else kill file xfer.
 */


int
br_filecheck(struct httpd * hp)
{
   char * clonefile = "agent_setup";
   char * custom;
   char * cp;

   /* allow the clone file */
   if(strncmp(hp->requri + 1, clonefile, strlen(clonefile) ) == 0)
   {
      hp->requri++;
      return  0;
   }

   /* Allow VFS files (VFS will enforce authentication */
   if(vfslookup_locked(hp->requri))
   {
      return 0;
   }

   /* Allow Customization files */
   custom = br_getdir(br_imagedir);
   if(custom)
   {
      cp = hp->requri;
      while(*cp == '/')
         cp++;

      if(strstr(custom, cp))
      {
         return 0;
      }
   }
   
   /* reject everything else */
   return -1;     /* Bad, kill the transfer */
}


/* routine to select an image file for a passed MAC host */

char *
br_image(struct host_entry * host)
{
   char *   image = "unknown.gif";
   struct image * img;
   struct btnode * btnp;

   if(!host)
   {
      dtrap(); /* bug */
      return image;
   }
   /* if host has an assigned image; use that */
   if(host->imagefile[0] != 0)
      return (host->imagefile);

   /* if host is the router, use router image */
   if(host == brstates[host->iface].br_router)
      return "rte.jpg";

   if(host->h_flags & H_BCAST)
      return "bcast.gif";

   /* try MAC address lookup. If we get a hit, replace the unknown with it. */
   btnp = bt_access((struct btnode *)image_tree, 
      (u_long)(&host->macaddr[0]), bt_mactest );

   if(btnp)
   {
      img = (struct image *)btnp->data;
      image = img->imagefile;
   }
   else
   {
      if(host->h_flags & H_WINDOWS)
         image = "pc.gif";
   }
   if(host->h_flags & H_LINUX)
      image = "linux.gif";

   return image;
}

/*
Copyright (C) 2003-2008 by Praemio Inc. All rights reserved.

Author: John Bartas - jbartas@praemio.com

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/



