/* brav.c
 *
 * Buster Anti-virus & fast scan routines. 
 *
 * Copyright 2003-2008 by Praemio Inc. All rights reserved.
 * Author: John Bartas - jbartas@praemio.com
 * This software is released under the terms of the LGPL.
 * See LGPL Notice at end of this file; Full license in ../license.txt
 *
 */


#include "busterport.h"
#include "brnet.h"
#include "ip.h"
#include "brdb.h"

u_long   sigct = 0;         /* total sigs put in table */
struct virussig * viruslist = NULL;     /* Start of linked list of sigs (for admin) */

#ifdef VIRUS_CHECK    /* whole antivirus support file can be removed */

#ifndef BR_COMMUNITIES
#error "VIRUS_CHECK option requires comminities"
#endif  /* BR_COMMUNITIES */

/* BRAV statistics */
u_long   virus_scanned = 0;      /* total bytes scanned */
u_long   virus_bithits = 0;      /* number of bitmask table matches */
u_long   virus_partial = 0;      /* number of partial matches */
u_long   virus_found = 0;        /* number of full matches */

struct virussig * viruslist;     /* Start of linked list of sigs (for admin) */
struct virussig * viruslast;     /* End of linked list of sigs. */



#define SUMS   3


/* br_patscan() - scan passed data for a pattern
 *
 * Current viruys database is represented in the external array pattern_bits[],
 * which has a bit corrosponding to each of 32M possible hash values.
 * pattern_bits[] is "sparsely packed"  to accelate searching, (currently
 * indicating about 8000 virus patterns with 32,000,000 bits. Thus we should get
 * about 1 hit in pattern_bits[] for every 4000 bytes of packet data we scan; or
 * about one in every three full-size packets.
 * If a bit is set, that means the traditional hashing array pat_table[] contains at
 * least one entry which matches the hash value. pat_table is mopre densely
 * packed, containing only HASH_PATTERN_ENTRIES. 
 * 
 * pattern_bits[] contains HASH_PATTERN_BITS entries.
 * pat_table[] contains HASH_PATTERN_ENTRIES entries.
 *
 * INPUT:
 *    u_char * pktdata,    - packet data to search
 *    int length,          - length of pktdata
 *    struct endpoint * ep - structure with "context" data for TCP stream
 *
 * The data block passed to this routine assumed to be sequential units
 * in a data stream. The "context data" is needed to catch patterns
 * which begin near the end of one block (aka packet), and continue
 * at the beginning of the next packet.
 *
 * RETURNS: pointer to patern entry found in data, or (most often) NULL
 * if not found. If a pattern entry (pep) is returned, the byte pointed 
 * to by the prefix pointer passed will NULL be set to the 
 */


struct pattern_entry *
br_patscan(u_char * pktdata,
           int length,
           struct endpoint * ep)
{
   int   i, sumx, savex;   /* imsc indecies */
   u_long   hashx;         /* hash index */
   u_long   sum[SUMS];     /* sliding sums */
   u_long   save[PATTERN_LENGTH];  /* saved 3-byte values */
   u_long   acc;           /* accumulator */
   u_char * data;
   struct pattern_entry * pep;

   virus_scanned += length;    /* stats for testing & verification */
   if(virus_scanned < (u_long)length)  /* wrapped? */
      virus_bithits = 0;         /* then reset bithits too */

   /* Use prefix data first, then switch to packet data  */
   data = ep->pdata;

   /* seed the accumulators and sums from the prefix. If we run out of 
    * prefix data we switch to new packet data until we've crunched 
    * PATTERN_LENGTH bytes.
    */
   acc = sumx = savex = 0;
   for(i = 0; i < PATTERN_LENGTH; i++)
   {
      /* see if it's time to switch to packet data  */
      if(i == ep->pdlen)   /* end of prefix data? */
         data = pktdata;

      acc = ((acc << 8) & 0x00FFFFFF) + *data++;

      /* initialize or update sum[] */
      if(i < SUMS)
         sum[sumx++] = acc;      /* initialize */
      else
         sum[sumx++] += acc;     /* accumulate */

      if(sumx >= SUMS)
         sumx = 0;
      save[savex++] = acc;
      if(savex >= PATTERN_LENGTH)
         savex = 0;
   }
   /* if we have not already done so, switch to packet data  */
   if(i == ep->pdlen)
      data = pktdata;

   /* The main virus scanning loop. It should be really, really fast! */
   for(i = 0; i < (length - PATTERN_LENGTH); i++)
   {
      /* check for sum in huge hashed bitmask. Divide index "sum[sumx]" by 8 so
       * we can first check to see if any bit in the indexed byte is set. If
       * so, then we check to see if the indicated bit is set. Since we
       * don't expect to get many hits, it's usually faster this way.
       */

      hashx = (HASH_PATTERN_BITS - 1) & (sum[sumx] >> 3);
      if(pattern_bits[hashx])    /* got a byte hit? */
      {
         /* see if we got 1 one in 32M bit hit */
         if(pattern_bits[hashx] & (1 << (sum[sumx] & 0x07)))
         {
            virus_bithits++;

            /* Bit was set, so we should have an entry in the conventional hash table
             * of pattern values. First, get the pointer to the hash bucket.
             */
            pep = pat_table[sum[sumx] & (HASH_PATTERN_ENTRIES - 1)];
            if(!pep)
            {
               dtrap();    /* table maintainance error? */
               return NULL;
            }

            /* look for a full match in hash bucket list. The while loop below scans
             * the hash bucket's linked list, checking each entry for a full data match
             * with the data passed to this routine.
             */
            while(pep)
            {
               int   cmplen;     /* length for compare */

               /* Set the compare length. Ideally this will be the length of the
                * pattern in the hash bucket, however if we don't have that many bytes
                * left in the passed data buffer, then only compare what we have.
                */
               if((length - i) > pep->patlen)
                  cmplen = pep->patlen;
               else
                  cmplen = length - i;
               if(MEMCMP(pep->pattern, 
                  (data - (PATTERN_LENGTH + (SUMS-1))), cmplen) == 0)
               {
                  /* data matched hash table pattern, as far as we can check */
                  if(cmplen == (length - i))
                  {
                     /* We have a partial match, so return the length (in bytes)
                      * for the pep->pattern portion which matched. This will be
                      * returned as "prefix data" in the next call to this routine.
                      */
                     virus_partial++;
                     ep->pep = pep;
                     ep->partial = cmplen;
                  }
                  else
                  {
                     /* We got a full pattern match, indicate this by clearing the
                      * "partial" length
                      */
                     virus_found++;
                     ep->partial = 0;
                  }

                  return pep;    /* return pointer to pattern info */
               }
               pep = pep->next;
            }
         }
      }

      /* Get here if the currently summed data block did not match any entry an the
       * hash tables.
       */
      acc = ((acc << 8) & 0x00FFFFFF) + *data++;

      sum[sumx] -= save[savex];     /* subtract sum of data passing out of window */
      sum[sumx] += acc;             /* add accumulator to new sum */
      save[savex] = acc;            /* save accumulator for later subtract */

      if(++savex >= PATTERN_LENGTH)   /* bump index if wrapped */
         savex = 0;
      if(++sumx >= SUMS)    /* bump index, handle wrapped */
         sumx = 0;
   }
   return NULL;
}

char * 
getvname(struct pattern_entry * pep)
{
   char * cp;

   cp = pep->data;      /* poinmt to binary length byte */
   cp += ((*cp) + 1);   /* past binary data */

   return cp;
}

/* found_virus() - called from main code when a virus is found in a packet 
 * stream. 
 */

void
found_virus(struct conn_entry * conn, struct pattern_entry * pep)
{

   /* Kill first, log it later */
   if(br_cfg.cflags & BC_VIRUS_KILL)
   {
      /* kill connection */
      br_tcpreset(conn, 1);   /* reset local end */
      br_tcpreset(conn, 0);   /* reset remote end */
   }

   if(br_cfg.cflags & BC_VIRUS_ALERT)
   {
      virus_detected.parm = (void*)conn;
      alarm_general(&virus_detected, NULL, pep);
   }

   br_log(0, "Found virus \"%s\" in data stream; IP addresses:%u.%u.%u.%u<->",
      getvname(pep), PASS_IPADDR(conn->rem.ce_addr));

   br_log(0, "%u.%u.%u.%u ports:%d<->%d\n",
      PASS_IPADDR(conn->loc.ce_addr), conn->rem.ce_port, conn->loc.ce_port);

   return;
}


/* br_virussig() - Add a virus signature to the database.
 *
 * IN - readbuf - text of a signature DB record
 * IN - comm - community assigning signature
 * Param 3: TRUE if item should be committed to database,
 *
 * caller can assign Id
 *
 * Returns NULL if OK, else error message.
 */

//#define TRAPSTRING 1
#ifdef TRAPSTRING
char * trapstring = "_0023_0005_002";    /* hook debugger on this */
#endif   /* TRAPSTRING */

static char binerrbuf[256];

char * 
br_blkbinary(char * readbuf, struct comm_dbud * comm, int commit)
{
   char *   vname;      /* Virus name */
   char *   sigtext;    /* Text of HEX for signature */
   char *   cp;         /* scratch */
   u_char * sigp;       /* pointer to binary signature */
   int      siglen;     /* Length of BINARY dig in octets */
   int      hashx;
   int      len;        /* generic length */
   int      wildcard;   /* TRUE if sig contains wildcard chars */
   struct virussig * newsig;
   struct pattern_entry * pep;
   struct virussig * tmpsig;

#ifdef TRAPSTRING
   /* This lets the debugger trap the execution for a certain virus title */
   if(strncmp(trapstring, data, strlen(trapstring)) == 0)
   {
      dtrap();
   }
#endif   /* TRAPSTRING */

   vname = br_getfield(readbuf, "name");
   if(!vname)
      return ("missing name");
   sigtext = br_getfield(readbuf, "data");
   if(!sigtext)
   {
      /* Older spec used "signature"; allow that too. */
      sigtext = br_getfield(readbuf, "signature");
      if(!sigtext)
         return ("missing signature");
   }

   wildcard = 0;

   /* Check the signature for valid hex syntax */
   for(cp = sigtext; *cp > ' '; cp++)     /* whitespace ends signature */
   {
      /* Allow 0-9, a-z, and A-Z */
      if((*cp >= '0') && (*cp <= '9'))
         continue;
      if((*cp >= 'a') && (*cp <= 'f'))
         continue;
      if((*cp >= 'A') && (*cp <= 'F'))
         continue;
      /* Check also for wildcard chars */
      if((*cp == '?') || (*cp == '*') || 
         (*cp == ')') || (*cp == '(') ||
         (*cp == '}') || (*cp == '{') ||
         (*cp == '-') || (*cp == '|'))
      {
         wildcard = TRUE;  /* allow these, but flag them */
         continue;
      }
      return("bad char in signature");
   }
   /* Fall to here when all chars are checked */
   if(((cp - sigtext) & 1) && (!wildcard))
   {
      return ("uneven number of chars in signature");
   }

   siglen = strcrlen(sigtext);
   if(siglen <= 0)
   {
      return ("no signature data");
   }
   

   /* If we're just checking syntax, return OK now */
   if(!commit)
      return NULL;

   if(wildcard)
   {
      /* Just save the text sig, make smarter later */
      sigp = (u_char*)npalloc(siglen + 1);
      strncpy((char*)sigp, sigtext, siglen);
   }
   else
   {
      /* convert sig to binary and copy into buffer */
      siglen >>= 1;  /* length of binary sig image */
      sigp = (u_char*)npalloc(siglen);
      if(!sigp)
         return ("npalloc failed");
      hex2bin(sigtext, sigp, siglen);
   }

   newsig = (struct virussig *)npalloc(sizeof(struct virussig));
   if(!newsig)
      return "newsig alloc failed";

   newsig->sig = sigp;       /* attach signature binary to record */
   newsig->siglen = siglen;  /* set length of signature  binary */
   /* extract generic ctl info */
   read_ctlinfo(readbuf, comm, CTF_TYPESIG, &newsig->ctl);   
   newsig->ctl.comm = comm;

   /* Save signature name in record */
   len = strcrlen(vname);
   newsig->name = npalloc(len + 1);   /* extra byte for null term */
   if(!newsig->name)
   {
      npfree(newsig->sig);
      npfree(newsig);
      return "newsig alloc failed";
   }
   /* Copy name into record */
   strncpy(newsig->name, vname, len);

   /* look up the signature in the hash table, to see if it's a 
    * duplicate of one we alreay have.
    */
   hashx = PAT_TABHASH(sigp);    /* get hash value */

   /* If hash bucket has entry, (potential match) search the bucket */
   for(pep = pat_table[hashx]; pep; pep = pep->next )
   {
      if(siglen == pep->patlen)
      {
         if(MEMCMP(pep->pattern, sigp, pep->patlen) == 0)
         {
            /* Eact match, see if community is duplicate */
            tmpsig = (struct virussig *)pep->data;
            if(comm->cm_priority == tmpsig->ctl.comm->cm_priority)
            {
               /* Dups are allowed, return OK now */
               dprintf("%s has duplicate signature in community %s", 
                  tmpsig->name, comm->cm_name );
               npfree(sigp);
               return NULL;
            }
            /* Insert into control's duplicate list */
            if(comm->cm_priority > tmpsig->ctl.comm->cm_priority)
            {
               /* Replace head of list with new entry */
               newsig->ctl.dupctrl = &tmpsig->ctl;
               pep->data = newsig;
            }
            else
            {
               /* Add at head of duplicate list */
               newsig->ctl.dupctrl = &tmpsig->ctl;
               tmpsig->ctl.dupctrl = &newsig->ctl;
            }
         }
      }
   }


   if(wildcard)
   {
      /* Flag wildacrds, but don;t try to ass them to the pattern 
       * lookup system. The pattern lookup doesn't work with 
       * wildcards - we'll need to find an alternate way to 
       * look for these later.
       */
      newsig->flags |= VF_WILDCARDS;
   }
   else
   {
      /* Virus is not a duplicate, not a wildcard, and formatting is 
       * somewhat sane. Add it to the active pattern scanner database.
       */
      pep = new_patent(newsig->sig, siglen);  /* make new pattern entry */
      if(!pep)
      {
         npfree(newsig->sig);
         npfree(newsig->name);
         npfree(newsig);
         return "newsig alloc failed";
      }
      pep->pe_type = PET_VIRUS;
      pep->data = newsig;    /* data is virus record */
   }

   sigct++;    /* update global signature counter */
   br_dbdate = br_gettime();

   /* Add the new sig to linked list of sigs */
   if(viruslist == NULL)            /* NO list yet? */
   {
      viruslist = newsig;           /* it's only entry in list */
   }
   else        /* Add to existing list */
   {
      viruslast->next = newsig;    /* add new sig to end of list */
      newsig->next = NULL;         /* it's at end of list, no next */
   }
   viruslast = newsig;          /* new end of list */

   comm->cm_currec++;

   return NULL;
}


/* br_filesigs(FILE * fp) - helper routine for br_writesigs(), et.al. 
 *
 * This tranverses the whole virus sig list, writing their data to 
 * the already-open file passed.
 *
 * Returns number of sigs writen, or negative ENP_ error.
 */

long
br_filesigs(FILE * fp, struct comm_dbud * comm)
{
   int      i;
   char *   cp;
   u_char * bp;         /* binary data pointer */
   struct virussig * virusdb;
   long   sigs;         /* local sigcount */
   char    readbuf[1024];

#define RBLEN sizeof(readbuf)

   sigs = 0;
   for(virusdb = viruslist; virusdb; virusdb = virusdb->next)
   {
      /* skip if this sig is not in passed community */
      if(virusdb->ctl.comm != comm)
         continue;

      sigs++;
      sprintf(readbuf, "type: Binary\r\nname: %s\r\nData: ", 
         virusdb->name);

      cp = readbuf + strlen(readbuf);     /* find spot in buffer */

      if(virusdb->flags & VF_WILDCARDS)
      {
         /* Wildcard sigs are stored as text */
         memcpy(cp, virusdb->sig, virusdb->siglen);
         cp += virusdb->siglen;
      }
      else
      {
         /* check to make sure sig won't overflow readbuf */
         if((&readbuf[RBLEN] - cp) <= (virusdb->siglen << 1))
         {
            dprintf("Virus signature for \"%s\" too long for file buffer\n",
               virusdb->name);
            return ENP_RESOURCE;
         }

         /* format the sig into hexadecimal text */
         bp = virusdb->sig;
         for(i = 0; i < virusdb->siglen; i++)
         {
            *cp++ = hexdigit( (u_char)((*bp) >> 4) );
            *cp++ = hexdigit( (u_char)((*bp++) & 0x0F) );
         }
      }
      *cp++ = '\r';  /* Append CRLF to text field */
      *cp++ = '\n';
      *cp = 0;    /* null terminate sig string */

      /* add number & comment */
      append_ctl(&virusdb->ctl, readbuf, RBLEN );
      comm->cm_currec++;      /* Bump community current rec count */

      fprintf(fp, "%s\r\n", readbuf);
   }
   return sigs;
}


/* br_writesigs() - write the virus signature database, in "Clam"
 * text format, to the file passed. The file may be the 
 * pre-existing viruses.db, or a new file.
 *
 * Returns 0 if OK or an ENP_ error code.
 */

int
br_writesigs(char * filename)
{
   FILE *   fp;
   long     err = 0;
   struct comm_dbud * comm;

   /* Open empty file for writing. If file exists, (i.e. "viruses.db") 
    * its contents are destroyed.
    */
   fp = fopen(filename, "w");
   if(!fp)
   {
      dprintf("Unable to open virus sig file %s for writing.\n",
         filename);
      return ENP_FILEIO;
   }

   fprintf(fp, "# %s\n# Antivirus signatures file.\n\n[virus]\n\n", filename);

   /* Write all sigs in all space space for now, maybe let user 
    * pick a space later? 
    */
   for(comm = br_communities; comm; comm = comm->next)
   {
      err = br_filesigs(fp, comm);
      if(err < 0)
         break;
   }

   if(err >= 0)
      dprintf("Wrote virus %lu sigs to AV file %s.\n", err, filename);
   else
      dprintf("Error (%d) writing virus sigs to AV file %s.\n", 
         err, filename);

   fclose(fp);

   return (int)err;
}

/*
Copyright (C) 2003-2008 by Praemio Inc. All rights reserved.

Author: John Bartas - jbartas@praemio.com

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/

#endif /* VIRUS_CHECK - whole antivirus support file can be removed */
