/* brdb.h
 *
 * Main definitions file for Buster specific objects.
 *
 *
 * Copyright 2003-2008 by Praemio Inc. All rights reserved.
 * Author: John Bartas - jbartas@praemio.com
 * This software is released under the terms of the LGPL.
 * See LGPL Notice at end of this file; Full license in ../license.txt
 */

#ifndef _BRDB_H_     /* Prevent double includes */
#define _BRDB_H_     1

#include "btree.h"      /* required for port & IP structures */


extern char promptbuf[24];    /* menu prompt text */


/* some system limits */
#define  MAXNAMELEN  32    /* User names, passwords, etc. */
#define  MAXDATALEN  128
#define  MAXFILENAME 64    /* max length of a Agent file name */
#define  MAXPATHNAME 128   /* max length of path & name */
#define  MAXURL    350     /* Max lenght of buster server URL */

/* Path to root for Busters file in the native FS, if any. This 
 * is Set by port code at startup  On Appliances w only VFS may 
 * be just slash '/' 
 */
extern   char     homedir[MAXPATHNAME];   

/* Definitions for a one-shot low use message buffer */
#define  MSGSIZE  256
extern   char brmsg[MSGSIZE];  /* scratch message buffer */


extern   char  systemname[MAXNAMELEN];    /* name of system */
extern   char  br_password[MAXNAMELEN];   /* network password */

extern   char  br_urlbuf[1500];           /* GUI url handling */

extern   char * nextarg(char * argp);

/* Get digest for a passed password */
char *   br_pwdigest(char * password, u_long mask);

/* Convert digest to printable ascii */
void     br_printdigest(char * text, u_char * digest, int digest_len);


/* binary data trees */
extern   struct btnode *   ip_tree;       /* controlled IP addrs */
extern   struct btnode *   dn_tree;       /* controlled domains */
extern   struct btnode *   image_tree;    /* images for local MACs */
extern   struct btnode *   tcpp_tree;     /* controlled TCP ports */
extern   struct btnode *   udpp_tree;     /* controlled UDP ports */
extern   struct btnode *   path_tree;

extern    void * br_tree_mutex;  /* mutex for accessing the above trees */

/* Mask of enabled features */
extern   u_long   featuremask;

/* Version fields for Buster binaries. See notes in brconfig.c  */
extern char *   br_version;
extern u_long   br_versnum;

/* Application name keeps changing (fucking marketing). Try to automate this */
extern char * br_appname;


struct tcphdr;       /* predecl */
struct endpoint;     /* predecl */
struct conn_entry;   /* predecl */
struct comm_dbud;    /* predecl */


/* BR_HOSTED is non-zero if port target is on an OS (e.g. Windows, Linux),
 * not an embedded system.
 */
#define  BR_HOSTED      1

/* Buster mode, primarily guided by messaging */
enum br_modes 
{
   OFFLINE,       /* Power down, prog unloaded, etc. */
   PASSIVE,       /* Only monitors the segment */
   STANDBY,       /* Monitors the segment, may go active */
   HOSTONLY,      /* Buster only acts on one host */
   ASSERTING,     /* Wants to be active (e.g. just powered up.) */
   ACTIVE         /* Buster acts on the whole net segment */
};

extern   enum br_modes br_mode;


/* bit flag definitions for system evflags. The first
 * two are mutually exclusive.
 */
enum br_sysstates
{
   COLDSTART,     /* started from cold state */
   WAKEUP,        /* woke after hibernation */
   HIBERNATING,   /* entering/leaving hibernation */
   CHANGEIFACE,   /* We changed view iface */
};

extern enum br_sysstates br_sysstate;

/* mode ename text, indexed by br_mode */
extern char * br_modenames[6];

/* routine called to kick-off active mode process */
int   br_setmode(enum br_modes);

void  br_log(int flags, char * msg, ...);


/* portability routines to get system date and time for agent.log */
char *   br_getlogtime(void);
char *   br_getlogdate(void);
void     br_user_alert(char * msg);    /* send a local operational alert */
void     br_powerresume(void);         /* back from hibernation */
void     br_lanchanged(void);          /* LAN changed during hibernation */
int      br_ifsetup(void);             /* Fill in system net iface data */
void     br_startcticks(void);         /* start cticks */

/* br_nbiosocket( long socket )
 * 
 * Portability routine to set a socket into non-blocking mode. BSD 
 * sockets botched this by using "ioclt()", so it's different on both 
 * just about every system.
 *
 * Returns 0 if OK, else negative error ENP_ code. 
 */

int        br_nbiosocket( long socket );

#ifndef INVALID_SOCKET
#define INVALID_SOCKET  -1
#endif

#ifndef SOCKTYPE
#define SOCKTYPE long
#endif

extern   u_long   br_downtime;         /* time/date Buster went to hibernate */
extern   int      br_powerdown_mode;   /* mode before suspend message */
extern   u_long   br_sleeptime;        /* time of last hibernation in seconds */

/* portability routine to get UTF timestamp value (GMT) */
extern   u_long   br_gettime(void);

/* Portability routine to convert timestamp to file text */
extern   int      br_writedatetime(u_long intime, char * out, int outlen);

/* Formatted MAC address print/read */
extern   char *   print_mac(u_char * addr);

/* Convert ascii hex chars to 32 bit unsigned */
extern u_long atoh(char * hexchars);

char  hexdigit(u_char byte);

/* Check an IP address format, return if OK; return error if not */
extern    char *    br_checkip( ip_addr * output, char * iptext);


/* bad/illegal timestamp value */
#define BR_BADSTAMP 0xFFFFFFFF

/* Max length of a date/time string (from br_writedatatime() */
#define DB_MAXDTL 16
#define DATEBUFLEN  64


/* Agent hard error codes are negative */
#define ENP_NOMEM       -1      /* no heap memory */
#define ENP_NOBUFFER    -2      /* no network buffer */
#define ENP_RESOURCE    -4      /* unable to alloc other resource */
#define ENP_NO_ROUTE    -5      /* no network route for operation */
#define ENP_PARAM       -6      /* bad parameter to subroutine */
#define ENP_LOGIC       -3      /* nonsensical situation - logic error */
#define ENP_FILEIO      -7      /* Problem in file system */
#define ENP_SENDERR     -8      /* network send problem */
#define ENP_TIMEOUT     -9      /* got sick of waiting */

/* indeterminate return code for net routines */
#define ENP_SEND_PENDING 1      /* Send pending in lower layers */ 
#define ENP_NOT_MINE     2


/* NV file reader helpers */
void     br_nullterm(char * arg);
char *   br_getcomment(char * readbuf);


/* Buster configuration. Includes times for aging out closed 
 * TCP or idle and UDP connections 
 */

struct br_config
{
   u_long   keep_tcpconn;        /* default 15 seconds (in cticks) */
   u_long   keep_udpconn;        /* Time to keep idle UDP connections  */
   u_long   keep_icmpconn;       /* Time to keep idle ICMP session */
   u_long   max_connections;     /* max. connections, total */
   u_long   max_connperhost;     /* max. connections per host */
   long     alarm_connperhost;   /* connections per host to raise alarm */
   u_long   keep_iphost;         /* cticks to keep idle remote host */
   int      br_alarminterval;    /* interval between first two alarms */
   int      cflags;              /* boolean config values */
   int      syns_per_minute;     /* syn flood threshold */
   int      screen_refresh;      /* default refresh interval */
   u_long   logevents;           /* Bitmask, see  LE_ bits below */
   int      logsize;             /* Max. size of log in Kbytes */

   /* Sort and Filter parameters, set from preferences page */
   int      hide_parm;           /* bitmask of HP_ values */
   u_char   sortparm;            /* sort parameter, i = ip, p = port, t = traffic, etc */
   int      filterport;          /* port to sort by, 0 if none */
   int      filterparm;          /* map & list filter mask */
   int      udp_idle;            /* seconds to wait before labeling UDP and ICMP as idle */
   u_long   mactmo;              /* MAC timeout in minutes, 0 = no timeout */

   /* interval between updates from clear2net, in minutes */
   long     update_interval;
};

extern struct br_config br_cfg;

/* some recommended defaults:
   int      hide_parm = HP_AGENT | HP_LOCAL;
   char     sortparm = 'i'; 
   unshort  filterport = 0; 
   int      filterparm = FP_ALL; 
   int      udp_idle = 600;
   long     update_interval = 240;   
*/

/* Read/Write config to/from a text file */
extern int  br_saveparms( void );
extern int  br_readparms( void );
extern int  br_initparms( void );


/* bits for br_cfg.cflags */
/* The first six drive auto-learn */
#define BC_USER_IP      0x00000001      /* User has set IP address */
#define BC_USER_MASK    0x00000002      /* User has set IP mask info */
#define BC_USER_ROUTER  0x00000004      /* User has set router */
#define BC_USER_ISP     0x00000008      /* User has set ISP address */
#define BC_USER_DNS1    0x00000010      /* User has set DNS1 */
#define BC_USER_DNS2    0x00000020      /* User has set DNS2 */

#define BC_VIRUS_ALERT  0x00000040      /* Alert if TCP packets have viruses */
#define BC_VIRUS_KILL   0x00000080      /* Kill connection if TCP packets have viruses */
#define BC_DESPAM       0x00000100      /* Default to despamming new pop clients */
#define BC_CONTENT      0x00000200      /* content control enabled */
#define BC_HOSTONLY     0x00000400      /* Blocking only perfomed for host PC */
#define BC_DELIDLE      0x00000800      /* Delete Idle MAC hosts */
#define BC_NOEXPERT     0x00001000      /* no expert console at startup */

/* bits for br_cfg.hideparm */
#define  HP_BUSTER      0x01        /* Hide Buster activity */
#define  HP_LOCAL       0x02        /* Hide LAN activity */

/* bits for br_cfg.logevents */
#define  LE_TCPTMO         0x01     /* Log TCP connection deletions */
#define  LE_UDPTMO         0x02     /* Log UDP session deletions */
#define  LE_ICMPTMO        0x04     /* Log ICMP session deletions */
#define  LE_IPHTMO         0x08     /* Log IP host deletions */
#define  LE_LOGDELMAC      0x10     /* Log aged-out MAC hosts */

/* NULL MAC address, for setting MAC addresses to all zeros */
extern u_char nullmac[8];


/* The fast pattern lookup used to find URLs, viruses, etc.,
 * is based on two large tables. One is a standard hash table
 * of patterns. Each pattern must be exactly PATTERN_LENGTH
 * bytes long. The other table is a huge bitmask, which must
 * be sparsely populated. Each bit represents a possible hash
 * value, with 1 (set) bits indicating that the standard table has
 * an entry matching the hash value. The lookups in the bitmask
 * table are fast because the table is so sparsly populated that
 * actual pattern comparisons can usually be avoided.
 */

/* size of bitmask of patterns in hash table */
#define  HASH_PATTERN_BITS     0x400000   /* 4 Mbytes, 32 mbits */
#define  HASH_PATTERN_ENTRIES  0x20000    /* actual pattern records */

/* Length of hash data for fast pattern searchs. Note: the algorithm
 * cannot search for patterns smaller than this.
 */
#define  PATTERN_LENGTH       9


/* The following table sizes must be powers of 2 */
#define  HASH_CONN_ENTRIES      1024   /* Active Connection table */
#define  HASH_HOSTS               64   /* MAC hosts on local net */
#define  HASH_IPHOSTS            256   /* Active IP hosts list */
#define  HASH_PORT_ENTRIES       128   /* port blocking table */
#define  HASH_IP_ENTRIES        2048   /* IP hosts blocking list */
#define  HASH_DN_ENTRIES     0x01000   /* Controlled Domain names */



extern unshort ip_id;

/* Timing values based in cticks */
#define  SECOND      TPS
#define  MINUTE      (SECOND * 60)
#define  HOUR        (MINUTE * 60)
#define  DAY         (HOUR * 24)

/* "Infinite" value for 32 bit TTL fields */
#define  TTL_FOREVER    0xFFFFFFFF


/* Global flag; set to nonzero whenever Buster is in the receved
 * packet logic. This is used the prevent database deletions while
 * Buster may be referencing data items.
 */
extern   int   br_inreceiver;

/* Sometimes we need a way to signal the DB code no to delete anything
 * temorarily. For example when the wbe server is displaying a conneciton
 * list, we dont; want the connections dissappearing. Here's the flag
 * for this:
 */
extern   int   br_dbnodelete;


/* alarm instance, for embedding in host or conn struct. These cover
 * situations like a host which generates syn floods - several hosts
 * could do this in a short period; this would be serveral instances
 * of one alarm_type. This allows us to generate one alarm for a host
 * that generates several syn floods in a short (< 1 sec) period,
 * but later generate another alarm if the host is still doing
 * it after (for example) 5 minutes.
 */

struct alarm_inst
{
   int      alarmcount;       /* alarms sent since clearing */
   u_long   last_ctick;       /* time of last alarm */
};


/* state of current Buster configuration (one per iface) */
struct br_state
{
   ip_addr     br_lease;      /* duration of IP lease; 0 if unknown */
   ip_addr     br_leasestart; /* start time of lease; 0 if unknown */
   struct host_entry *  br_router;  /* Mac level router info  */
   u_long      machosts;      /* number of MAC hosts on LAN */
   u_long      total_conns;   /* current connection count */
   char        current_lan[MAXNAMELEN];   /* text name of current lan */
   u_long      evflags;       /* network event flags */
   u_long      ip_tick;       /* ctick of last state change */
   int         ifindex;       /* Index into nets[] for this iface */
   int         rx_disabled;   /* receiver temporarily disabled */
   int         br_rtconf;     /* confidence (0-255) in router guess */
   struct alarm_inst noip;    /* No IP address on iface */
   struct alarm_inst dupip;   /* Duplicate IP addresses on net */
};


extern struct br_state brstates[MAXNETS];

extern   u_long   br_suspended;     /* timer for br_state->rx_disabled */

int   br_suspendresume(int tick_count);   /* set suspend state */
void  br_portsuspendresume(int disable);  /* portability */


#define MAX_CONFIDENCE  0xFF  /* Certain of the data */

/* date & time routines and variables */
extern   u_long   br_gettime(void); /* get 32 bit UTF date/time */
extern   u_long   br_subtracttime(u_long newtime, u_long oldtime);
extern   u_long   br_addtime(u_long minutes, u_long agenttime);
extern   u_long   br_lastresume;    /* ctick when br_powerresume() was last called */

/* Portability routines - called from Buster code; must be
 * implemented in port code.
 */


/* Called after hibernation to recheck interface data*/
extern   int   br_checknet(int iface); /* re-check an iface*/

/* Called to re-read iface IP info from system */
extern   int   br_setipinfo(int iface);

/* Select the View and inet interfaces based on traffic */
extern   void     br_autoselect(void);

/* Let user know LAN changed (moving laptop?) and clear DB */
extern   void     br_lanchanged(void);


/* ifconfig related helper routines. */
extern   int   islocal_ip(int iface, ip_addr ipaddr);
extern   int   br_isbcast(int iface, ip_addr ipaddr);
extern   void  br_setrouter(int iface, struct host_entry * machost, int confidence);
extern   void  br_clearrouter(int iface);
extern   void  br_setiface(int * setif, int newif);
extern   void  br_cleardbs(void);

extern   int   br_ifview;     /* iface currently selected for viewing */
extern   int   br_ifinet;     /* iface for reaching the internet */
extern   int   br_autoifview; /* bool for autoselect br_ifview */
extern   int   br_autoifinet; /* bool for autoselect br_ifinet */


/* filterparm mask bits */
#define FP_TCP    0x01        /* Show TCP when filtering */
#define FP_UDP    0x02        /* Show UDP when filtering */
#define FP_ICMP   0x04        /* Show ICMP when filtering */
#define FP_ALL    (FP_TCP|FP_UDP|FP_ICMP)


/* bit flag definitions for network evflags. The first
 * three are mutually exclusive 
 */
#define  EV_IPDHCP      0x0001      /* Using DHCP for IP addr */
#define  EV_IPSTATIC    0x0002      /* Static IP addr (not DHCP) */
#define  EV_IPAUTO      0x0004      /* Use auto-IP for IP address */
#define  EV_IPADDRSET   0x0008      /* IP addr set & valid */
#define  EV_ROUTERSET   0x0010      /* confirmed network router */
#define  EV_NETNAMED    0x0020      /* We have a known, named network*/
#define  EV_LANSTALE    0x0040      /* Lan info needs writing */
#define  EV_LANUKMSG    0x0080      /* Displayed "Lan Unknown" message */


/* Static data lists */

/********** Control descriptor *************/

/* this structue is enmbedded in each of the objects used to 
 * manage things that Buster seeks to control. It allow us to
 * use one orthogonal set of routines for all control functions.
 */

/* First, define the array of motives for blocking sites/ports/IPs/paths: */

struct bk_motive 
{
   int   code;    /* One of the OF_* values */
   char * desc;   /* VERY short (2-7 chars) description */
};

#define OF_OTHER        1 
#define OF_ANTIVIRUS    2
#define OF_SPAM         3
#define OF_ADVERTS      4
#define OF_CONTENT      5
#define OF_SPYWARE      6
#define OF_FRAUD        7
#define OF_COOKIES      8
#define OF_POPUPS       9
#define OF_HACKING      10


extern   struct   bk_motive motives[12];

/* The actual control structure. These are embedded in larger structures
 * which govern how we restrict IP addresses, ports, domain names, URLS,
 * etc. Notes:
 *
 * - If the timeout field is non-zero then access is allowed while it
 * counts down.
 * - Action mask flag may also allow access, so we can enable a host without
 * removing it frpom the database.
 *
 * Each control has a back pointer to a community which installed the 
 * controlled item. Duplicate items may be submitted by different 
 * communities, but only one may be active at a time. Which one this is
 * is determined by the user's community preferences. Duplicates from lower
 * priority communities are NOT kept in the master lists, but are saved
 * in the dupctrl linked list. If the item is explicitly deleted by the
 * user hten the whole dupcrl list is deleted. If the senior community
 * is deleted then the next highest priority community's dup entry (if
 * any) is moved into master list. 
 */

struct ctdesc
{
   struct bk_motive * offense;      /* Reason we find this offensive */
   u_long   action_mask;            /* CTF_ bits for control actions */
   u_long   timeout;                /* number of ticks to allow tmp access */
   u_long   actions;                /* number of times this caused action */
   u_long   lasttick;               /* ctick of last action */
   /* "desc" and "tags" are gone. Communities were mis-using 
    * these, eg: clamAV had "Clam Virus" in every single "desc".
   char *   desc;
   char *   tags;
    */

   void *   replacement;            /* replacement content, if any */
   struct comm_dbud * comm;         /* Database community */
   struct ctdesc * dupctrl;         /* pointer to duplicate list, if any */
};

#define CTF_TYPEMASK    0x0000FF   /* mask to type of controlled object */
#define CTF_TYPEIP      0x000001   /* type values for above mask */
#define CTF_TYPEPORT    0x000002   /* TCP or UDP port number */
#define CTF_TYPENAME    0x000003   /* DNS name */
#define CTF_TYPEPATH    0x000004   /* path/name potion of URL */
#define CTF_TYPEWORD    0x000005   /* Web content word */
#define CTF_TYPEMAC     0x000006   /* MAC host image record */
#define CTF_TYPESIG     0x000007   /* Binary signature (virus) */


#define CTF_BLOCK       0x000100   /* reset both direction */
#define CTF_FILELOC     0x000200   /* insert file to local */
#define CTF_FILEREM     0x000400   /* insert file to remote */
#define CTF_ALERT       0x000800   /* generate an alert */
#define CTF_WHITELIST   0x001000   /* allow access when this bit is set */
#define CTF_SEQUENCE    0x002000   /* part of a cluster (e.g. phrase) */
#define CTF_UPDATE      0x004000   /* Contains updated info for comm server */
#define CTF_SUBMIT      0x008000   /* New submission for comm server */

/* Macro: TRUE if controled item is whitelisted. True if 
 * access bit set AND timer is NOT set 
 */
#define WHITELISTED(ctl) \
   ((ctl->action_mask & CTF_WHITELIST) && (ctl->timeout == 0))

/* Set & get ctl access */
extern   void  ctl_setaccess(struct ctdesc * ctl, long access, void * object);
extern   int   ctl_getaccess(struct ctdesc * ctl);


/* helper function to append ctl info to a text item */
void  append_ctl(struct ctdesc * ctl, char * buffer, int bufsize);

/* helper function to initialize ctl info from a text item */
void
read_ctlinfo(char *  readbuf,          /* IN - text buffer which may have info */
            struct comm_dbud * comm,   /* IN - community */
            int      ctltype,
            struct ctdesc * ctl);      /* OUT - filled in ctl */

/* motive lookup routines */
struct bk_motive *   mot_bynum(int number);
struct bk_motive *   mot_byname(char *);


/********** Controlled port list *************/

/* Controlled port list. This is designed for generic client
 * port ranges. if the entry only covers a single port, then
 * the single port value is placed in both cp_hiport and
 * cp_loport. The single port btree contains both TCP and
 * UDP ports, and a single entry can apply to UDP, TCP or both.
 */

struct control_port
{
   struct btnode * btnp;         /* binary tree link info */
   unshort  cp_port;             /* lowest port value */
   int      cp_flags;            /* bitmask of CPF_ bits */
   struct ctdesc ctl;            /* control struct */
   char imagefile[MAXFILENAME];  /* associated image, "" if none */
};

#define  CPF_TCP     0x01        /* port applies to TCP packets */
#define  CPF_UDP     0x02        /* port applies to UDP packets */
#define  CPF_LOCAL   0x04        /* applies to local port */
#define  CPF_REMOTE  0x08        /* applies to remote port */
#define  CPF_RSONLY  0x10        /* applies to remote sender (RS) only */

#ifndef CP_ALLOC
#define  CP_ALLOC(size) (struct control_port *)npalloc(size)
#define  CP_FREE(ptr)   npfree(ptr)
#endif

/* new_cp() - make new cp entry. type is CPF_TCP or CFP_UDP */
struct control_port *   new_cp(unshort, int type, struct comm_dbud * comm);
void  del_cp(struct control_port * cpt, int type);

/* binary tree compare routine for port range */
int   bt_porttest(struct btnode * n, u_long key);

struct control_port * cpt_lookup(unshort port, int transport);


/********** Controlled IP address list *************/

struct control_ip
{
   struct btnode * btnp;      /* binary tree link info */
   u_long   ci_ipaddr;        /* IPv4 address to block */
   unshort  ci_port;          /* optional port */
   int      ci_flags;         /* bitmask of CIF_ bits */
   u_long   ci_timeout;       /* ctick when it expires */
   struct ctdesc ctl;         /* control struct */
   struct control_dn * ci_name;  /* if applicable, else NULL */
   /* chain of control_ips associated with a ci_name */
   struct control_ip * ci_dnlist;
};

/* bits for ci_flags mask */
#define  CIF_IPDEST  0x01       /* address applies to dest */
#define  CIF_IPSRC   0x02       /* address applies to source */
#define  CIF_LOCAL   0x04       /* port applies to dest */
#define  CIF_REMOTE  0x08       /* port applies to source */
#define  CIF_DNSREP  0x10       /* Entry came from DNS reply to blocked URL */
#define  CIF_USER    0x20       /* Entry from GUI form */
#define  CIF_COMM    0x40       /* Entry is from a community */

#ifndef IPE_ALLOC
#define  IPE_ALLOC(size) (struct control_ip *)npalloc(size)
#define  IPE_FREE(ptr)   npfree(ptr)
#endif

struct control_ip * new_ipe(ip_addr ipaddr, struct ctdesc * ctl,
      int typeflag, struct comm_dbud * comm);
void  del_ipe(struct control_ip * ci, int verbose);
struct control_ip * ipe_lookup(ip_addr ipaddr);

/* binary tree compare routine for IP addresses */
int   bt_iptest(struct btnode * n, u_long ipaddr);

void * control_ip_mutex;


/***************** Scan pattern database  *****************/

struct pattern_entry
{
   struct pattern_entry * next;     /* Hash list link */
   u_char * pattern;       /* pattern used for hashing/matching */
   int      patlen;        /* length of pattern */
   void *   data;          /* Virus sig, etc. - depends on type */
   int      pe_type;       /* Virus, or whatever */
};

/* bitmasks to indicate which patterns are present */
extern char pattern_bits[HASH_PATTERN_BITS];

/* the actual hash table */
extern struct pattern_entry * pat_table[HASH_PATTERN_ENTRIES];

/* pe_type values */
#define PET_VIRUS    1

/* Pattern hash routine. This takes a pointer to PATTERN_LENGTH
 * bytes as input, and returns the "sliding hash window" value
 * of PATTERN_LENGTH bytes of the pattern.
 */
#define HASH_PATTERN(pat) \
  ((u_long)(*pat << 16) + \
   (u_long)(*(pat+1) << 8) + \
   (u_long)(*(pat+2) ) + \
   (u_long)(*(pat+3) << 16) + \
   (u_long)(*(pat+4) << 8) + \
   (u_long)(*(pat+5) ) + \
   (u_long)(*(pat+6) << 16)+ \
   (u_long)(*(pat+7) << 8) + \
   (u_long)(*(pat+8) ) )


/* pat_table entry hash macro. This takes the 24 bit
 * sliding window hash value and pares it down to address
 * the pattern hash table.
 */
#define PAT_TABHASH(pat)  (HASH_PATTERN(pat) & (HASH_PATTERN_ENTRIES - 1))

#ifndef PEP_ALLOC
#define  PEP_ALLOC(size) (struct pattern_entry *)npalloc(size)
#define  PEP_FREE(ptr)   npfree(ptr)
#endif

/* BRAV statistics */
extern   u_long   virus_scaned;       /* total bytes scanned */
extern   u_long   virus_bithits;      /* number of bitmask table matches */
extern   u_long   virus_partial;      /* number of partial matches */
extern   u_long   virus_found;        /* number of full matches */

/* contructor and destructor */
struct pattern_entry * new_patent(u_char * pat, int patlen);
void  del_patent(struct  pattern_entry * pep);

/* virus & pattern scan utility routines */
struct pattern_entry * br_patscan(u_char * data, int len, struct endpoint *);
void  found_virus(struct conn_entry * conn, struct pattern_entry * pep);
char * getvname(struct pattern_entry * pep);    /* get virus name */

int   br_writesigs(char * filename);   /* write vsig file */

/************** list of virus signatures **************/

struct virussig
{
   struct virussig * next;       /* list link */
   struct ctdesc ctl;            /* control struct */
   char *   name;                /* Virus name */
   u_char * sig;                 /* binary signature */
   int      siglen;              /* length of signature */
   int      flags;               /* mask of VF_ bits below */
};

#define  VF_WILDCARDS   0x01     /* virussig has wildcards */

extern   struct virussig * viruslist;
extern   u_long   sigct;         /* total sigs put in table */



/********** Controlled domain name list *************/

struct control_dn
{
   struct btnode * btnp;         /* binary tree link info */
   char *   cu_name;             /* text for the DN, from heap */
   u_long   cu_time;             /* ctick when it expires */
   struct control_ip * cu_cip;   /* IP addresses, NULL if unknown */
   struct ctdesc ctl;            /* control struct */
};

#ifndef DN_ALLOC
#define  DN_ALLOC(size)   (struct control_dn *)npalloc(size)
#define  DN_FREE(ptr)     npfree(ptr)
#endif

struct control_dn * new_dn(char * name, struct comm_dbud * comm);
void del_dname(struct control_dn *);
struct control_dn * dn_lookup(char * name);  /* also gets domain matches */

/* Insert cip structure into a blocked DNS entry list. */
int   br_insertcip( struct control_dn * dnp, struct control_ip * new_cip);

/* DN binary tree test routine */
int   bt_dntest(struct btnode * n, u_long key);

extern void * br_dns_mutex;


/**************** Controlled (blocked) URL path list  *****************/

/* blocked path structure. This is maintained in a binary tree */

struct b_path
{
   struct   btnode * btnp;    /* binary tree link info */
   int      length;           /* length of pathname */
   int      flags;            /* BP_ flags bitmask */
   struct ctdesc ctl;         /* control struct */
   char *   pathname;         /* text of path name */
};

#define BP_LEADWILD    0x01     /* lead is wildcard */
#define BP_TAILWILD    0x02     /* tail is wildcard */


int      bt_pathtest(struct btnode * n, u_long key);

struct b_path *   new_path(char * pathname, struct comm_dbud * comm);
void     del_path(struct b_path *);
struct b_path * path_lookup(char * pathname);


/************* Support for multiple users & passwords *************/
/************* Merge with Webio later? *************/

#ifndef MAXUSERLENGTH
#define MAXUSERLENGTH   32
#endif

#define MD5PASSLENGTH   32  /* Chars for storing MD5-ed password */

struct br_user
{
    char    username[MAXUSERLENGTH+2];
    char    password[MD5PASSLENGTH+2];
    u_long  usermask;
};

#define BR_MAXUSERS 16
extern struct br_user  users[BR_MAXUSERS];

extern  int add_user(char * username, char * password, long permitmask);

/* Check user access to the passed FILE or VFILE pointer */
extern  int user_access(char * username, char * password, void * fs); 


/* User permission bits. "Admin" has all these set */
#define UF_VIEWMAP     0x00000001    /* Allow viewing map pages */
#define UF_VIEWSELF    0x00000002    /* Allow viewing host map for self */
#define UF_VIEWBLOCK   0x00000004    /* Allow editing blocked URLs, IPs, etc. */
#define UF_VIEWHOST    0x00000008    /* Allow viewing host maps */
#define UF_VIEWIP      0x00000010    /* Allow viewing IP settings */
#define UF_VIEWPREF    0x00000020    /* Allow viewing preference settings */
#define UF_EDITPREF    0x00000040    /* Allow editing preference settings */
#define UF_EDITIP      0x00000080    /* Allow editing IP settings */
#define UF_EDITBLOCK   0x00000100    /* Allow editing blocked URLs, IPs, etc. */
#define UF_CONTENT     0x00000200    /* Allow editing content vocabularies */
#define UF_SPAMEDIT    0x00000400    /* Allow editing own spam settings */
#define UF_EMAILPW     0x00000800    /* Allow view email passwords */
#define UF_ALERTS      0x00001000    /* Allow editing alert settings */
#define UF_KILLCONN    0x00002000    /* Allow killing conmections */
#define UF_TELNET      0x00004000    /* Allowed access via telnet */
#define UF_EDITUSER    0x00008000    /* Allow editing other users */

#define NUM_UFBITS      16


/**************** binary tree of MAC-based images *****************/

struct image
{
   struct   btnode * btnp;    /* binary tree link info */
   u_char   macbase[8];       /* base MAC address of block */
   u_long   range;            /* number of MACs in block */
   char     imagefile[MAXFILENAME];    /* name of image file */

   /* Although we don't really use these to "control" images,
    * we use a ctl struct to give us orthogonal ID and comment
    * handling. Type is CTF_TYPEMAC.
    */
   struct   ctdesc   ctl;
};

/* binary tree compare routine for image structs */
int   bt_mactest(struct btnode * n, u_long key);

/* routine to select image for a host */
extern   char *   br_image(struct host_entry * host);

/* Pretty-print a MAC address for display. */
extern   char *   br_prtmac(u_char * macaddr);

#ifndef IMG_ALLOC
#define IMG_ALLOC(s)    (struct image *)npalloc(s)
#define IMG_FREE(p)     npfree(p);
#endif

extern  void    panic(char * msg);

/* Support for user loadable images */
extern   char * br_imagedir;     /* directory for user image files */
extern   char * br_getdir(char * dir); /* get listing of dir */

extern   void * br_opencustom(char * name, char * mode);


/****************** IP Connection (&session) list  *****************/ 


struct endpoint   /* A connection endpoint */
{
   ip_addr     ce_addr;       /* IP addr, in net endian  */
   unshort     ce_port;       /* TCP/UDP local (to this end) port */
   struct      host_entry * ce_mac;          /* ptr to local MAC host info */
   struct      ip_hostentry * ce_iphost;     /* ptr to local IP info */
   u_long      ce_outpkts;    /* packet count this->other */
   u_long      ce_outbytes;   /* byte count this->other */
   u_long      ce_seq;        /* next expected seq (if TCP) */

   /* trail of last packet data for virus scanning */
   u_char      pdata[PATTERN_LENGTH];  /* pointer into ce_pkt */
   int         pdlen;                  /* length of valid data in pdata */
   int         partial;          /* nonzero if end of pkt was partial virus match */
   struct pattern_entry * pep;   /* virus if "ep->partial" is set */

   u_char      app[MAXNAMELEN];  /* Application data (eg Mozilla, Apache) */
   int         sp_resets;        /* count of reset/spoof packets sent */
};

struct conn_entry
{
   struct conn_entry * next;   /* hash list link */ 

   struct endpoint   loc;     /* "local" (or first) endpoint */
   struct endpoint   rem;     /* remote, or second, endpoint */

   int         ce_type;       /* H_TCP, H_UDP, H_ICMP */
   u_long      ce_flags;
   int         ce_state;      /* TCP state */
   int         ce_iface;      /* index to net interface */
   u_long      ce_start;      /* cticks connection was detected */
   u_long      ce_time;       /* cticks of last activity */

   /* content checking stuff */
   u_long      ce_pushsock;   /* web server push socket */
   u_long      ce_wcount;     /* html: words in document so far */
   u_long      ce_vlast;      /* html: word of last content match */
   u_long      ce_weight;     /* html: total weight of words */
   long        ce_clen;       /* content length remaining in section */

   struct pop_client * ce_pcl;   /* non-NULL if this is POP session */
   
   struct block_event            /* info if we blocked conn */
   {
      int      type;
      void *   ptr;
   } block;

   char        data[MAXDATALEN]; /* URL, DNS name, etc. */

   struct conn_entry * sort_next;   /* sorted list link */ 
   int         hashx;            /* hash index of this entry */

   /* NN related variables */
   u_long   ce_loresets;           /* TCP resets from local host */
   u_long   ce_rmresets;           /* TCP resets from remote host */
   u_long   ce_lofins;             /* TCP fins from local host */
   u_long   ce_rmfins;             /* TCP fins from remote host */
   u_long   ce_rtt;                /* best TCP trip time (microsec) */
   u_long   ce_timedseq;           /* TCP seq number being timed */
   struct timeval ce_seqtime;      /* time of ce_timedseq */
   u_long   ce_hops;               /* smoothed average IP hop count */
   u_long   ce_sec_pkts;           /* packets in curent second */
   u_long   ce_sec_bytes;          /* bytes in curent second */
   u_long   ce_bandwidth;          /* peak per-second bandwidth */
};

/* Additional TCP states made up for Buster, supplements other TCPS_ values. */
#define  TCPS_RESET  11    /* one of the peers sent a reset */
#define  TCPS_KILLED 12    /* Buster killed it */


#define  CEF_LOCALNODES    0x01  /* both IPs are local */

/* client/serbver state flags: Each TCP and UDP conenction should
 * have only one of the next two set.
 * If neither is set then we've missed something and have to guess.
 */
#define  CEF_LOCALCLIENT   0x0002   /* local IP is client */
#define  CEF_LOCALSERVER   0x0004   /* local IP is server */
#define  CEF_SERVERMASK    0x0006   /* client/server bits */
#define  CEF_RESETLOCAL    0x0008   /* Buster reset local endpoint */
#define  CEF_RESETREMOTE   0x0010   /* Buster reset remote endpoint */
#define  CEF_HTTP          0x0020   /* Connection is HTTP */

/* the next few only apply if CEF_HTTP is set */
#define  CEF_HTTPREQDONE   0x0040   /* HTTP request header done */
#define  CEF_HTTPREPDONE   0x0080   /* HTTP reply header done */
#define  CEF_HTTPGET       0x0100   /* HTTP Get or Post command */
#define  CEF_ISIMAGE       0x0200   /* file appears to be image */
#define  CEF_ISTEXT        0x0400   /* file appears to be text */
#define  CEF_MULTIREQ      0x0800   /* Connection had multiple http requests */
#define  CEF_HTTPCHUNKED   0x1000   /* stream is in checked mode */

#define  CEF_CLOSEDWIN     0x00010000   /* closed a browser window on this connection */
#define  CEF_FAKERESET     0x00020000   /* Possible forged reset detected */
#define  CEF_FAKEFIN       0x00040000   /* Possible forged FIN detected */


#define CEF_FILE_FLAGS \
   (CEF_ISTEXT | \
    CEF_ISIMAGE | \
    CEF_HTTPGET | \
    CEF_HTTPCHUNKED | \
    CEF_HTTPREPDONE | \
    CEF_HTTPREQDONE)


extern char * tcp_states[13];    /* strings for displaying TCP states */

extern struct conn_entry * ce_table[HASH_CONN_ENTRIES];
extern void * ce_table_mutex;

/* hash value macro for connection entry. All values are in local endian */
#define HASH_CE(laddr, faddr, lport, dport) \
   ((htonl(laddr) + htonl(faddr) + lport + dport) & (HASH_CONN_ENTRIES - 1))

extern   u_long   total_conns[MAXNETS];  /* current connections */

struct conn_entry *
br_cnupdate(struct ip * pip, char * mac, void * trhead,
      int type, int iface, int * fromremote);

struct conn_entry *
br_cnlookup(ip_addr ip1, ip_addr ip2, unshort port1,
      unshort port2, int type);

struct conn_entry *
conn_delete(struct conn_entry * to_del, int hashindex);

/* Highest and lowest addresses of a conneciton in heap memory */
extern struct conn_entry * conn_lowest;
extern struct conn_entry * conn_highest;

/****************** local MAC-based hosts database ******************/

struct laninfo;         /* predecl */

struct host_entry 
{
   struct host_entry * next;  /* hash list link */
   int      iface;         /* iface this host was found on */
   u_char   macaddr[6];    /* mac address, 2 bytes unused on ethernet */
   char     locname[MAXDNSNAME];     /* Locally assigned name */
   struct ip_hostentry * iphost; /* primary (only?) IP address of host */
   ip_addr  ipaddr2;       /* secondary IP address of host */
   ip_addr  dhcp_addr;     /* Possible newly assigned addr from DHCP */
   u_long   from_ip;       /* IP packets from primary IP */
   u_long   from_ip2;      /* IP packets from secondary IP */
   u_long   from_other;    /* IP packets not from the above addrs */
   struct laninfo * lan;   /* LAN this host is on */

   long     ip_lease;      /* duration of IP address (-1 == perm) */
   int      disctype;      /* discovery mechanism, eg H_ARP */
   int      h_flags;       /* bitmask of the H_ flags */
   char     imagefile[MAXFILENAME];    /* image picture VFS file */

   u_long   sec_pkts;      /* packet count this second */
   u_long   sec_bytes;     /* byte count this second */

   u_long   sec_p[60];     /* Last minute's counters, in sec. */
   u_long   sec_b[60];
   int      sec_index;     /* index into sec_b and sec_p */

   u_long   min_p[60];     /* last hour, in minutes */
   u_long   min_b[60];     /* 1 minute weighted packet count*/
   int      min_index;     /* index into min_b and min_p */

   u_long   hour_p[24];    /* last day, in hours */
   u_long   hour_b[24];
   int      hour_index;    /* index into hour_b and hour_p */

   u_long   day_p[7];      /* last week, in days */
   u_long   day_b[7];
   int      day_index;     /* index into day_b and day_p */

   u_long   total_pkts;    /* total packet count */
   u_long   total_bytes;   /* total byte count */
   u_long   active;        /* time (ctick) of last activity */
   u_long   age;           /* ctick when we discovered host */
   u_long   ip_exp;        /* ctick when IP lease expires */
   ip_addr  ip_bcast;      /* IP subnet broadcast address used */
};

#define H_ROUTER        0x00000001  /* host is the local router */
#define H_BSD           0x00000002  /* host is BSD *NIX */
#define H_WINDOWS       0x00000004  /* host is windows */
#define H_LINUX         0x00000008  /* host is Linux */
#define H_BCAST         0x00000010  /* The broadcast address */
#define H_DUPLIP        0x00000020  /* IP address is duplicate */
#define H_MULTILOCAL    0x00000040  /* Host used Multiple local IP addresses */
#define H_AGENT         0x00000080  /* host is running Buster */
#define H_HIDDEN        0x00000100  /* host is hidden in screens */
#define H_MULTIIP       0x00000200  /* host has uses multiple IP addresses */
#define H_SELF          0x00000400  /* This host is one of our ifaces */
#define H_MULTIHOMED    0x00000800  /* Host has 2 or more MACs */

/* H_ types for how we got the host locname. Higher values
 * take precedence, i.e. user assigned trumps DHCP
 */
#define H_NAMEMASK      0x0000F000  /* got name from netbios */
#define H_NONAME        0x00000000  /* not named yet */
#define H_DNSNAME       0x00004000  /* got name from DNS */
#define H_DHCPNAME      0x00006000  /* got name from DHCP */
#define H_NETBIOSNAME   0x00008000  /* got name from netbios */
#define H_USERNAME      0x0000C000  /* Name entered manually */

/* host discovery methods types. Note that these must be kept
 * in sync with discovery_type[] table.
 */
#define H_ARP     1     /* Learned Via ARP */
#define H_PING    2     /* Learned via ping reply */
#define H_NETBEUI 3     /* Learned via Netbeui */
#define H_DHCP    4     /* Learned via other UDP packet */
#define H_UDP     5     /* Learned via other UDP packet */
#define H_TCP     6     /* Learned via any TCP packet */
#define H_ICMP    7     /* Learned via ping reply */
#define H_SNMP    8     /* Learned via ping reply */
#define H_OTHER   9     /* none of the other methods */


/* FF_ flags are used to build a mask when putting the hosts
 * into lines of text data. A FF flags mask is built to indicate
 * status of name, DHCP, etc.
 */

#define  FF_TYPEMASK    0x000000FF     /* one of the H_ types above */

#define  FF_LANROUTER   0x00000100  /* Set if host is THE router */
#define  FF_FROMFILE    0x00000200  /* Set if host read from file */

 
/* string table for pretty-printing discovery types */
extern char * discovery_type[10];

extern struct host_entry * hosttab[HASH_HOSTS];
extern struct ip_hostentry * nulliph;
extern void * hosttab_mutex;

/* hosts are hashed by adding bytes of mac address */
#define HASH_HOST(mac) \
   (int)(((*mac) + *(mac+1) + *(mac+2) + *(mac+3) + \
      *(mac+4) + *(mac+5)) & (HASH_HOSTS - 1))

extern struct host_entry * br_hostupdate(char * macaddr, ip_addr ipaddr, int type, int iface);
extern struct host_entry * br_hostlookup(char * macaddr, int iface);

#ifndef HOST_ALLOC
#define HOST_ALLOC(size)   (struct host_entry *)npalloc(size)
#define HOST_FREE(ptr)     npfree((void *)ptr)
#endif

#ifdef WINDOWS_HOSTED
/* On windows these can be regular heap; embedded product may
 * require them to work from an ISR.
 */
#define CN_ALLOC(size)   (struct conn_entry *)npalloc(size)
#define CN_FREE(ptr)     npfree((void*)ptr)

/* Also, Windows builds need a quick reference to the co-resident
 * Winsock IP information, for "host only" mode:
 */
extern   struct ip_hostentry * winsock_ip;
#endif   /* WINDOWS_HOSTED */

/* Fast macro to see if two mac addresses are equal. This compares 16 bits
 * at a time since 32 bit compares could produce an alignment fault.
 * It tests the last word first since this is most likely to be unequal.
 */
#define MAC_EQ(mac1, mac2) \
   ((*(unshort*)(mac1 + 4) == *(unshort*)(mac2 + 4)) && \
    (*(unshort*)(mac1 + 2) == *(unshort*)(mac2 + 2)) && \
    (*(unshort*)(mac1) == *(unshort*)(mac2)))

struct host_entry *
new_host(int iface, char * macaddr, ip_addr ipaddr, int type, long lease);

struct host_entry *
host_delete(struct host_entry * del_host, int hashindex);

struct host_entry **  br_sorthosts(int * count);
char *   br_getmacname(struct host_entry * machost);

/*************** alert/alarm/log descriptions and info **************/

/* alarms and log entries are generated by the core Buster code,
 * which needs to be capable of rapid response (concievably even
 * an ISR), and cannot take a long time todo something like a
 * file write. Thus the alarms and log entries are queued by the
 * core code, and processed later by a lower priority thread. An
 * Interniche queue, similar to the packet free/rcvd design, is
 * used for this.
 */

extern brqueue alarms_free;
extern brqueue alarms_queued;

#define ALERT_NUM_EVENTS     32

extern int  alert_init(void);    /* prepare the free event queue */
extern void alert_check(void);   /* check and process queued events */

#define AF_LOGENTRY  0x00000001     /* set if this if a log entry, clear if alarm */
#define AF_BLOCK     0x00000002     /* set if this for block events */


/* Busters alarm type definition. To disable an alert, set the
 * actions field to zero.
 */

struct alarm_type      /* Alarm type definition */
{
   char *   name;          /* short name (for control name building) */
   char *   longname;      /* name for table heading */
   char *   message;       /* sprintf text to display in alert */
   u_long   actions;       /* bitmask of alarm action flags */
   int      interval;      /* initial backoff interval between alarms, 0 if none */
   char     al_url[64];    /* URL for more info */
   void *   parm;          /* type-specific information */
};

extern struct alarm_type   newmac;
extern struct alarm_type   conn_limit;
extern struct alarm_type   blocked_service;
extern struct alarm_type   blocked_ip;
extern struct alarm_type   virus_detected;
extern struct alarm_type   bad_app;
extern struct alarm_type   syn_flood;
extern struct alarm_type   remote_scan;
extern struct alarm_type   alarm_text;    /* generic, low priority alarm w/text */
extern struct alarm_type   no_ipaddr;     /* Network has no IP address */
extern struct alarm_type   dup_ipaddr;    /* Two MACs have same IP */
extern struct alarm_type   comm_update;   /* Community update error */
extern struct alarm_type   comm_file;     /* Community file read/write error */
extern struct alarm_type   bad_dhcp;      /* Malformed DHCP packet */
extern struct alarm_type   forged_tcp;    /* Forged TCP control packet */


/* definitions of alarm actions: */
#define     AL_EMAIL    0x01     /* Send email to alert recipients */
#define     AL_AUDIO    0x02     /* sound buzzer/bell */
#define     AL_POPUP    0x04     /* Windows dialog box or browser popup */
#define     AL_CONSOLE  0x08     /* printf to system console */
#define     AL_NUMBER       4


extern struct alarm_type * alarms[];     /* list of alarm types */

extern int alertcount;  /* number of alarms[] entries */

struct alert_alarm
{
   struct alarm_type * type;
   struct alarm_inst * instance;
   void *      parm;
};

/* One a_method for each alert "method" (beep, messagebox, email, icon) */
struct a_method
{
   char * name;         /* short name (for control name building) */
   int   bitmask;
};

extern struct a_method a_methods[];

extern int   alert_methodct;  /* number of a_methods[] entries */


#define BRLOG_PARMS  10

struct alert_log
{
   char * format;                /* format string for log entry */
   long  va_parms[BRLOG_PARMS];  /* br_log parameters */
};


/* the structure that is to be queued */
struct alert_event
{
   qp    next;       /* queue link */
   int   flags;      /* mask of AF_ flags above */
   union {
      struct alert_log     log;
      struct alert_alarm   alarm;
   } data;
};


#ifndef AL_ALLOC    /* Alert alloc routine */
#define AL_ALLOC(size)  npalloc(size)
#define AL_FREE(ptr)    npfree(ptr)
#endif

/* Note: this next one varies the passed size. 256 should be enough. */
#ifndef LM_ALLOC    /* Alert message alloc routine */
#define LM_ALLOC(size)          calloc(1, size)
#define LM_FREE(ptr)            free(ptr)
#endif

/* Log messages used for TCP blocks. By having these messages at well-know
 * addresses the loggin code can identify the TCP related log items for
 * passing to the blocking log window
 */
extern char * tcpLogMsgs[];

/* instances for global alarm types */
extern struct alarm_inst gen_cflood;
extern struct alarm_inst norouter_inst;
extern struct alarm_inst dupipaddr;
extern struct alarm_inst baddhcp;


#define  SECOND      TPS
#define  MINUTE      (SECOND * 60)
#define  HOUR        (MINUTE * 60)

/* general alarm generator routine */
extern void alarm_general(struct alarm_type *, struct alarm_inst *, void * parm);

/* br_popup() is a portabillity wrapper for MessageBox-like popups. This may
 * be a no-op on embedded systems.
 */
int   br_popup(struct alarm_type * type, char * errtext);
int   br_beep( void );                    /* Make a beep */
void  br_flash(int iterate, int level);   /* Flash Icon or LED */


/******************* Blocklog push code *********************/
struct blklog
{
   struct blklog * next;   /* list link */
   long        sock;       /* Browser link for this session */
   ip_addr     watchee;    /* client to display logs for */
   ip_addr     watcher;    /* client wathcing the  display */
};

struct blklog * blklogs;    /* list of active logs */

void  blklog_log(struct alert_event * evt);  /* Write event to block log */

      
/*************** IP-based hosts database **************/

struct other_agent;    /* predecl */

struct ip_hostentry 
{
   struct ip_hostentry * next;   /* hash list link */
   ip_addr  ipaddr;              /* IPv4 address of host */
   int      iface;               /* iface where it was detected */

   /* Three potential names for this host. Order of preference for display is
    * user, DNS, and Netbeui.
    */
   char     dnsname[MAXDNSNAME];    /* DNS (real) name */
   char     classId[MAXDNSNAME];    /* DHCP "Class ID" (option  60) */
   u_long   tm_detected;         /* ctick when we detected host */
   u_long   tm_lastpkt;          /* ctick of last pkt receipt */
   u_long   tm_lastconn;         /* ctick of last connection */
   u_long   tm_nextdns;          /* ctick of DNS query */
   int      dns_sent;            /* dns requests for this host's name */

   /* counts of packets we've seen sent by this host. */
   u_long   icmp_pkts;           /* ICMP packets sent */
   u_long   tcp_syns;            /* Unacked TCP syn packets sent */
   u_long   tot_pkts;            /* total packets sent */
   u_long   conns;               /* current connections */

   /* Network Neutrality (NN) related variables */
   u_long   ip_rtt;              /* smoothed average TCP trip time */
   u_long   ip_hops;             /* smoothed average IP hop count */
   u_long   bandwidth;           /* peak per-second bandwidth */
   u_long   sec_pkts;            /* packets in curent second */
   u_long   sec_bytes;           /* bytes in curent second */
   struct alarm_inst forgery;    /* forged FIN or reset */

   
   struct host_entry * mac_host; /* MAC host - router, NULL or local MAC */
   struct alarm_inst cflood;     /* connection flood alarm */
   struct alarm_inst scan;       /* port or IP address scan */
   struct alarm_inst synflood;   /* SYN flood alarm */
   struct alarm_inst badapp;     /* Verbotten application used */
   struct alarm_inst badserver;  /* Verbotten server accessed */
   int      flags;                  /* bitmask of the IH_ flags below */
   char     imagefile[MAXFILENAME]; /* picture of this IP host */
   struct other_agent * buster;     /* NULL if no agent on this IP */
   char     ratingHtml[256];        /* HTML for rating cell */
};

#ifndef IPH_ALLOC
#define IPH_ALLOC(size)   (struct ip_hostentry *)npalloc(size)
#define IPH_FREE(ptr)     npfree((void *)ptr)
#endif

#define IH_SERVER       0x0001      /* Host is/was a server */
#define IH_HACKER       0x0002      /* Hostile activity suspected */
#define IH_BROADCAST    0x0004      /* This is the broadcast pseudo-host */
#define IH_MULTICAST    0x0008      /* A Multicast pseudo-host */
#define IH_FAVIGET      0x0010      /* Last "GET" was for favicon */
#define IH_FAVICON      0x0020      /* Saw an OK for a favicon request */
#define IH_LOCAL        0x0040      /* Local, IP address */

#define IH_DHCP         0x0080      /* assigned by DHCP */
#define IH_DHCPEXPIRED  0x0100      /* Old DHCP address */

/* Get IP hosts name from passed IP address */
char *   br_getipname(ip_addr ipaddr);


#define HASH_IPH(ipaddr) \
   ((ipaddr + (ipaddr >> 24)) & (HASH_IPHOSTS-1))

extern struct ip_hostentry * ip_hosttab[HASH_IPHOSTS];
extern void * ip_hosttab_mutex;


/* IP host constructor */
struct ip_hostentry *   iphost_new(int iface, ip_addr ipaddr);

/* IP host destructor */
struct ip_hostentry * 
iphost_delete(struct ip_hostentry * to_del, int hashindex);

/* lookup IP host based on IP address and iface */
struct ip_hostentry *   br_iplookup(int iface, ip_addr ipaddr);

/* lookup IP address, take match on any iface */
struct ip_hostentry * br_ipanyif(ip_addr hostaddr);

/* get screen (text) name to use for an IP host */
char * br_getname(struct ip_hostentry * ip_host);

#ifdef WEBIO
/* SSI routine fetch the Buster host and iphost entries matching an IP
 * passed in name/value format from a form.
 */
int
br_formhost(struct httpform * form,          /* IN, has "host" entry */
            struct host_entry ** host,       /* OUT */
            struct ip_hostentry ** iphost);   /* OUT */
#endif

/* Fast macro to see if two mac addresses are equal. This compares 16 bits
 * at a time since 32 bit compares could produce an alignment fault.
 * It tests the last word first since this is most likely to be unequal.
 */
#define MAC_EQ(mac1, mac2) \
   ((*(unshort*)(mac1 + 4) == *(unshort*)(mac2 + 4)) && \
    (*(unshort*)(mac1 + 2) == *(unshort*)(mac2 + 2)) && \
    (*(unshort*)(mac1) == *(unshort*)(mac2)))


/* Copy a MAC address - ethernet version */
#define MACCPY(dest, src) memcpy(dest, src, 6)


/******* generic Buster textual functions *******/

char *   br_portname(unshort port);    /* get text name for port value */
char *   cn_state(struct conn_entry * conn); /* state of connection */
char *   cn_age(u_long ticks);         /* pretty-print ctick ticks */


/******** TCP manipulation functions *********/

int   br_tcpreset(struct conn_entry * conn, int local);
int   br_tcpblock(struct conn_entry * conn, int remote);

/* called for HTTP client packets */
int   br_htblock(PACKET pkt, struct tcphdr * tcph, 
           struct conn_entry * conn, int fromrem);


/********* misc prototypes **************/

extern int  same_subnet(ip_addr ip1, ip_addr ip2);
extern int  dnc_namereq(struct ip_hostentry * );
extern void br_adddnssvr(ip_addr dns_ipaddr);
extern int  dns_copyout(char * dest, char * src);
extern void br_clienthttp(struct conn_entry * conn, u_char * data, int dlen);
extern void br_serverhttp(struct conn_entry * conn, u_char * data, int dlen);
extern int  br_pop3(struct conn_entry * conn, u_char * data, int dlen,
               ip_addr srcip, ip_addr dstip);
extern int  pps_level(int pps);
int         br_spooftcp(struct conn_entry * conn, u_char * content, 
               int clength, int local, unshort flags);

/* Get name to use - may be "no name", will not be dot notation */
char *   br_getname(struct ip_hostentry * iphost);

extern char * noname;    /* possible return from br_getname: "(no name)" */

/* Get name to use - may be dot notation, will not be "no name" */
char *      br_getnametext(ip_addr ipaddr);

/* From Interniche misclib: */
extern int hex2bin(char * inbuf, unsigned char * outbuf, int bcount);

extern   void  br_running(void);


/* From Interniche emailer: */
int   smtpalert(char *buf, unsigned len, int (*callback)(char*, int) );

int   br_rtdisc(int iface);      /* Send router dicovery packet */
extern char * getlong(char * cp, u_long * val);

/* "Database" if insertion content */
extern char closeonly[];
extern int closeonly_length;

#ifdef WEBIO
extern struct conn_entry * get_form_conn(struct httpform * form);
#endif

/*************************** POP_3 database ********************************/

#define  MAXPOPLEN   100      /* max username, password, etc */

void      br_spamcheck(void);

struct pc3;    /* predecl */

/* struct pop_client - one of these is created for each email
 * end user detected on the LAN. They hold data and resources for
 * despamming the associated mail servers.
 */

struct pop_client
{
   struct pop_client * next;     /* linked list */
   u_long      interval;         /* averaged check interval (in cticks) */
   u_long      lastck;           /* last check interval (cticks) */
   u_long      lastdespam;       /* last time we despammed (cticks) */
   u_long      despamming;       /* start time while we are despamming, else 0  */
   int         spamstate;        /* one of the DS_ despamming states */
   int         unstable;         /* low if stable */
   int         pflags;           /* mask of POPF_ flags */
   int         pop_state;        /* one of the PS_ states below */

   struct p3c * pcl_popc;        /* Interniche level pop client */

   char        pcl_name[MAXPOPLEN];
   char        pcl_password[MAXPOPLEN];
   ip_addr     pcl_server;       /* don't use struct, since it may not exist yet */

   /* STAT results */
   u_long      num_msgs;
   u_long      num_bytes;

   /* State variables from looping through server's message list */
   char *      msg_list;      /* list of messages from server (from heap) */
   int         list_len;      /* length of actual string data in msg_list */
   char *      next_msg;      /* Pointer to next message in msg_list */
   char *      msg_buff;      /* Bufer for current message body (from heap) */
   int         msg_num;       /* number of current message */
   int         msg_size;      /* size of current message */

   struct ip_hostentry *   pcl_client;    /* pointer to client IP info */
   struct conn_entry *     conn;    /* current connection, if any */
};

#define  POPF_VERIFIED  0x01     /* User/Pass was OKed at least once by server */
#define  POPF_APOP      0x02     /* Client uses APOP instead of USER/PASS */
#define  POPF_FILTER    0x04     /* Client is being filtered by Buster */

extern struct pop_client * pop_clients;

#ifndef POPC_ALLOC
#define POPC_ALLOC(size)    (struct pop_client *)npalloc(size)
#define POPC_FREE(pcl)      npfree(pcl);
#endif

extern   int   pop_stable;


struct pop_client * br_newpcl(char * name, struct endpoint * client, ip_addr server);
int   br_delpcl(struct pop_client *);
struct pop_client * br_lookuppcl(char * username, struct conn_entry * conn);

/* define POP states */
#define  PS_AUTH_START     1     /* awaiting USER command */
#define  PS_AUTH_USER      2     /* USER command sent */
#define  PS_USER_OK        3     /* saw USER OK */
#define  PS_AUTH_PASS      4     /* PASS sent to server */
#define  PS_TRANSACTION    5     /* PASS OK, now in TRANSACTION */
#define  PS_UPDATE         6     /* server deleting messages */

/* define despam states */
#define  DS_IDLE           0     /* Not currently despamming */
#define  DS_CONNECTING     1     /* Connecting via pop */
#define  DS_STATING        2     /* Getting list stats */
#define  DS_LISTING        3     /* Requested message list */
#define  DS_MSG_READ       4     /* Requesting message body */
#define  DS_MSG_STATUS     5     /* Examining received body */
#define  DS_MSG_BODY       6     /* Examining received body */
#define  DS_MSG_DEL        7     /* Deleting based on spam analysis */
#define  DS_CLOSING        8     /* closing despam session */


/*********************** Agent to Agent protocol *****************/

/* records of other Agents detected on local network.
 *
 * These are used to determine which buster should be the sole
 * active buster at any point in time. They also co-ordinate
 * handover when Agents exec a new upgrade image.
 */

struct other_agent
{
   struct other_agent * next;
   ip_addr     ipaddr;
   u_long      last_activity;    /* in cticks */
   u_long      version;          /* e.g. 1.0.1.3 */
   u_long      instance;         /* instance of software */
   int         number;           /* deprecate??? */
   enum br_modes mode;           /* One of the status enums */
   char        macaddr[8];
   unshort     httpport;         /* Port web server listens to */
   struct ip_hostentry * iphost; /* Busters IP info */
};

extern struct other_agent * busterlist;


/* bustergram structure us the header of UDP packets exchanged
 * between Agents.
 */

struct bustergram
{
    //   enum brmodes mode;      /* One of the status enums */
   u_long   version;       /* version of sender */
   u_long   instance;      /* instance of software */
   long     number;        /* deprecate??? */
   u_char   authdata[16];  /* MD5 authentication digest */
   u_char   macaddr[8];    /* MAC address */
   u_long   active_ip;     /* IP address of actice interface */
   unshort  activeport;    /* port for web server access */
   int      mode;          /* reserved */
};

extern  unshort  msgport;   /* UDP port for bustergrams */

/* message related routines */
int   br_initmsg(void);          /* init the message system */
int   br_sendmsg(int opcode, ip_addr fhost);
void  mode_callback(void);       /* Timer tick */
/* UDP callback routine for received bustergrams */
int   bg_callback(PACKET, struct ip *);


/********** netbios headers (move to different file?) **********/

/* Microsoft Worm Vector (Netbios) header - looks a lot like DNS.... */

struct msworm_hdr
{
   unshort  id;      /* transaction ID */
   unshort  flags;   /* flags and opcode */
   unshort  qrrs;    /* number of questions RRs */
   unshort  ans;     /* number of answers RRs */
   unshort  auth;    /* number of authority RRs */
   unshort  add;     /* number of additional RRs */
   u_char   data;    /* first byte of data area */
};

struct netbios_dgram
{
   u_char   type;       /* 17 is "group datagram" */
   u_char   flags;      /* 0x02 is Bnode, first fragment, no more frags */
   unshort  id;         /* who cares.... */
   ip_addr  ipaddr;     /* in case you didn't get the IP/UDP headers.... */
   unshort  port;
   unshort  length;
   unshort  offset;
   u_char   data;       /* start of Netbios name gibberish */
};

#define  MAX_ICMP_TYPE  15
extern char * icmp_types[MAX_ICMP_TYPE + 1];
#define  BAD_ICMP_TYPE  1


/*************** Content filtering words ***************/

#define  MAXVWORD       16    /* MAX Length of word */
#define  VOCABHASHSIZE  1024  /* Size of word hash table */


extern   int   filter_web;    /* filtering flag, was active_vocabs */

/* A single word in the filtering list */
struct vword
{
   struct vword * nexthash;      /* Next word in hash bucket */
   struct vword * sort;          /* Next word in sorted list */
   struct vword * answerlist;    /* Next matching word */
   char     word[MAXVWORD];      /* text of word */
   int      weight;              /* 1-10 or -1 for SHOWSTOPPER */
   struct ctdesc  ctl;           /* word's control struct */
   u_long   hits;                /* number of times we hit word */
};

#define  VW_ENDPHRASE   (void*)(-1)      /* word is part of a phrase */

#define  WT_SHOWSTOPPER    -1    /* Infinitly high weight */

/* hash table containg all the words in the system. */
extern struct vword * wordhashtab[VOCABHASHSIZE];

struct vword * word_lookup(char * word);
struct vword * new_word(char * word, struct comm_dbud * comm);
void  del_word(struct vword * wordp);
int   wordhash(char * word);   /* make hash value for word */
int   wordcmp(char * word1, char * word2);
void  wordcpy(char * word1, char * word2);
int   br_writewords(struct comm_dbud * comm, FILE * fout);

/************************* LAN info database ***********************/

/* laninfo - structure containing info about various LANs this Agent
 * may frequent. Name is used to access host lists in br_lanfile.
 */

struct laninfo
{
   struct laninfo * next;
   char     name[MAXNAMELEN]; /* e.g. "Home" or "office" */
   u_char   router_mac[8];    /* MAC address or router */
   ip_addr  router_ip;        /* IP address of router */
   int      flags;            /* mask of the LN_ flags below */
};

extern   struct laninfo * br_lans;  /* linked list of known lans */
extern   char * br_lanfile;         /* Name of LAN info file */
extern   int   lan_file_read;

#define  LN_TEMP     0x01           /* temporary (unnamed) lan */

int   br_writelan(void);            /* Save LAN MAC info */
int   br_filelan(FILE * fout, struct laninfo * );  /* Write LAN host info to file */
int   br_readlan(struct laninfo *); /* read in hosts for named LAN */
int   br_readhost(char * readbuf, struct laninfo *);  /* Parse LAN MAC host info into RAM */
void  br_checklan(void);            /* switching to different lan */
int   br_readlans(void);            /* read hosts (network.db) file */

/* Constructor & destructor of LAN object */
struct laninfo *  new_lan(char * name, u_char * router_mac, ip_addr router_ip);
void  del_lan(struct laninfo *);    /* destructor */
struct laninfo *  br_lanlookup(char * name);


/************************* whois server support ***********************/

#define  WHOSERVERS  5
#define  WHOBUFSIZE  5000


enum  whostates
{
   WHO_DNSING,       /* doing DNS on server name */
   WHO_CONNECTING,   /* opening TCP connection to server */
   WHO_CONNECTED,    /* connected, ready for query */
   WHO_ANSWERING,    /* Sent request, waiting for info */
   WHO_DONE,         /* Answer is ready */
   WHO_FAILED,       /* serious error, reply is unsable */
};

struct whoserver
{
   char     dnsname[MAXDNSNAME];    /* DNS name of server */
   unshort  whoport;
   char     whobuf[WHOBUFSIZE];     /* buffer for reply */
   int      received;               /* count of received bytes */
   int      flags;                  /* mask of the WH_ flags below */
   int      errors;                 /* consecutive errors since last success */
   enum whostates state;

   /* per-session stuff */
   ip_addr     ipaddr;              /* IP address of server */
   long        socket;
};

#define  WH_IPTYPE      0x01     /* server good for IP address lookups */
#define  WH_NAMETYPE    0x02     /* server good for DNS name lookups */


extern struct whoserver whoservers[WHOSERVERS];

int   br_whois(ip_addr ipaddr, char ** errmsg, int iptype);



/************** Community Database support ***************/

enum cmstate {
   IDLE,          /* between updates */
   READ_HEADER,   /* Reading community update from socket */
   SAVE_UPDATE,   /* Saving update stream to temp file. */
   PARSING,       /* Writing updates to local DB */
   FILING,        /* Backing up to local file */
   FAILED         /* Last try failed */
};

struct comm_dbud  /* Community database update info */
{
   struct comm_dbud * next;   /* community list link */
   char     cm_url[MAXURL];   /* DB server URL */
   char *   cm_urlpath;       /* path portion of cm_url */
   enum  cmstate cm_state;    /* IDLE, DNS, READ, etc. */
   ip_addr  cm_ipaddr;        /* IP address, net endian, from URL */
   unshort  cm_port;          /* port, net endian, 80 or from URL */
   int      cm_priority;      /* priority of this communty */
   char     cm_name[MAXFILENAME];      /* community name */
   char     cm_filename[MAXFILENAME];  /* filename for database */
   char     cm_tmpname[MAXFILENAME];   /* name of tmp file */
   long     cm_socket;        /* connection socket */
   FILE *   cm_fd;            /* Communnities open DB file */
   FILE *   cm_tmp;           /* tmp DB file from network update */
   u_long   cm_update;        /* date of last update */
   u_long   cm_interval;      /* seconds to wait for next update */
   u_long   cm_revision;      /* revision of last update */
   u_long   cm_records;       /* total number of records */
   u_long   cm_currec;        /* scratch - current number of records */
   u_long   cm_deadman;       /* Deadman timer for stalled downloads */
   int      cm_data;          /* bytes of unporcessed data in cm_buf */
   char     cm_buf[5000];     /* buffer for socket I/O */
   char     cm_host[MAXURL];  /* copy of host from cm_url */
   struct alarm_inst cm_alarm;   /* Alarm for community IO error */
   int      cm_flags;         /* mask of CM_ flags */
};

#define   CM_NEWDATA    0x01  /* Community has new data for filing/feedback */
#define   CM_NOMAC      0x02  /* Don't use MAC address in next update URL */

/* Enable code to work around temp server problems */
#define SERVER_WORKAROUND  1

extern  void *  br_comm_mutex;

extern  u_long  br_dbdate;     /* date/time of last update */
extern struct comm_dbud * br_communities;
extern struct comm_dbud local_comm;

int   br_initcomm(void);   /* Init the communities */

#ifdef BR_COMMUNITIES
extern  u_long    br_dbdate;     /* date/time of last update */
extern  char *    br_prtcomm(struct comm_dbud * comm);
#else /*  BR_COMMUNITIES not defined */
#define br_dbdate 0
#define br_prtcomm(x) "No community support in this build"
#endif


/* Community basic object management */
struct comm_dbud *   br_cmlookup(char * url);
struct comm_dbud *   br_cmbyfile(char * filename);
struct comm_dbud *   br_cmnew(char * url, char * name, int priority);
int                  br_cmdel(struct comm_dbud * comm);
char *               br_cmupdate(struct comm_dbud * comm);

/* Community DB file read/write/check */
int   br_dbread(struct comm_dbud * comm);
int   br_dbwrite(struct comm_dbud * comm);
char* br_dbcheck(FILE * fp, struct comm_dbud * comm);
char* br_readdbfile(FILE * fp, struct comm_dbud * comm);

/* Extract a specified field from a record */
char *   br_getfield(char * rec, char * field);

/* Community single record parse routines */
char *   br_parseitem(char * itemtext, struct comm_dbud * comm, int commit);
char *   br_blkbinary(char * readbuf, struct comm_dbud * comm, int commit);
char *   br_mkimage(char * base, struct comm_dbud * comm, int commit);
char *   br_blkip(char * buf, struct comm_dbud * comm, int commit);
char *   br_blkword(char * readbuf, struct comm_dbud * comm, int commit);
char *   br_blkdns(char * readbuf, struct comm_dbud * comm, int commit);
char *   br_blkpath(char * readbuf, struct comm_dbud * comm, int commit);
char *   br_blkport(char * readbuf, struct comm_dbud * comm, int commit);

/* Community btree record-type write routines */
void  br_blkdnsout(struct btnode * n, long param, int depth);
void  br_blkipout(struct btnode * n, long param, int depth);
void  br_blkpathout(struct btnode * n, long param, int depth);
void  br_blkportout(struct btnode * n, long param, int depth);
void  br_imageout(struct btnode * n, long param, int depth);

/* Binary sig write routine */
long  br_filesigs(FILE * fp, struct comm_dbud * comm);

/* Get length of text up to first control char */
int   strcrlen(char * text);

/************ Firmware/Program upgrade support *************/

/* record for DBTYPE_UPDATE */
#define  BR_UDPLEN   128   /* Max. length of update path/name string */

struct   br_upgrade
{
   u_long   version;    /* version of new update image */
   u_long   vclass;     /* class of update image (windows, ARM) */
   u_long   ipaddr;     /* IP address of image server */
   unshort  port;       /* port on ipaddr to use */
   char     pathname[BR_UDPLEN];    /* path/name for HTTP GET cmd */
};

#define  VCLASS_WINTEL  1     /* Windows/Pentium */
#define  VCLASS_ARM     2     /* reserved */

#define PUSH_VERSION(v)  \
   (unsigned)(v>>24),     \
   (unsigned)((v>>16)&0xff),\
   (unsigned)((v>>8)&0xff),\
   (unsigned)(v&0xff)

/* Callback to start upgrade sockert. This should be called from a
 * thread created by br_newimage(), not in the context of br_newimage()
 * itself.
 */
SOCKTYPE br_requpgrade(ip_addr ipaddr, unshort port, char * pathname);


/************ User selectable SRC server support *************/

struct srcserv
{
   struct srcserv * next;
   char     ms_name[MAXNAMELEN]; /* Name for server */
   char     ms_url[MAXURL];      /* Server's URL */
   int      ms_weight;           /* 1-100 */
   int      ms_active;           /* Currently activated by user */
   u_long   ms_hits;
};

extern   struct srcserv * myservers;    /* User SRC server list */
extern   int   br_srcserving; /* True if content swapping is active */

extern char * br_nextms(void);


/************ Support for "plug-in" feature *************/

/* Defined for when the plug-in is invoked */
#define PL_NEWNAME      1  /* On each new DNS name (web server) */
#define PL_NEWIPADDR    2  /* on each new IP address */
#define PL_NEWCONN      4  /* On each new conneciton */
#define PL_EACHPKT      8  /* On every received packet (dangerous!) */

extern   int   pl_mastermask;

struct plug_in
{
   struct plug_in * next;     /* list link */
   void(*callback)(void * object, struct plug_in *);
   int      pl_type;          /* one of the PL_ defiones */
};


struct plug_in *  add_plugin(
                void(*callback)(void * object, struct plug_in *), 
                int imask);

int  del_plugin(struct plug_in *);

void  pl_newitem(void * object, int typecode);

struct plug_in * plugins;


#endif   /* _BRDB_H_  end - Prevent double includes */

/*
Copyright (C) 2003-2008 by Praemio Inc. All rights reserved.

Author: John Bartas - jbartas@praemio.com

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/



