/* brdns.c
 * 
 * Definitions for Buster's DNS tracking & sniffing system 
 * 
 * Copyright 2003-2008 by Praemio Inc. All rights reserved.
 * Author: John Bartas - jbartas@praemio.com
 * This software is released under the terms of the LGPL.
 * See LGPL Notice at end of this file; Full license in ../license.txt
 */

#ifndef _BRDNS_H_
#define  _BRDNS_H_  1


/* Tunable DNS constants */
#define  DNSSERVERS     3 
#define  DNSAXMADDRS    10    /* IP addrs in return from gethostbyname() */
#define  DNSMAXALIAS    2


/* Fixed DNS constants */
#define  DNSNAMELEN     256
#define  DNSUDPLEN      512

/* DNS Record type codes */
#define  DNS_TYPE_QUERY    0     /* Type value for question record */
#define  DNS_TYPE_IPADDR   1     /* Type value for IPv4 address */
#define  DNS_TYPE_AUTHNS   2     /* Authoritative name server */
#define  DNS_TYPE_ALIAS    5     /* Alias for queried name */
#define  DNS_TYPE_ZONE     6     /* Start of zone authority */
#define  DNS_TYPE_PTR      0x0c  /* PTR (inverse lookup) */
#define  DNS_TYPE_AAAA     0x1c  /* IPv6 address ("AAAA" record) */

/* DNS header overlay */
struct dns_hdr
{
   unshort  id;
   unshort  flags;
   unshort  qdcount;
   unshort  ancount;
   unshort  nscount;
   unshort  arcount;
};

/* From BSD: Description of data base entry for a single host.  */

#ifndef _WINSOCKAPI_
struct hostent
{
    char *   h_name;        /* Official name of host. */
    char **h_aliases;       /* Alias list.  */
    int   h_addrtype;       /* Host address type.  */
    int   h_length;         /* Length of address.  */
    char **h_addr_list;     /* List of addresses from name server.  */
};
#endif

#define  h_addr   h_addr_list[0] /* Address, for backward compatibility.  */

extern ip_addr  dns_servers[DNSSERVERS];

extern int  br_dnsquery(char * dname, ip_addr * result );
extern int  br_dnsreply(struct dns_hdr * dns, 
                        struct conn_entry * conn, unshort dnslen);

#endif  /* _BRDNS_H_ */

/*
Copyright (C) 2003-2008 by Praemio Inc. All rights reserved.

Author: John Bartas - jbartas@praemio.com

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/

