/* brmenus.c
 *
 * menu routines. 
 *
 * ROUTINES: 
 *  br_writedatetime(), cn_state(), br_portname(), 
 *  cn_age(), br_hostlist(), br_connlist(), br_hashload(), 
 *  br_state(), br_remotelist(), br_rxtoggle(), br_vtest(), 
 *  br_vspeed(), cip_list(), br_listcip(), br_getname(), br_atest(), 
 *  br_logtest(), br_threads(), br_readdb(), br_writedb(), 
 *  br_senddbq(), br_setbud(), br_dbstate(),  
 *  br_sendhello(), br_blist(), br_content(), br_b64decode(), 
 *  ck_checknet(), br_hibernate(), br_traffic(), br_exception(), 
 *  br_dopanic(),  
 *
 * Copyright 2003-2008 by Praemio Inc. All rights reserved.
 * Author: John Bartas - jbartas@praemio.com
 * This software is released under the terms of the LGPL.
 * See LGPL Notice at end of this file; Full license in ../license.txt
 */


#include "busterport.h"
#include "brnet.h"
#include "ip.h"
#include "arp.h"
#include "brdb.h"
#include "brdns.h"
#include "brmenus.h"

char promptbuf[24] = {"Buster> "};
char * prompt = &promptbuf[0];

#ifdef NNSQUAD
#else
#endif         /* NNSQUAD or not */      

extern  char * br_getname(struct ip_hostentry * ip_host);
extern  u_long   pattern_dups;
extern  int   wrap_stats(void * pio, char * params);
extern  void  br_portthreads(void * pio, char * params);

int br_hostlist(void * pio, char * params);
int br_connlist(void * pio, char * params);
int br_hashload(void * pio, char * params);
int br_state(void * pio, char * params);
int br_remotelist(void * pio, char * params);
int br_rxtoggle(void * pio, char * params);
int br_vtest(void * pio, char * params);
int br_vspeed(void * pio, char * params);
int ck_checknet(void * pio, char * parms);
int br_listcip(void * pio, char * params);
int br_atest(void * pio, char * params);
int br_queues(void * pio, char * params);       /* dump pkt queue stats */
int br_logtest(void * pio, char * params);
int br_threads(void * pio, char * params);
int br_readdb(void * pio, char * params);
int br_writedb(void * pio, char * params);
int br_senddbq(void * pio, char * params);      /* send update request */
int br_setbud(void * pio, char * params);       /* set new community in system */
int br_dbstate(void * pio, char * params);      /* show DB update variables */
int br_sendhello(void * pio, char * params);    /* send interbuster hello packet */
int br_blist(void * pio, char * params);        /* list Busters on local segment */
int br_content(void * pio, char * params);      /* run a file through content filter */
int br_b64decode(void * pi, char * params);
int br_traffic(void * pio, char * params);      /* show weighted traffic stats */
int br_exception(void * pio, char * params);    /* test exception handling */
int br_dopanic(void * pio, char * params);      /* test panic handling */
int br_hibernate(void * pio, char * params);    /* simulate hibernation */
int br_images(void * pio, char * params);       /* list loaded MAC images */
int br_inlist(void * pio, char * params);       /* list known Internet hosts */
int br_ifstats(void * pio, char * params);      /* print Buster per-interface counters */
int br_system(void * pio, char * params);       /* print Buster genic system stuff */
int br_dtrap(void * pio, char * params);        /* trap to debugger vi dtrap */
int br_sendarp(void * pio, char * params);      /* send gratuitous arp */
int br_dumparp(void * pio, char * params);      /* Dump local (not system) arp table */
int br_arpscan(void * pio, char * params);      /* Send ARPs to local IP adresses */


int
br_menuhelp(void * io, char * parm)
{
    int     menu;
    struct  br_menuentry * menup;

    for(menu = 0; menulist[menu] != NULL; menu++)
    {
        for(menup = menulist[menu]; menup->text; menup++)
        {
            bm_printf(io, "%-12s - %s\n", menup->cmd, menup->text);
        }
    }
    USE_ARG(parm);
    return 0;
}


static int matchlen(char * s1, char * s2)
{
    int len = 0;
    while((*s1) && (*s1++ == *s2++))
        len++;
    return len;
}

int
br_menucmd(void * outdev, char * input)
{
    int     menu;
    int     len;
    int     inputlen;       /* length of input string */
    int     err;
    char *  end;
    char *  firstspace;
    struct  br_menuentry * tmp;
    struct  br_menuentry * best;    /* best match for command input */
    int     bestlen = 0;            /* Match length of "best" */

    /* Strip trailing  junk - CR, etc */
    end = input + strlen(input) - 1;
    while((end > input) && (*end <= ' '))
    {
        *end = 0;
        end--;
    }

    if((input == NULL) || (*input == 0))
    {
        dtrap();
        return ENP_PARAM;
    }

    if(*input == '\n')
    {
        bm_printf(outdev, prompt);
        return 0;
    }

    if( *input == '?')
    {
        br_menuhelp(outdev, NULL);
        bm_printf(outdev, prompt);
        return 0;
    }

    /* Look for the menu command which is the best match to "input". "input" 
     * must not be longer than best match, Although best match may be longer 
     * than input. 
     */
    firstspace = strchr(input, ' ');
    if(firstspace)
        *firstspace = 0;        /* terminate first arg */

    inputlen = strlen(input);   
    best = NULL;
    for(menu = 0; menulist[menu] != NULL; menu++)
    {
        for(tmp = menulist[menu]; tmp->cmd; tmp++)
        {
            len = matchlen(tmp->cmd, input);
            if((len > bestlen) && (len == inputlen))
            {
                best = tmp;
                bestlen = len;
            }
        }
    }
    if(!best)
    {
        bm_printf(outdev, "Command \"%s\" not found\n", input);
        return ENP_PARAM;
    }

    if(firstspace)
       *firstspace = ' ';  /* unterminate rag string */

    err = best->func( outdev, nextarg(input));
    bm_printf(outdev, prompt);
    return err;
}

struct br_menuentry mainmenu[] = 
{
    {  "atest",    br_atest,      "send a test alarm [type]"},
    {  "b64decode",br_b64decode,  "test base64 decoder"},
    {  "blist",    br_blist,      "list Busters on local segment"},
    {  "buster",   br_state,      "display Buster state info"},
    {  "content",  br_content,    "run a file through content filter"},
    {  "ckcheck",  ck_checknet,   "check the br_checknet() routine"},
    {  "nodes",    br_hostlist,   "list known 'nodes' on the net"},
    {  "conns",    br_connlist,   "list current connections"},
    {  "cipaddrs", br_listcip,    "list control IP addresses"},
    {  "dtrap",    br_dtrap,      "trap to debugger via dtrap"},
    {  "dumparp",  br_dumparp,    "dump internal arp table"},
    {  "exception",br_exception,  "test the exception handling"},
    {  "hashload", br_hashload,   "display hash table loads"},
    {  "help:",    br_menuhelp,   "print list of commands"},
    {  "hello:",   br_sendhello,  "send HELLO message"},
#ifdef    WEBIO_GUI
    {  "images",   br_images,     "list loaded MAC images"},
#endif
    {  "iface",    br_ifstats,    "print per-interface counters"},
    {  "inlist",   br_inlist,     "list known Internet hosts"},
    {  "ltest",    br_logtest,    "force a log entry"},
#ifdef MEM_WRAPPERS
    {  "memory",   wrap_stats,    "heap memory usage"},
#endif
    {  "queues",   br_queues,     "dump pkt queue stats"},
    {  "remote",   br_remotelist, "display remote machine list"},
    {  "rxtoggle", br_rxtoggle,   "switch net monitor on/off"},
    {  "sendarp",  br_sendarp,    "send gratuitous arp"},
    {  "scanarp",  br_arpscan,    "Send ARPs to local IP adresses"},
    {  "system",   br_system,     "generic Buster system stats"},
    {  "threads",  br_threads,    "show thread status"},
    {  "traffic",  br_traffic,    "show weighted traffic stats"},
#ifdef VIRUS_CHECK
    {  "vspeed",   br_vspeed,     "check virus scanning speed"},
    {  "vtest",    br_vtest,      "test virus pattern search"},
#endif
    {  "hibernat", br_hibernate,  "fake a hibernate/wakeup"},
    {  "panic",    br_dopanic,    "test panic handling"},
    {  NULL, NULL, NULL },
};


struct br_menuentry db_menu[] = 
{
    {   "dbread",   br_readdb,     "(re)read database from file"},
    {   "dbwrite",  br_writedb,    "write database to file"},
    {   "dbreq",    br_senddbq,    "send database update request"},
    {   "dbserver", br_setbud,     "set IP of update server"},
    {   "dbstate",  br_dbstate,    "show DB update variables"},
    {   NULL, NULL, NULL }
};

struct br_menuentry * 
menulist[] = 
{
    &mainmenu[0],
    &db_menu[0],
    NULL
};


/* This must be kept in sync with the "H_" typedefs, e.g. "ICMP" 
 * below should be indexed by H_ICMP.
 */
char * discovery_type[] =
{
   "NULL",
   "ARP",
   "PING",
   "Netbios",
   "DCHP",
   "UDP",
   "TCP",
   "ICMP",
   "SNMP",
   "OTHER"
};

char * tcp_states[]=
{
   "closed",         /* 0   closed */
   "listen",         /* 1   listening for connection */
   "syn_sent",       /* 2   active, have sent syn */
   "syn-recd",       /* 3   have send and received syn */
   "established",    /* 4   established */
   "close-wait",     /* 5   rcvd fin, waiting for close */
   "fin-wait1",      /* 6   have closed, sent fin */
   "closing",        /* 7   closed xchd FIN; await FIN ACK */
   "last-ack",       /* 8   had fin and close; await FIN ACK */
   "fin-wait2",      /* 9   have closed, fin is acked */
   "time-wait",      /* 10  in 2*msl quiet wait after close */
   "reset",          /* Special, added for Buster */
   "killed",         /* Special, added for Buster */
};



/* return text for the state of a connection */

char *
cn_state(struct conn_entry * conn)
{
   if(conn->ce_flags & (CEF_RESETLOCAL|CEF_RESETREMOTE))
      return "killed";

   switch(conn->ce_type)
   {
   case H_TCP:
      return( tcp_states[conn->ce_state] );
      break;
   case H_UDP:
   case H_ICMP:
   case H_OTHER:
      if(conn->ce_time < (cticks - (5 * TPS)))
         return("IDLE");
       else
         return("ACTIVE");
      break;
   default:
      dtrap();
      return "";
   }
}


/* return text for a well-known port application (ie 80->"html") */

char *
br_portname(unshort port)
{
   switch (port)
   {
   case 0:
      return "O.S.";
   case 1:
      return "tcpmux";
   case 7:
      return "echo";
   case 11:
      return "sysstat";
   case 19:
      return "chargen";
   case 20:
      return "ftp data";
   case 21:
      return "ftp cmd";
   case 22:
      return "ssh";
   case 23:
      return "telnet";
   case 25:
      return "SMTP-email";
   case 53:
      return "DNS query";
   case 67:
      return "DHCP client";
   case 68:
      return "DHCP server";
   case 69:
      return "TFTP";
   case 79:
      return "finger";
   case 80:
      return "WWW";
   case 98:
      return "Linuxconf";
   case 109:
      return "POP2-email";
   case 110:
      return "POP3-email";
   case 111:
      return "rpc";
   case 113:
      return "ident";
   case 119:
      return "news";
   case 135:
      return "pop-up";
   case 137:
      return "NBNS";
   case 138:
   case 139:
      return "NetBios";
   case 143:
      return "IMAP4";
   case 161:
      return "SNMP";
   case 162:
      return "SNMP-trap";
   case 177:
      return "X-Win";
   case 190:
      return "MS-SSDP";    /* UDP only */
   case 443:
      return "SHTTP";
   case 445:
      return "Netbios";
   case 513:
      return "rwho";
   case 515:
      return "print";
   case 535:
      return "CORBA";
   case 600:
      return "backdoor";
   case 635:
      return "MountD";
   case 1080:
      return "socks?";
   case 1114:
      return "SQL?";
   case 1243:
      return "Sub7-virus?";
   case 1433:
      return "MS-SQL/virus?";
   case 1524:
      return "backdoor?";
   case 1800:
      return "MS UPnP";
   case 2049:
      return "NFS?";
   case 3128:
      return "Squid?";
   case 5632:
      return "PC anywhere";
   case 6776:
      return "Sub-7 virus";
   case 7070:
      return "RealAudio";
   case 8000:
   case 8001:
   case 8080:
   case 8888:
      return "HTTP-Proxy";
   case 17027:
      return "Adware";
   case 27374:
      return "Sub-7";
   case 30100:
      return "NetSphere";
   case 31337:
      return "BackOrifice";
   case 31789:
      return "Hack-a-Tack";
   case 41508:
      return "Inoculan";
   default:
      return NULL;
   }
}


/* pretty-print a passed tick value in hours, min. sec. format */

char age_text[24];  /* return buffer */

char *
cn_age(u_long ticks)
{
   char * cp = age_text;

   if(ticks > HOUR)
   {
      sprintf(cp, "%lu hours, ", (u_long)(ticks/HOUR) );
      cp += strlen(cp);
      ticks -= ((ticks/HOUR)*HOUR);
   }
   if(ticks > MINUTE)
   {
      sprintf(cp, "%lu min., ", (u_long)ticks/MINUTE);
      cp += strlen(cp);
      ticks -= ((ticks/MINUTE)*MINUTE);
   }
   sprintf(cp, "%lu sec.", (u_long)ticks/TPS);
   return age_text;
}


int
br_hostlist(void * pio, char * args)
{
   int      i;
   char *   cp;
   int      hosts = 0;
   struct host_entry * host;

   for(i = 0; i < HASH_HOSTS; i++)
   {
      if(hosttab[i])
      {
         for(host = hosttab[i]; host; host = host->next)
         {
            hosts++;

            bm_printf(pio, "Ethernet: %02x:%02x:%02x:",
               (u_char)host->macaddr[0], (u_char)host->macaddr[1], (u_char)host->macaddr[2] );
            bm_printf(pio, "%02x:%02x:%02x ",
               (u_char)host->macaddr[3], (u_char)host->macaddr[4], (u_char)host->macaddr[5] );

            bm_printf(pio, "IP: %u.%u.%u.%u ", PASS_IPADDR(host->iphost->ipaddr));
            bm_printf(pio, "via %s", 
               discovery_type[host->disctype & FF_TYPEMASK] );
            cp = br_getipname(host->iphost->ipaddr);
            if(cp && strcmp(cp, "(no name)"))
                bm_printf(pio, "  (%s)\n", cp );
            else
                bm_printf(pio, " \n " );
         }
      }
   }
   if(hosts)
      bm_printf(pio, "Total of %d computers found on the network.\n", hosts);
   else
      bm_printf(pio, "No computers found on the network yet.\n");

   return 0;
}

int
br_connlist(void * pio, char * args)
{
   int   i;
   int   count = 0;
   struct conn_entry * conn;

   for(i = 0; i < HASH_CONN_ENTRIES; i++)
   {
      if(ce_table[i] == NULL)
         continue;

      if(count == 0)
      {
      bm_printf(pio, 
      "IP Addresses        ports       pkts     bytes   state\n");
      }

      conn = ce_table[i];
      while(conn)
      {
         bm_printf(pio, "%u.%u.%u.%u <-> ", PASS_IPADDR(conn->loc.ce_addr) );
         bm_printf(pio, "%u.%u.%u.%u,   ", PASS_IPADDR(conn->rem.ce_addr) );

         bm_printf(pio, "%u <-> %u   ", 
            conn->loc.ce_port, conn->rem.ce_port );

         bm_printf(pio, "%lu  %lu   ", conn->rem.ce_outpkts, conn->loc.ce_outpkts );

         if(conn->ce_type == H_TCP)
            bm_printf(pio, "%d \n", conn->ce_type );
         else
            bm_printf(pio, "\n");
         
         conn = conn->next;
         count++;
      }
   }
   if(count == 0)
      bm_printf(pio, "No Connections detected\n");

   return 0;
}

/* Display the load distribution in various hash tables */

int
br_hashload(void * pio, char * args)
{
   int      i;
   int      total = 0;   /* total items listed */
   int      count = 0;   /* total items in current bucket */
   char *   option;
   struct conn_entry * conn;
   struct host_entry * host;
   struct ip_hostentry * iphost;

   option = args;
   if(!*option)
   {
      bm_printf(pio, "usage: hashload [hosts|ip|conns|virus]\n");
      return -1;
   }
   bm_printf(pio, "Hash distribution:\n");

   switch(*option)
   {
   case 'h':   /* list hosts */
      bm_printf(pio, "hosts table ");
      for(i = 0; i < HASH_HOSTS; i++)
      {
         count = 0;
         host = hosttab[i];
         while(host)
         {
            count++;
            host = host->next;
         }
         if(count)
         {
            total += count;
            bm_printf(pio, "%d[%d],", i, count);
         }
      }
      break;
   case 'c':
      bm_printf(pio, "connections table ");
      total = 0;
      for(i = 0; i < HASH_CONN_ENTRIES; i++)
      {
         count = 0;
         conn = ce_table[i];
         while(conn)
         {
            count++;
            conn = conn->next;
         }
         if(count)
         {
            total += count;
            bm_printf(pio, "%d[%d],", i, count);
         }
      }
      break;
   case 'i':
      bm_printf(pio, "IP hosts table ");
      total = 0;
      for(i = 0; i < HASH_IPHOSTS; i++)
      {
         count = 0;
         iphost = ip_hosttab[i];
         while(iphost)
         {
            count++;
            iphost = iphost->next;
         }
         if(count)
         {
            total += count;
            bm_printf(pio, "%d[%d],", i, count);
         }
      }
      break;
   case 'v':
    {
      int      max;     /* entrys per byte */
      u_char   mask;
      int      bitct;   /* Kyles Mom is a bit count */
      long     used;    /* buckets used in array */
      struct pattern_entry * maxp;
      int      hi,lo;

      bm_printf(pio, "Virus signature bit table ");
      total = 0;
      max = 0;
      hi = 0;
      lo = HASH_PATTERN_BITS;
      for(i = 0; i < HASH_PATTERN_BITS; i++)
      {
         mask = (u_char)pattern_bits[i];
         if(mask)
         {
            if(i > hi) hi = i;   /* save high and low index w/flags */
            if(i < lo) lo = i;
            count = 0;
            for(bitct = 0; bitct < 8; bitct++)
            {
               if(mask & 1)
                  count++;
               mask >>= 1;
            }
            if(count > max)
               max = count;
            total += count;
         }
      }

      bm_printf(pio, "total pattern bits:%u, max:%u, high:%d, low:%d\n",
         total, max, hi, lo);
      total = 0;
      max = 0;
      used = 0;
      for(i = 0; i < HASH_PATTERN_ENTRIES; i++)
      {
         struct pattern_entry * pep;

         count = 0;
         pep = pat_table[i];
         if(pep)
            used++;
         while(pep)
         {
            count++;
            pep = pep->next;
         }
         if(count > max)
         {
            max = count;
            maxp = pat_table[i];
         }
         total += count;
      }
      bm_printf(pio, "signatures:%d, max per bucket:%d\n", total, max);
      bm_printf(pio, "used %ld  buckets out of %d, with %ld duplicates.\n",
         used, HASH_PATTERN_ENTRIES, pattern_dups);
      break;
    }
   }

   if(total == 0)
      bm_printf(pio, "No items in hash table\n");
   else
      bm_printf(pio, "\n");

   return 0;
}


int
br_state(void * pio, char * args)
{
   int iface = 0;
   char  date[DB_MAXDTL];
   struct comm_dbud * comm;

   bm_printf(pio, "Network Event flags: 0x%x  Mode: %s\n", 
             (unsigned)brstates[iface].evflags, br_modenames[br_mode] );

   for(comm = br_communities; comm; comm = comm->next)
   {
      br_writedatetime(comm->cm_update, date, sizeof(date) );
      bm_printf(pio, "Community %s Last updated: %s\n", 
         comm->cm_url, date ); 
   }

   if(brstates[iface].br_router)
   {
      bm_printf(pio, "Router: %u.%u.%u.%u   ", 
         PASS_IPADDR(brstates[iface].br_router->iphost->ipaddr));
   }
   else
   {
      bm_printf(pio, "Router: unknown   "); 
   }
   bm_printf(pio, "DNS[0]: %u.%u.%u.%u   ", 
      PASS_IPADDR(dns_servers[0]));
   bm_printf(pio, "DNS[1]: %u.%u.%u.%u\n", 
      PASS_IPADDR(dns_servers[1]));

   return 0;
}


int
br_remotelist(void * pio, char * args)
{
   int      i;
   int      count;
   struct ip_hostentry * ip_host;

   count = 0;

   for(i = 0; i < HASH_IPHOSTS; i++)
   {
      ip_host = ip_hosttab[i];
      while(ip_host)
      {
         count++;
         if(count == 1)
         bm_printf(pio, "name\tIP address;\tconns;\tlastpkt;\tlastconn;\n");
         bm_printf(pio, "%s\n", br_getname(ip_host) );
         bm_printf(pio, "%u.%u.%u.%u\t", PASS_IPADDR(ip_host->ipaddr));
         bm_printf(pio, "%lu,\t%lu,\t%lu\n", 
            ip_host->conns, ip_host->tm_lastpkt, ip_host->tm_lastconn);
         ip_host = ip_host->next;
      }
   }
   if(count == 0)
      bm_printf(pio, "No remote machines detected\n"); 

   return 0;
}


int
br_rxtoggle(void * pio, char * args)
{
   brstates[br_ifview].rx_disabled = !brstates[br_ifview].rx_disabled;
   bm_printf(pio, "receiver %sabled\n",
      brstates[br_ifview].rx_disabled?"dis":"en"); 

   return 0;
}

/* br_vtest() - test pattern seach logic */

#ifdef VIRUS_CHECK
u_char  virus1[] = {
  /* complete signature image starts here */
   0x40, 0xcd, 0x21, 0xb8, 0x00, 0x42, 0x33, 0xc9,
   0x99, 0xcd, 0x21, 0x8b, 0xd6, 0xb9, 0x03, 0x00,
   0xb4, 0x40, 0xcd, 0x21, 0x8b, 0x4c, 0x19, 0x8b,
   0x54, 0x1b, 0xb8, 0x01, 0x57, 0xcd, 0x21, 0xb4,
   0x3e, 0xcd, 0x21, 0x32, 0xed, 0x8a, 0x4c, 0x18,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
   };

u_char  virus2[] = {
   0x99, 0xcd, 0x11, 0x22, 0x33, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
  /* complete signature image starts here */
   0x40, 0xcd, 0x21, 0xb8, 0x00, 0x42, 0x33, 0xc9,
   0x99, 0xcd, 0x21, 0x8b, 0xd6, 0xb9, 0x03, 0x00,
   0xb4, 0x40, 0xcd, 0x21, 0x8b, 0x4c, 0x19, 0x8b,
   0x54, 0x1b, 0xb8, 0x01, 0x57, 0xcd, 0x21, 0xb4,
   0x3e, 0xcd, 0x21, 0x32, 0xed, 0x8a, 0x4c, 0x18,
   0x99, 0xcd, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   };

u_char  virus3[] = {
   0x99, 0xcd, 0x21, 0x8b, 0xd6, 0xb9, 0x03, 0x00,
   0xb4, 0x40, 0xcd, 0x21, 0x8b, 0x4c, 0x19, 0x8b,
   0x54, 0x1b, 0xb8, 0x01, 0x57, 0xcd, 0x21, 0xb4,
   0x3e, 0xcd, 0x21, 0x32, 0xed, 0x8a, 0x4c, 0x18,
   0xFF, 0x00, 0x11, 0x22, 0x33, 0x44, 0x55, 0x66,
   0x21, 0x25, 0xcd, 0x21, 0x8c, 0xc8, 0x8e, 0xd8,
   0x8e, 0xc0, 0x58, 0xbb, 0x00, 0x01, 0x53, 0xc3, 
   0x42, 0xcd, 0x21, 0x72, 0x0a, 0xba, 0x0d, 0xcd, 
  /* "wrapped" complete signature image starts here */
  /* this one should fail */
   0x40, 0xcd, 0x21, 0xb8, 0x00, 0x42, 0x33, 0xc9,
   };


int
br_vtest(void * pio, char * args)
{
   char *   option;
   struct   pattern_entry * pep;
   struct endpoint ept;    /* for pass to patscan */

   
   option = args;
   if(!*option || (strlen(option) != 1))
   {
      bm_printf(pio, "usage: vtest [1|2|3|etc., ]\n");
      return -1;
   }

   ept.pdlen = PATTERN_LENGTH;

   if(*option == '1')
   {
      /* To test situation where sig is split over two packets, 
       * we copy first fews bytes into the previous packet 
       * buffer 
       */
      MEMCPY(&ept.pdata[5], virus1, PATTERN_LENGTH - 5);
      pep = br_patscan(virus1 + 4, sizeof(virus1) - 4, &ept);
   }
   else if(*option == '2')
      pep = br_patscan(virus2, sizeof(virus2), &ept);
   else if(*option == '3')
      pep = br_patscan(virus3, sizeof(virus3), &ept);
   else
   {
      bm_printf(pio, "usage: vtest [1|2|3|etc., ]\n");
      return -1;
   }

   if(pep)
   {
      struct virussig * vp;

      vp = (struct virussig *)pep->data;
      bm_printf(pio, "Found virus \"%s\".\n", vp->name);
   }
   else
      bm_printf(pio, "Found no virus.\n");

   return 0;
}




int
br_vspeed(void * pio, char * args)
{
   char *   op1;
   char *   op2;
   int      bcount, bsize;
   int      i;
   u_char   block[2049];
   struct   pattern_entry * pep;
   struct endpoint ept;    /* for pass to patscan */

   op1 = args;
   op2 = nextarg(op1);

   bsize = atoi(op1);
   bcount = atoi(op2);
   
   if((bsize <= 0) || (bcount <= 0))
   {
      bm_printf(pio, "usage: vspeed block_size block_number\n");
      return -1;
   }
   if(bsize >= sizeof(block))
   {
       bm_printf(pio, "block_size limited to %d\n", sizeof(block) - 1 );
       return -1;
   }

   for(i = 0; i < bcount; i++)
   {
      pep = br_patscan(block, bsize, &ept);
      if(pep)
      {
         bm_printf(pio, "vspeed: whoops, found a virus\n");
         return -1;
      }
   }
   bm_printf(pio, "vspeed: scanned %d x %d (%lu)\n", 
      bcount, bsize, (u_long)bcount * (u_long)bsize );

   return 0;
}

#endif  /* VIRUS_CHECK */


/* cip_list() - printer ("helper") routine for br_listcp to 
 * pass the bt_depthfirst() 
 */

void
cip_list(struct btnode * n, long param, int depth)
{
   struct control_ip * cip;
   ip_addr  ipaddr;

   cip = (struct control_ip *)n->data;
   ipaddr  = htonl(cip->ci_ipaddr);

   /* indent the display line by the depth */
   while(depth--)
      bm_printf((void*)param, " ");

   /* print the IP address range */
   bm_printf((void*)param, "%u.%u.%u.%u", PASS_IPADDR(ipaddr) );

   /* print any DNS association */
   if(cip->ci_name)
      bm_printf((void*)param, "     (%s)\n", cip->ci_name->cu_name );
   else
      bm_printf((void*)param, "\n");

   return;
}

/* br_listcip() - Menu entry point to list restricted IP address tree */

int
br_listcip(void * pio, char * args)
{
   bm_printf(pio, "List of restricted IP addresses\n");
   /* Pass printer routine to the btree function to recurse the tree */
   bt_depthfirst(ip_tree, cip_list, (long)pio, 0);
   return 0;
}


/*  br_getname(iphost); - return pointer to the name we should display
 * for an IP host entry.
 */

char * noname = "(no name)";

char *
br_getname(struct ip_hostentry * iphost)
{
   char * name;

   name = noname;       /* set default return */

   if(!iphost)
      return name;

   /* If the IP host points to an associated MAC, and the 
    * MAC has a name, and the the MAC is not associated 
    * with another IP host, then return the MACs name. 
    */
   if((iphost->mac_host) && 
      (iphost->mac_host->locname[0]) && 
      (iphost->mac_host->iphost == iphost))
   {
      return &iphost->mac_host->locname[0];
   }

   if(iphost->dnsname[0])
      name = &iphost->dnsname[0];
   else if(iphost->ipaddr == nets[br_ifview]->n_router)
      name = "router";
   else if(iphost->ipaddr == nets[br_ifview]->n_ipaddr)
      name = &systemname[0];

   return name;
}

char * 
br_getmacname(struct host_entry * machost)
{
   if(!machost)
   {
      dtrap();    /* sloppy programing */
      return noname;
   }

   if(machost->locname[0])
      return &machost->locname[0];

   if(machost->iphost)
      return (br_getname(machost->iphost));

   return noname;
}

char *
br_getipname(ip_addr ipaddr)
{
   struct ip_hostentry * iphost;

   if(ipaddr == htonl(0x7F000001))
      return &systemname[0];

   iphost = br_ipanyif(ipaddr);
   if(iphost)
      return br_getname(iphost);
   else
      return "(unknown)";
}


#undef TESTHOSTALARM 
//#define TESTHOSTALARM 1

static char * br_testalarm = 
"Big long string to test alarms display of strings without "
"embedded linefeeds, on any other text formatting.";

int 
br_atest(void * pio, char * args)
{
#ifdef TESTHOSTALARM
   u_char   macaddr[] = { 0x00, 0x02, 0x2d, 0xb2, 0x17, 0x2f };
   struct host_entry * host;
   host = br_hostlookup((char*)macaddr);
   if(host)
   {
      alarm_general(&newmac, NULL, host);
      bm_printf(pio, "Sent test \"host\" alarm\n");
   }
#else
   alarm_general(&alarm_text, NULL, br_testalarm);
   bm_printf(pio, "Sent test \"text\" alarm\n");
#endif
   return 0;
}

int
br_logtest(void * pio, char * args)
{
   br_log(0, "log entry from \"logtest\" menu command, counting: %x %u %d %x %u %x %u %d\n",
      1, 2, 3, 4, 5, 6, 7, 8);
   bm_printf(pio, "Made test log entry\n");
   return 0;
}

int
br_threads(void * pio, char * args)
{
   br_portthreads(pio, args);
   return 0;
}



int
br_readdb(void * pio, char * args)
{
   char *   filename;
   int   err;
   struct comm_dbud * comm;

   filename = args;
   if(!filename || *filename == 0)
   {
      dprintf("please specifiy name of DB file to read\n");
      return -1;
   }

   for(comm = br_communities; comm; comm = comm->next)
   {
      if(stricmp(comm->cm_filename, filename) == 0)
         break;
   }
   if(!comm)
   {
      dprintf("DB file name must belong to a database space. Choices are:\n");
      for(comm = br_communities; comm; comm = comm->next)
         dprintf("   %s\n", comm->cm_filename);
      return -1;
   }
   err = br_dbread(comm);
   if(err)
      bm_printf(pio, "Database file error %d.\n", err);
   else
      bm_printf(pio, "Database file done.\n");

   return err;
}

int
br_writedb(void * pio, char * args)
{
   int err;
   struct comm_dbud * comm;

   for(comm = br_communities; comm; comm = comm->next)
   {
      if(comm->cm_filename[0] == 0)
         continue;
      err = br_dbwrite(comm);
      if(err)
         bm_printf(pio, "Error %d writing database file %s.\n", 
            err, comm->cm_filename);
      else
         bm_printf(pio, "Wrote database file %s.\n", comm->cm_filename);
   }

   return 0;
}

/* br_senddbq() manually send a database update request */

int
br_senddbq(void * pio, char * args)
{
   char date[60];
   char *   cp;
   struct comm_dbud * comm;

   cp = args;

   comm = br_cmbyfile(cp);
   if(!comm)
   {
      dprintf("Bad community file name %s\n", cp);
      return ENP_PARAM;
   }

   br_writedatetime(comm->cm_update, date, sizeof(date) );

   cp = br_cmupdate(comm);
   if(cp)
      bm_printf(pio, "Error %s sending DB request\n", cp);
   else
      bm_printf(pio, "Sent DB request to %s, last UD: %s\n",
         comm->cm_url, date );

   return !(cp==0);
}

/* br_setbud() - set new community in system */

int
br_setbud(void * pio, char * args)
{
   char *   url;
   char *   name;
   struct comm_dbud * comm;
   int      priority;

   url = args;
   if(!url || (*url == 0))
   {
      bm_printf(pio, "Please enter URL & name for community server\n");
      return -1;
   }
   name = nextarg(url);
   if(!url || (*url == 0))
   {
      bm_printf(pio, "Please enter URL & name for community server\n");
      return -1;
   }
   *(name - 1) = 0;  /* Null terminate url */

   priority = (cticks & 900) + 1;   /* Make up a random but low priority */
   comm = br_cmnew(url, name, priority);
   if(!comm)
   {
      dprintf("Community create error\n");
      return -1;
   }
   

   return 0;
}


/* br_dbstate() - show state of community DB update variables */

int
br_dbstate(void * pio, char * args)
{
   int      i;
   char     date[DB_MAXDTL];
   struct comm_dbud * comm;

   for(comm = br_communities; comm; comm = comm->next)
   {
      bm_printf(pio, "Update status for file %s\n", comm->cm_filename);

      i = br_writedatetime(comm->cm_update, date, sizeof(date));
      if(i)
      {
         bm_printf(pio, "community never updated.\n");
      }
      else
         bm_printf(pio, "last full update: %s\n", date);

      bm_printf(pio, "version of last:  %lu\n", comm->cm_revision);
      bm_printf(pio, "Seconds to next:  %lu\n", comm->cm_interval);
   }
   return 0;
}



/* br_sendhello() - send interbuster hello packet */
int
br_sendhello(void * pio, char * args)  
{
   int   err;
   
   /* send out the hello bustergram packet */
   err = br_sendmsg(br_mode, 0xFFFFFFFF);      /* broadcast it */
   if(err)
      bm_printf(pio, "Error sending %d Buster HELLO message\n", err);
   else
      bm_printf(pio, "Sent Buster HELLO message\n");

   return err;
}

/* br_blist() - list Busters on local segment */

int
br_blist(void * pio, char * args)
{
   struct other_agent * obs;

   if(busterlist == NULL)
   {
      bm_printf(pio, "no Busters in list.\n");
      return 0;
   }

   for(obs = busterlist; obs; obs = obs->next)
   {
      bm_printf(pio, "MAC:%02x%02x%02x-%02x%02x%02x....",
         obs->macaddr[0],
         obs->macaddr[1],
         obs->macaddr[2],
         obs->macaddr[3],
         obs->macaddr[4],
         obs->macaddr[5] );
      bm_printf(pio, "Inst: 0x%lx....Ver: %u.%u.%u.%u.\n", 
         obs->instance, PUSH_VERSION(obs->version));
   }

   return 0;
}


extern void br_htmscan(struct conn_entry * conn, u_char * data, int dlen);

/* run a file through content filter */

int
br_content(void * pio, char * args) 
{
   char *   cp;
   FILE * fp;
   int   trysize;
   int   gotsize;
   struct conn_entry conn;
   char     readbuf[212];

   cp = args;
   if(!cp || (*cp == 0) )
   {
      bm_printf(pio, "Please specify name of file to filter\n");
      return -1;
   }

   fp = fopen(cp, "r");
   if(!fp)
   {
      bm_printf(pio, "Can't open file %s\n", cp);
      return -1;
   }

   MEMSET(&conn, 0, sizeof(conn));
   trysize = 200;
   while((gotsize = fread(readbuf, 1, trysize, fp)) > 0)
   {
      br_htmscan(&conn, (u_char*)readbuf, gotsize);
   }
   
   bm_printf(pio, "weight: %ld in %ld words\n",
      conn.ce_weight, conn.ce_wcount);

   return 0;
}

extern   int   base64_decode(char * in, int inlen, char * out, int * outlen);

char *   b64test = "AGpiYXJ0YXNAY2xlYXIybmV0LmNvbQB0YXNoYTU=";
char     b64buf[255];

/* br_b64decode(void * pio, char * args) - run a string through base64_decode */

int
br_b64decode(void * pio, char * args)
{
   int      outlen;
   int      err;
   char *   cp;

   outlen = sizeof(b64buf);

   err = base64_decode(b64test, strlen(b64test), b64buf, &outlen);

   /* Assume the result is SMTP PLAIN auth, and carve it up into 
    * printable strings. Display each string.
    */
   if(err || (outlen == 0))
   {
      bm_printf(pio, "Bad result; err:%d, len %d\n", err, outlen);
      return -1;
   }

   bm_printf(pio, "Result; ");
   cp = b64buf;
   while( cp < &b64buf[outlen] )
   {
      while(*cp <  ' ')    /* find next word */
         cp++;
      bm_printf(pio, " %s ", cp);
      while(*cp >=  ' ')   /* fins end of word */
         cp++;
   }
   bm_printf(pio, "\n");
   return 0;
}

int
ck_checknet(void * pio, char * args)
{
   int      ans;
   ans = br_checknet(0);
   if(ans == 1)
      bm_printf(pio, "br_checknet says \"Things look hazy, Ask again later\"\n");
   else
      bm_printf(pio, "br_checknet says \"%s\"\n", ans==0?"same net":"changed net");
   return 0;
}


extern u_long  br_lastresume;

/* br_hibernate() - force a fake a hibernate/wakeup cycle
 * 
 */

int
br_hibernate(void * pio, char * args)
{
   char * cp;

   cp = args;
   if(!cp || (*cp == 0) )
   {
hib_parm:
      bm_printf(pio, "Please specify LAN status: \n"
         "1 - same lan info\n"
         "2 - change router IP & MAC address\n"
         "3 - change router MAC address only\n" );
      return -1;
   }

   switch(*cp)
   {
   case '1':    /* no changes */
      break;

   case '2':    /* change the LAN router info */
      if((brstates[br_ifview].br_router) &&
         (brstates[br_ifview].br_router->iphost) &&
         (brstates[br_ifview].br_router->iphost->ipaddr))
      {
         brstates[br_ifview].br_router->iphost->ipaddr++;
         nets[br_ifview]->n_router++;
      }
      /* fall into code to change the MAC address */

   case '3':    /* change the router MAC info */
      brstates[br_ifview].br_router->macaddr[2]++;
      break;

   default:
      goto hib_parm;
   }

   /* Cob the state variables as though we've gone into
    * hibernation, and then call resume to see what it does.
    */
   br_downtime = br_gettime();   /* set downtime */
   br_lastresume = 0;   /* force br_poweresume to execute */
   br_powerresume();    /* pretend we're resuming */
   return 0;
}


/* br_traffic() - show weighted traffic stats */

int
br_traffic(void * pio, char * args)
{
   int   i;

   for(i = 0; i < (int)ifNumber; i++)
   {
      bm_printf(pio, "%lu - Traffic on net %d (%s)\n",
         nets[i]->traffic, i, nets[i]->description);

   }
   return 0;
}

/* br_exception() - test exception handling */

int
br_exception(void * pio, char * args)
{
   int x;
   bm_printf(pio, "Trapping....\n");
   x = 0;
   x = 4/x;    /* divide by zero */
   bm_printf(pio, "Oops..\n");

   return 0;
}

/* br_dopanic() - test panic handling */

int
br_dopanic(void * pio, char * args)
{
   panic("test from Buster menu");
   USE_ARG(pio);
   return 0;
}

/* br_images() - list loaded MAC images */

#ifdef    WEBIO_GUI
int
br_images(void * pio, char * args)
{
   char * cp;

   cp = br_getdir(br_imagedir);
   if(!cp)
   {
      bm_printf(pio, "No images in %s\n", br_imagedir);
      return 0;
   }

   bm_printf(pio, "%s\n", cp);

   return 0;
}
#endif  /* WEBIO_GUI */

int
br_inlist(void * pio, char * args)
{
   struct ip_hostentry * ip_host;
   int   hashval;
   int   hostct = 0;

   for(hashval = 0; hashval< HASH_IPHOSTS; hashval++)
   {
      for(ip_host = ip_hosttab[hashval]; ip_host; ip_host = ip_host->next)
      {
         if(hostct++ == 0)
            bm_printf(pio, "  IP  - conns - iface - name\n");

         bm_printf(pio, "%u.%u.%u.%u - %lu - %d - %s\n",
            PASS_IPADDR(ip_host->ipaddr),
            ip_host->conns,
            ip_host->iface,
            ip_host->dnsname);
      }
   }

   if(hostct == 0)
      bm_printf(pio, "No Internet hosts found\n");

   return 0;
}

int
br_ifstats(void * pio, char * args)
{
   char  * cp;
   int   iface = 0;

   cp = nextarg(args);
   if(cp && *cp != 0)
   {
      iface = atoi(cp);
      if((iface < 1) || (iface > (int)ifNumber))
      {
         bm_printf(pio, "Please enter iface number 1-%d\n", ifNumber);
         return -1;
      }
      iface--;
   }

   bm_printf(pio, "%s - %s (%s, %s)\n", 
             nets[iface]->name, 
             nets[iface]->description, 
             nets[iface]->ifAdminStatus==1?"up":"down", 
             nets[iface]->ifOperStatus==1?"up":"down");

   bm_printf(pio, "IP: %u.%u.%u.%u  mask: %u.%u.%u.%u  router: %u.%u.%u.%u\n", 
             PASS_IPADDR(nets[iface]->n_ipaddr),
             PASS_IPADDR(nets[iface]->n_mask),
             PASS_IPADDR(nets[iface]->n_router));

   bm_printf(pio, "\tTotal received pkts: %9lu \n", 
      nets[iface]->rxpkts);
   bm_printf(pio, "\tIP pkts:    %9lu", 
      nets[iface]->rxip);
   bm_printf(pio, "\tARP pkts:   %9lu\n", 
      nets[iface]->rxarp);
   bm_printf(pio, "\t802.3 pkts: %9lu", 
      nets[iface]->rx8023);
   bm_printf(pio, "\tFFFF type pkts: %9lu\n", 
      nets[iface]->rxFFFF);
   bm_printf(pio, "\tunknown MAC type: %9lu", 
      nets[iface]->rxbadtype);
   bm_printf(pio, "\t802.3 pkts w/unknown MAC type: %lu\n", 
      nets[iface]->rxbad8023);
   bm_printf(pio, "\treceived when iface had no IP address: %lu\n", 
      nets[iface]->rx_noip);
   bm_printf(pio, "\treceived when iface disabled: %lu\n", 
      nets[iface]->rx_disabled);
   bm_printf(pio, "\tRX pkts that triggered IP check: %lu\n", 
      nets[iface]->rx_ipcheck);
   bm_printf(pio, "\tRaw packets sent: %lu\n", 
             nets[iface]->txpkts);

   return 0;
}


/* print Buster generic system stuff */

int 
br_system(void * pio, char * params)
{
    int i;

    bm_printf(pio, "Rev: %s\n", br_version );
    bm_printf(pio, "Up for %lu seconds (%lu ticks)\n", cticks/TPS, cticks);
    bm_printf(pio, "Interfaces: (ifAdminStatus/ifOperStatus) \n");
    for(i = 0; i < (int)ifNumber; i++)
    {
        if( !nets[i] || !nets[i]->name[0] )
            break;
        bm_printf(pio, "%s - %s (%s, %s)\n", 
                  nets[i]->name, 
                  nets[i]->description, 
                  nets[i]->ifAdminStatus==1?"up":"down", 
                  nets[i]->ifOperStatus==1?"up":"down");
    }
    return 0;
}

/* trap to debugger vi dtrap */
int br_dtrap(void * pio, char * params)
{
    USE_ARG(pio);
    USE_ARG(params);
    dtrap();
    return 0;
}


/* br_sendarp() - send gratuitous arp */

int 
br_sendarp(void * pio, char * params)
{
    char *      errmsg;
    ip_addr     arptarget;

    errmsg = br_checkip(&arptarget, params);
    if(errmsg)
    {
        bm_printf(pio, "IP addr format error: %s\n", errmsg);
        return -1;
    }
    if(send_arp_net(nets[br_ifview], arptarget) == NULL)
    {
        bm_printf(pio, "Error sending ARP request\n");
        return -1;        
    }
    bm_printf(pio, "ARP sent OK, get result with dumparp\n");
    return 0;
}


/* br_dumparp() - Dump local (not system) arp table */

int 
br_dumparp(void * pio, char * params)
{
    int     i;
    int     entries = 0;
    struct arpentry *   entry;

    for(i = 0; i < MAXARPS; i++)
    {
        for(entry = arp_table[i]; entry; entry = entry->next)
        {
            if(entry->ipaddr == 0)
                continue;

            entries++;
            if(entries == 1)    /* On first entry, print header line */
                bm_printf(pio, "  IP            MAC              pending  age  idle\n");

            bm_printf(pio, "%u.%u.%u.%u   %s    %c   %lu sec.   %lu sec.\n",
                    PASS_IPADDR(entry->ipaddr),
                    print_mac(entry->macaddr),
                    entry->pending?'y':'n',
                    (cticks - entry->lasttime)/TPS,
                    (cticks - entry->createtime)/TPS );
        }
    }
    if(entries == 0)
        bm_printf(pio, "No entries in table yet\n");
    return 0;
}

/* Send ARPs to local IP adresses */
int br_arpscan(void * pio, char * params)
{
    int err;

    err = arp_scan( br_ifview );
    if(err)
        bm_printf(pio, "Error %d starting scan.\n", err);
    else
        bm_printf(pio, "Scanning net %d, use \"dumparp\" for results .\n", br_ifview);

    return err;
}


/* dump pkt queue stats */

int br_queues(void * pio, char * params) 
{
   
    bm_printf(pio, "Empty pkts; cur:%u, max:%u, min:%u\n", 
       emptybufq.q_len, emptybufq.q_max, emptybufq.q_min );
    bm_printf(pio, "Small pkts; cur:%u, max:%u, min:%u\n", 
       smallbufq.q_len, smallbufq.q_max, smallbufq.q_min );
    bm_printf(pio, "Large pkts; cur:%u, max:%u, min:%u\n", 
       largebufq.q_len, largebufq.q_max, largebufq.q_min );

   return 0;
}

/*
Copyright (C) 2003-2008 by Praemio Inc. All rights reserved.

Author: John Bartas - jbartas@praemio.com

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


