/*
 * brnet.h
 *
 * Copyright 2003-2008 by Praemio Inc. All rights reserved.
 * Author: John Bartas - jbartas@praemio.com
 * This software is released under the terms of the LGPL.
 * See LGPL Notice at end of this file; Full license in ../license.txt
 *
 * Agent network interface definitions
 *
 */


#ifndef _BRNET_H_
#define  _BRNET_H_  1


struct netiface;    /* predecl */

#define ip_addr unsigned long

struct pktbuffer
{
    struct pktbuffer * next;    /* queue link */
    char   * pk_buff;       /* Start of actual buffer */
    unsigned pk_blen;       /* Size of actual buffer (0 if external) */
    char   * pk_prot;       /* Start of Interesting data */
    unsigned pk_plen;       /* Start of data */
    long     pk_tstamp;
    int      pk_flags;      /* PF_ bits mask */
    unshort  pk_type;
    struct netiface * pk_net;
};

typedef struct pktbuffer * PACKET;

/* pk_flags bits */
#define PF_EMPTY    0x01        /* Packet alloced w/o pk_buff */
#define PF_FREE     0x02        /* in a "free" queue */



PACKET   pk_alloc(unsigned size);
void     pk_free(PACKET);
int      pk_init(void);

/* count & sizes for the free buffer queues */
extern   unsigned   smallbufs;      /* number of small buffers */
extern   unsigned   smallbufsize;   /* size of small buffers */
extern   unsigned   largebufs;      /* number of big buffers */
extern   unsigned   largebufsize;   /* size of big buffers */


/* One of these is layered over each pcap device for portability */

struct netiface
{
    char  name[12];             /* device ID name (eg "eth0") */
    char  description[128];     /* something more verbose */
 
    ip_addr  n_ipaddr;          /* Primary IP address */
    ip_addr  n_mask;            /* Mask, if known */
    ip_addr  n_router;          /* Primary router, if known */
    ip_addr  n_netbr;           /* subnet broadcast address */

    int     (*n_init)(int);     /* iface startup routine */
    int     (*n_send)(PACKET);  /* packet send routine */
    int     (*n_close)(int);    /* iface close routine */
    void    (*n_stats)(void * pio, int iface);   
    u_long  traffic;            /* weighted level of recent traffic */

    int     ifOperStatus;       /* RFC1213 variable */
    int     ifAdminStatus;      /* RFC1213 variable */
    u_long  ifLastChange;       /* RFC1213 - last status chage */
    char *  n_macaddr;
    void *  n_local;
    int     n_flags;

    u_long  txbytes;    /* total bytes sent */
    u_long  txpkts;     /* total pkts sent */
    u_long  txerrors;   /* Hard send errors */
    u_long  rxbytes;    /* total bytes received */
    u_long  rxpkts;     /* total pkts received */
    u_long  rxerrors;   /* Hard receive errors */
    u_long  discards;   /* dropped, no buffer */

    /* received packet details */
    u_long      rxip;       /* IP pkts */
    u_long      rxarp;      /* ARP pkts */
    u_long      rx8023;     /* 802.3 pkts */
    u_long      rxFFFF;     /* FFFF type pkts */
    u_long      rxbadtype;  /* unknown MAC type pkts */
    u_long      rxbad8023;  /* 802.3 pkts w/unknown MAC type */
    u_long      rx_noip;       /* received when iface had no IP address */
    u_long      rx_disabled;   /* received when iface disabled */
    u_long      rx_ipcheck;  /* RX pkts that triggered IP check */
};

typedef  struct netiface * NET;

extern unsigned   ifNumber;      /* how many ifaces in system */

/* n_flags bits */
#define NF_DHCPC    0x01         /* IP was set via DHCP */
#define NF_NOPRO    0x02         /* Driver doesn't do promisuous mode */

extern  int if_netnumber(NET);   /* get index for pointer */

extern   NET   nets[MAXNETS];    /* pointers to the static network structs */
extern  struct netiface staticnets[MAXNETS];    /* the actual structs */

struct eth_hdr
{
    u_char  e_dst[6];
    u_char  e_src[6];
    u_short e_type;
};

struct snap_hdr
{
    u_char   llc_etc[6];
    unshort  type;    
};

#define  net_num(net)   if_netnumber(net)

typedef struct   q_ptr    {
    struct   q_ptr   *   qe_next; /* it's just a pointer to next elt */
} q_ptr;

typedef q_ptr * qp;    /* pointer to a queue element */

typedef   struct   queue   
{        /* queue header */
    qp    q_head;       /* first element in queue */
    qp    q_tail;       /* last element in queue */
    int   q_len;        /* number of elements in queue */
    int   q_max;        /* maximum length */
    int   q_min;        /* minimum length */
    void * mutex;       /* Thread/interrupt protection */
} brqueue;

void        putq(brqueue*, void*);    /* add item to tail of queue */
void *      getq(brqueue*);           /* de-queue item from head of queue */
brqueue *   createq(void);

extern   brqueue    largebufq;      /* Full size packet buffers */
extern   brqueue    smallbufq;      /* small packet buffers */
extern   brqueue    emptybufq;      /* empty buffers, for RX */

/* Over head for IP and lower (Ethernet) headers. This is rounded up
 * to word-align IP and higher headers 
 */
#define IP_OVERHEAD     40

#define  ETHERNET       6   /* SNMP type for Ethernet */


/* Macros for passing IP addresses to printfs. */
#if (BYTE_ORDER==LITTLE_ENDIAN)
#define  PASS_IPADDR(ip)\
   (unsigned)(ip & 0x000000ff),\
   (unsigned)((ip>>8) & 0x000000ff),\
   (unsigned)((ip>>16) & 0x000000ff),\
   (unsigned)(ip>>24)
#else
#error CHECK this...
#define  PASS_IPADDR(ip)\
   (unsigned)(ip>>24),\
   (unsigned)((ip>>16)&0xff),\
   (unsigned)((ip>>8)&0xff),\
   (unsigned)(ip&0xff)
#endif

#endif   /* _BRNET_H_ */

   /*
Copyright (C) 2003-2008 by Praemio Inc. All rights reserved.

Author: John Bartas - jbartas@praemio.com

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


