/*
 * brq.c
 *
 * Buster simple queueing system.
 *
 * Copyright 2003-2008 by Praemio Inc. All rights reserved.
 * Author: John Bartas - jbartas@praemio.com
 * This software is released under the terms of the LGPL.
 * See LGPL Notice at end of this file; Full license in ../license.txt
 *
 */

#include "busterport.h"
#include "brnet.h"


/* FUNCTION: putq()
 *
 * q_add(que, item) - Adds an item to tail of queue. 
 *
 */

void 
putq(brqueue  * q,        /* the queue */
       void * item)     /* item to delete */
{
    q_ptr * elt = item;

    TH_LOCK_MUTEX(q->mutex);
    elt->qe_next = 0;

    if (q->q_head == NULL) 
        q->q_head = elt;
    else
        q->q_tail->qe_next = elt;

    q->q_tail = elt;
    elt->qe_next = NULL;

    if(++q->q_len > q->q_max)
        q->q_max = q->q_len;
    TH_UNLOCK_MUTEX(q->mutex);   /* restore int state */
    return;
}


/* FUNCTION: getq()
 *
 * Dequeue and return the first element of the specified queue.  
 *
 * PARAM1: queue * q
 *
 * RETURNS: pointer to the first element if any, or 0 if the queue is empty.
 */

void * getq(brqueue * q)
{
    q_ptr * temp;

    TH_LOCK_MUTEX(q->mutex);          /* protect the queue */

    temp = q->q_head;
    if (temp == NULL)       /* queue empty? */
    {
        TH_UNLOCK_MUTEX(q->mutex);
        return (0);         /* yes, show none */
    }

    q->q_head = temp->qe_next;      /* unlink */
    temp->qe_next = 0;              /* clear pointer */
    if (q->q_head == 0)             /* queue now empty? */
        q->q_tail = 0;              /* yes, update tail pointer too */

    q->q_len--;             /* update queue length */
    if (q->q_len < q->q_min)
        q->q_min = q->q_len;

    TH_UNLOCK_MUTEX(q->mutex);

    return ((void*)temp);
}

/*
Copyright (C) 2003-2008 by Praemio Inc. All rights reserved.

Author: John Bartas - jbartas@praemio.com

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


