/*
 * FILENAME: btree.h
 *
 * Copyright 2003-2008 by Praemio Inc. All rights reserved.
 * Author: John Bartas - jbartas@praemio.com
 * This software is released under the terms of the LGPL.
 * See LGPL Notice at end of this file; Full license in ../license.txt
 *
 * Binary tree definitions
 */

#ifndef _BTREE_H_
#define _BTREE_H_ 1

#ifndef BT_ALLOC
#define BT_ALLOC(size)          npalloc(size)
#define BT_FREE(ptr)            npfree(ptr)
#endif /* BT_FREE */

/* enum values to support the route table btree software */
enum BT_SKEW {	BT_NONE,	BT_LEFT, BT_RIGHT };
enum BT_RES { TREE_ERROR = 0, TREE_OK, TREE_BALANCE };
enum BT_REL { KEY_GT, KEY_EQ, KEY_LT };

typedef  unsigned long AVLDATUM;

/* one of these structures should be the first item in any structure
 * which is to be managed in a btree.
*/ 
struct btnode 
{
   struct btnode * left;      /* btree brach to higher side of tree */
   struct btnode * right;     /* btree brach to lower side of tree */
   enum BT_SKEW skew;         /* which branch is longer */
   /* data item for this node. First field in data item MUST be a 
    * back pointer to this btnode structure. 
    */
   struct btnode ** data;     
};

enum BT_RES
bt_insert(struct btnode ** parent,
         struct btnode * new,
         u_long new_key,
         int (*cmpfunc)(struct btnode *, u_long) );


enum BT_RES
bt_remove(struct btnode **n,
         u_long key,
         int (* cmpfunc)(struct btnode *, u_long));

struct btnode *
bt_access(struct btnode * n,
      u_long key,
      int (* cmpfunc) (struct btnode *, u_long ));


void
bt_depthfirst(
   struct btnode * n,
   void(*func)(struct btnode * n, long param, int depth),
   long param,
   int depth);

#endif   /* _BTREE_H_ */


/*
Copyright (C) 2003-2008 by Praemio Inc. All rights reserved.

Author: John Bartas - jbartas@praemio.com

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


