/* busterport.h
 *
 * Agent config file for Packet Island MS device.
 *
 * Copyright 2003-2008 by Praemio Inc. All rights reserved.
 * Author: John Bartas - jbartas@praemio.com
 * This software is released under the terms of the LGPL.
 * See LGPL Notice at end of this file; Full license in ../license.txt
 */

#ifdef WIN32
/* If we're testing on Windows, go get that file and ignore this one */
#include "..\winapp\busterport.h"
#else

#define u_long unsigned long
#define u_char unsigned char
#define unshort unsigned short
#define u_short unsigned short

#include <ctype.h>
#include <stdarg.h>
#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <pthread.h>
#include <string.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>



#define MAXNETS         12
#define MAXDNSNAME      256


#define MAC_HDRLEN      14  /* Bytes ion pk_bugg before IP header */


/* Map some sockets portability stuff to Linux sockets */
#ifndef SOCKET_ERROR
#define SOCKET_ERROR    -1
#endif
#ifndef INVALID_SOCKET
#define INVALID_SOCKET    -1
#endif

#define socketclose(socket)     close(socket)
#define sock_errno(socket)      (errno)



extern u_long cticks;

#define TPS     10      // cticks per second


/* ifdeffable per-port option switches */
#define OMIT_CONNECTIONS    1   /* Connection recording option */
//#define SMTP_ALERTS       1   /* System has SMTP alert option */
//#define WEBIO_GUI         1   /* Maintain images for GUI */
//#define VIRUS_CHECK       1   /* Build does virus checking */
//#define POP3_MONITOR      1   /* Watch POP connections in detail */
//#define BLOCKING_LOG      1   /* log blocks to toolbar or window */
//#define DNS_REVLOOKUPS    1   /* Do reverse DNS name lookups */
//#define PI_SITEADVISOR    1   /* Support Site Advisor Plung-in */
//#define BR_COMMUNITIES    1   /* Get blocking DB from communities */



#define SYS_STARTUP_LAG 1   /* Doesn't matter on PI Probe */

extern void LinuxTrap(char * filename, int linenum); // embedded break point

#define dtrap() LinuxTrap(__FILE__,__LINE__)

#ifdef arch_arm
#define EMBEDDED_DEVICE
#endif


extern int disc_log(char * string, ...);

/* Map dprintf to the probes LOG file */
#define dprintf( ... )  disc_log( __VA_ARGS__ )


extern  int stristr(char *, char *);

#define stricmp(s1, s2) strcasecmp(s1, s2)
#define _strnicmp(s1,s2,len) strncasecmp(s1,s2,len)

/* thread control macros */
#define TH_PROTO(tname)         void * tname( void * parm)
#define TH_OBJECT(name)         pthread_t name
#define TH_OBJECT_PTR(name)     pthread_t * name
#define TH_LOCK_MUTEX(m)        pthread_mutex_lock((pthread_mutex_t *)(m) )
#define TH_UNLOCK_MUTEX(m)      pthread_mutex_unlock((pthread_mutex_t *)(m))
#define TH_ENTRY(func)          void * func( void * parm )
#define TH_RETURN_OK()          return(NULL)
#define TH_EXIT(name)           // define to nothing


#define KBYTE 1024
#define APP_PRIORITY    20
#define APP_STACK_SIZE  (8 * KBYTE)

struct br_thread
{
    TH_OBJECT_PTR(th_ptr);  /* pointer to static task object */
    char *  name;            /* name of task */
    void *  (*entry)(void*);   /* pointer to code to start task at */
    int     priority;         /* priority of task */
    int     stacksize;        /* size (bytes) of task's stack */

    /* Port specific members follow */
    TH_OBJECT( thread );

    /* state transition counters */
    u_long   blocks;
    u_long   sleeps;
    u_long   wakes;
    u_long   yields;

    void *   parameter;     /* Parameter to new thread */
};

extern  void    TH_SLEEP( u_long ticks );
extern  void    TH_WAKE( long taskId );
extern  int     TH_NEWTASK(struct br_thread *);

#define MAXTHREADS  12
extern struct br_thread * pthreads[MAXTHREADS];


/* Heap routines for fixed-size discovery objects */
#define CN_ALLOC(size)          calloc(1, size)
#define CN_FREE(ptr)            free(ptr)
#define HOST_ALLOC( size)       calloc(1, size)
#define HOST_FREE(ptr)          free(ptr)
#define IPH_ALLOC(size)         calloc(1, size)
#define IPH_FREE(ptr)           free(ptr)
#define CP_ALLOC(size)          calloc(1, size)
#define CP_FREE(ptr)            free(ptr)
#define PEP_ALLOC(size)         calloc(1, size)
#define PEP_FREE(ptr)           free(ptr)
#define DN_ALLOC(size)          calloc(1, size)
#define DN_FREE(ptr)            free(ptr)
#define BTN_ALLOC(size)         calloc(1, size)
#define BTN_FREE(ptr)           free(ptr)
#define POPC_ALLOC(size)        calloc(1, size)
#define POPC_FREE(ptr)          free(ptr)
#define AL_ALLOC(size)          calloc(1, size)
#define AL_FREE(ptr)            free(ptr)
#define BT_ALLOC(size)          calloc(1, size)
#define BT_FREE(ptr)            free(ptr)
#define PL_ALLOC(size)          calloc(1, size)
#define PL_FREE(ptr)            free(ptr)
#define SA_ALLOC(size)          calloc(1, size)
#define SA_FREE(ptr)            free(ptr)
#define IMG_ALLOC(size)         calloc(1, size)
#define IMG_FREE(ptr)           free(ptr)
#define IPE_ALLOC(size)         calloc(1, size)
#define IPE_FREE(ptr)           free(ptr)
#define PB_ALLOC(size)          calloc(1, size)
#define PB_FREE(ptr)            free(ptr)
#define WD_ALLOC(size)          calloc(1, size)
#define WD_FREE(ptr)            free(ptr)
#define LAN_ALLOC(size)         calloc(1, size)
#define LAN_FREE(ptr)           free(ptr)
#define CD_ALLOC(size)          calloc(1, size)
#define CD_FREE(ptr)            free(ptr)
#define OBP_ALLOC(size)         calloc(1, size)
#define OBP_FREE(ptr)           free(ptr)
#define PK_ALLOC(size)          calloc(1, size)
#define PK_FREE(ptr)            free(ptr)
#define PKS_ALLOC(size)         calloc(1, size)
#define PKS_FREE(ptr)           free(ptr)
#define PKL_ALLOC(size)         calloc(1, size)
#define PKL_FREE(ptr)           free(ptr)
#define AT_ALLOC(size)          calloc(1, size)
#define AT_FREE(ptr)            free(ptr)


/* Heap routines for variable size memory blocks. The comments include 
 * approximate requirements for max size 
 */
#define LM_ALLOC(size)          calloc(1, size) // Alert message - 1024 should be OK
#define LM_FREE(ptr)            free(ptr)
#define DNM_ALLOC(size)         calloc(1, size) // DNS name - 256 max
#define DNM_FREE(ptr)           free(ptr)
#define CTD_ALLOC(size)         calloc(1, size) // Control description - 128 max?
#define CTD_FREE(ptr)           free(ptr)
#define CTT_ALLOC(size)         calloc(1, size) // Control Tags - 256 max?
#define CTT_FREE(ptr)           free(ptr)
#define PTH_ALLOC(size)         calloc(1, size) // Blocked path - 256 max?
#define PTH_FREE(ptr)           free(ptr)


/* Send menu print to stdout for now */
#define bm_printf    fprintf


#define MEMCPY(dest, src, len)  memcpy(dest, src, len)
#define MEMCMP(dest, src, len)  memcmp(dest, src, len)
#define MEMSET(dest, val, size) memset(dest, val, size)
#define MEMMOVE(dest, src, len) memmove(dest, src, len)

#ifndef TRUE
#define TRUE -1
#endif
#ifndef FALSE
#define FALSE 0
#endif

#define USE_ARG(foo) (foo=foo)

#ifndef min
#define min(a,b)    (a>b?a:b)
#endif


#endif /* WIN32 */

/*
Copyright (C) 2003-2008 by Praemio Inc. All rights reserved.

Author: John Bartas - jbartas@praemio.com

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/



