/*
 * dhcpdef.h
 *
 * Copyright 2003-2008 by Praemio Inc. All rights reserved.
 * Author: John Bartas - jbartas@praemio.com
 * This software is released under the terms of the LGPL.
 * See LGPL Notice at end of this file; Full license in ../license.txt
 */



#ifndef _DHCPDEF_H
#define  _DHCPDEF_H 1


#define  BOOTP_SERVER_PORT    67
#define  BOOTP_CLIENT_PORT    68

/* DHCP packet type codes */
#define  DHCP_INVALIDOP 99
#define  DHCP_DISCOVER  1
#define  DHCP_OFFER     2
#define  DHCP_REQUEST   3
#define  DHCP_DECLINE   4
#define  DHCP_ACK       5
#define  DHCP_NAK       6
#define  DHCP_RELEASE   7

/* DHCP options  */

#define  DHOP_PAD       0     /* single char of padding, always 0 */
#define  DHOP_SNMASK    1     /* client's subnet mask */
#define  DHOP_ROUTER    3     /* set default router */
#define  DHOP_TIMES     4     /* List of time servers */
#define  DHOP_DNSRV     6     /* IP address of DNS server */
#define  DHOP_NAME      12    /* host name */
#define  DHOP_DUMPF     14    /* Coredump file */
#define  DHOP_DOMAIN    15    /* host's domain */
#define  DHOP_CADDR     50    /*  requested IP address */
#define  DHOP_LEASE     51    /* lease time */
#define  DHOP_TYPE      53    /* DHCP type  */
#define  DHOP_SERVER    54    /* server IP address */
#define  DHOP_REQLIST   55    /* client's parameter  list */
#define  DHOP_MSG       56    /* message explaining NAK  */
#define  DHOP_MAXSIZE   57    /* maximum DHCP message size */
#define  DHOP_CLASSID   60    /* Class of client */
#define  DHOP_CLIENT    61    /* client ID (i.e. hardware address) */
#define  DHOP_USERCLASS 77    /* User opaque data */
#define  DHOP_DNSNAME   81    /* Client Fully Qualified Domain Name */
#define  DHOP_END       255   /* marks end of valid options */



#define  BOOTREQUEST    1     /* Old bootp op codes */
#define  BOOTREPLY      2

/* DHCP hardware type */
#define  ETHHWTYPE      1     /* ethernet hardware type */

#define  RFC1084_MAGIC_COOKIE htonl(0x63825363)
#define  RFC1084_PAD          ((unsigned  char) 0)
#define  RFC1084_SUBNET_MASK  ((unsigned  char) 1)
#define  RFC1084_GATEWAY      ((unsigned  char) 3)
#define  RFC1084_END          ((unsigned  char) 255)


#define  DHCP_OPTSIZE   312   /* Size of options field */

/* The bootp/DHCP header */
struct bootp   {
   unsigned char     op;
   unsigned char     htype;
   unsigned char     hlen;
   unsigned char     hops;
   unsigned long     xid;
   unsigned short    secs;
   unsigned short    flags;
   unsigned long     ciaddr;
   unsigned long     yiaddr;
   unsigned long     siaddr;
   unsigned long     giaddr;
   unsigned char     chaddr[16];
   unsigned char     sname[64];
   unsigned char     file[128];
   unsigned char     options[DHCP_OPTSIZE];
};

/* Extract IP address in NET endian */
#define EXTRACT_IP_ADDR(data, ipaddr) \
{   data++;   \
    ipaddr = *(ip_addr*)(data);   \
}


u_char * find_opt(u_char opcode, u_char * opts);    /* get option from packet */

#endif   /* _DHCPDEF_H */

/*
Copyright (C) 2003-2008 by Praemio Inc. All rights reserved.

Author: John Bartas - jbartas@praemio.com

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


