/*
 * ip.h
 *
 * Copyright 2003-2008 by Praemio Inc. All rights reserved.
 * Author: John Bartas - jbartas@praemio.com
 * This software is released under the terms of the LGPL.
 * See LGPL Notice at end of this file; Full license in ../license.txt
 *
 * Portions Copyright 1986 by Carnegie Mellon 
 * Portions Copyright 1983 by the Massachusetts Institute of Technology.
 *
 * IP protocol definitions
 *
 */

#ifndef _IPHFILE_H_
#define  _IPHFILE_H_ 1

#define ip_addr unsigned long

/* Plain old IPv4  packet header overlay */
struct ip   
{
   u_char   ip_ver_ihl;    /* 4 bit version, 4 bit hdr len in 32bit words */
   u_char   ip_tos;        /* Type of Service */
   unshort  ip_len;        /* Total packet length including header */
   unshort  ip_id;         /* ID for fragmentation */
   unshort  ip_flgs_foff;  /* mask in flags as needed */
   u_char   ip_time;       /* Time to live (secs) */
   u_char   ip_prot;       /* protocol */
   unshort  ip_chksum;     /* Header checksum */
   ip_addr  ip_src;        /* Source name */
   ip_addr  ip_dest;       /* Destination name */
};

extern  u_short cksum(void * buffer, unsigned wordct);

extern void mk_iphdr( PACKET pkt, unsigned pid, u_char prot, unshort fragword, int len);

/* Some useful definitions */

#define  IPHSIZ   sizeof(struct  ip)   /* internet header size */
#define  IP_VER   4     /* internet version */
#define  IP_TSRV  0     /* default type of service */
#define  IP_ID    0     /* kernel fills in IN id */
#define  IP_FLGS  0     /* no fragmentation yet */

/* fragmentation flag bits, for masking into 16bit flags/offset word */
#define  IP_FLG_DF   0x4000   /* Don't   Fragment (DF) bit */
#define  IP_FLG_MF   0x2000   /* More Fragments (MF) bit */
#define  IP_FLG_MASK 0xe000   /* for masking out all flags from word */


#ifndef  IP_TTL         /* maximum time to live */
#define  IP_TTL   64    /* overridable from system headers */
#endif   

/* ethernet-ish packet types in NET endian: */
#define  IP_TYPE     htons(0x0800)
#define  ARP_TYPE    htons(0x0806)

/* Some macros for finding IP offsets in incoming packets */
#define  ip_header(pkt)     (struct  ip *)(pkt->pk_prot)

#define  ip_hlen(pip)      (((pip)->ip_ver_ihl  &  0x0f) << 2)
#define  ip_data(pip)      ((char *)(pip) +  ip_hlen(pip))
#define  ip_optlen(pip)    (ip_hlen(pip)  -  20)


/* Host endian values for IP address class & subnet manipulation */
#define  AMASK    htonl(0x80000000)
#define  AADDR    htonl(0x00000000)
#define  BMASK    htonl(0xC0000000)
#define  BADDR    htonl(0x80000000)
#define  CMASK    htonl(0xE0000000)
#define  CADDR    htonl(0xC0000000)
#define  IPLBA    htonl(0x7F000000) /* IP LoopBack net Address */

extern  char * print_ipad( ip_addr);


/* UDP Header structure */

struct udp
{
    unshort  ud_srcp;    /* source port */
    unshort  ud_dstp;    /* dest port */
    unshort  ud_len;     /* length of UDP packet */
    unshort  ud_cksum;   /* UDP checksum */
};

#define  UDP_PROT    17 /* UDP Internet protocol number */

/* Some reserved UDP ports */
#define  DNS_PORT    53
#define  SNMP_PORT   161

/* Calls to send a whois UDP datagram. This can be mapped to sockets
 * and winsock.
 */

int   udp_socksend(ip_addr target, unshort destport, PACKET pkt);
long  udp_sockopen(ip_addr, unshort lport, unshort fport, 
                   int(*callback)(PACKET pkt, void * Id),  
                   void * Id_value );


struct tcphdr
{
    u_short  th_sport;      /* source port */
    u_short  th_dport;      /* destination port */
    u_long   th_seq;        /* sequence number */
    u_long   th_ack;        /* acknowledgement number */
    u_char   th_doff;       /* data offset: high 4 bits only */

    /* these macros get/set the raw value, usually 5 */
#define  GET_TH_OFF(th) (th.th_doff >> 4)
#define  SET_TH_OFF(th, off)  (th.th_doff =  (u_char)(off   << 4))

   u_char   th_flags;
#define     TH_FIN   0x01
#define     TH_SYN   0x02
#define     TH_RST   0x04
#define     TH_PUSH  0x08
#define     TH_ACK   0x10
#define     TH_URG   0x20
   u_short     th_win;     /* window */
   u_short     th_sum;     /* checksum */
   u_short     th_urp;     /* urgent pointer */
};


/* RFC standard TCP states */
#define     TCPS_CLOSED         0
#define     TCPS_LISTEN         1
#define     TCPS_SYN_SENT       2
#define     TCPS_SYN_RECEIVED   3
#define     TCPS_ESTABLISHED    4
#define     TCPS_CLOSE_WAIT     5
#define     TCPS_FIN_WAIT_1     6
#define     TCPS_CLOSING        7
#define     TCPS_LAST_ACK       8
#define     TCPS_FIN_WAIT_2     9
#define     TCPS_TIME_WAIT      10


/* common TCP server ports */
#define  POP3_PORT      110
#define  SMTP_PORT      25

extern   int   http_port;        /* default web server port */
#define  HTTP_PORT   http_port   /* map it to web server */

extern  unshort tcp_cksum(struct ip * pip);


struct icmp     /* ICMP header (ping-like version) */
{
    u_char      type;
    u_char      code;
    unshort     chksum;
    unshort     id;
    unshort     seq;
};

#define  ICMP_PROT    1     /* IP hdr type for ICMP packets */

/* structure of an icmp destination unreachable packet */

#define  ICMPDUDATA     8   /* size of extra data */

struct destun
{
    u_char      dtype;
    u_char      dcode;
    unshort     dchksum;
    unshort     dno1;
    unshort     dno2;
    struct ip   dip;    /* the offending IP packet */
    u_char      ddata[ICMPDUDATA];
};

#define  ICMPQDATA     64  /* size of extra data */

struct quench
{
    u_char      ptype;
    u_char      pcode;
    unshort     pchksum;
    struct ip   pip;    /* the offending IP packet */
    u_char      data[ICMPDUDATA];
};

#define  ICMPQUENCH_SIZE ( sizeof(struct quench) + ICMPQDATA )

/* structure of an icmp time exceeded packet */

struct timex
{
    u_char      ttype;
    u_char      tcode;
    unshort     tchksum;
    unshort     tno1;
    unshort     tno2;
    struct ip   tip;
    u_char      tdata[8];
};

/* ICMP packet types */
#define  ECHOREP     0     /* ICMP Echo reply */
#define  DESTIN      3     /* Destination Unreachable */
#define  SOURCEQ     4     /* Source quench */
#define  REDIR       5     /* Redirect */
#define  ECHOREQ     8     /* ICMP Echo request */
#define  TIMEX       11    /* Time exceeded */
#define  PARAM       12    /* Parameter problem */
#define  TIMEREQ     13    /* Timestamp request */
#define  TIMEREP     14
#define  INFO        15    /* Information request */

/**** DNS support ***/

/* DNS header flag bits */
#define  DNSF_QR     0x8000   /* query (0), or response (1) */
#define  DNSF_OPMASK 0x7800   /* 4 bit opcode kinds of query, 0==standard */
#define  DNSF_AA     0x0400   /* set if Authoritive Answers */
#define  DNSF_TC     0x0200   /* set if truncated message */
#define  DNSF_RD     0x0100   /* Recursion Desired bit */
#define  DNSF_RA     0x0080   /* Recursion Allowed bit */
#define  DNSF_Z      0x0050   /* 3 reserved bits, must be zero */
#define  DNSF_AU     0x0020   /* answer was authenticated */
#define  DNSF_RCMASK 0x000F   /* Response Code mask */

/* DNS Reponse Code values: */
#define  DNSRC_OK       0     /* good response */
#define  DNSRC_EFORMAT  1     /* Format error */
#define  DNSRC_ESERVER  2     /* Server Error */
#define  DNSRC_ENAME    3     /* Name error */
#define  DNSRC_EIMP     4     /* Not Implemented on server */
#define  DNSRC_EREFUSE  5     /* Server refused operation */
#define  DNSRC_UNSET    0xFF  /* No reponse yet (used only in dns_querys struct) */


extern ip_addr   br_dnslookup(char * hostname);


#endif   /* _IPHFILE_H_ */

/*
Copyright (C) 2003-2008 by Praemio Inc. All rights reserved.

Author: John Bartas - jbartas@praemio.com

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/

