/* brdbud.c
 *
 * Copyright 2006 by Praemio, Inc. All rights reserved
 *
 * Buster plug-in for McAfee site advisor
 *
 * Copyright 2003-2008 by Praemio Inc. All rights reserved.
 * Author: John Bartas - jbartas@praemio.com
 * This software is released under the terms of the LGPL.
 * See LGPL Notice at end of this file; Full license in ../license.txt
 *
 *
 */

#include "busterport.h"
#include "brnet.h"
#include "ip.h"
#include "brdb.h"

#ifdef PI_SITEADVISOR   // Whole file can be ifdeffed

/* Flag indicating if Buster Site Advisor plug-in is active */
int   sa_active = TRUE;

/* Flag indicating if Site Advisor "bad" sites should be blocked */
int   sa_siteblock = TRUE;


/* Plugin declarations for the Site Advisor plug-in */
void  sa_newhost(void * hostipe, struct plug_in * sapl);



/* Task structures for the Site Advisor update task */
TH_OBJECT(to_sathread);
TH_PROTO(sa_thread);

TH_PROTO(sa_update);

struct br_thread sa_task = 
{
   &to_sathread,
   "Site Advisor Plug-In",
   sa_thread,
   APP_PRIORITY,
   APP_STACK_SIZE,
};



/* Plugin init routine. This should be added to ????. It should 
 * do all the plug-in setup, including registering with the 
 * plug-in system.
 */

int
sa_init()
{
   int   err;

   /* Start the Site Advisor plug-in update thread */
   err = TH_NEWTASK(&sa_task);
   if(err)
   {
      dprintf("Can't start Site Advisor thread");
      return ENP_RESOURCE;
   }
   add_plugin(sa_newhost, PL_NEWNAME);
   return 0;
}


enum sa_state { SA_NEW, SA_REQSENT, SA_DONE };


char qstring[] = {
"GET /DSS/Query?Entitlement=FOO&Type=domain&version=2&name=%s"
"&client_ver=FF_25.0_4499&locale=en-US&aff_id=0 HTTP/1.1\r\n"
"Host: dss2.siteadvisor.com\r\n"
"User-Agent: Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.7.5) Gecko/20041107 Firefox/1.0\r\n"
"Accept: text/xml,application/xml,application/xhtml+xml,text/html;q=0.9,text/plain;q=0.8,image/png,*/*;q=0.5\r\n"
"Accept-Language: en-us,en;q=0.5\r\n"
"Accept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.7\r\n"
"Keep-Alive: 300\r\n"
"Connection: keep-alive\r\n"
"Pragma: no-cache\r\n"
"Cache-Control: no-cache\r\n\r\n"
};

  
#define  SA_INSIZE   8000


struct sa_session    /* Site Advisor Session */
{
   struct   sa_session *  next;     /* list link */
   enum     sa_state      state;
   long     sa_socket;     /* socket of active connection */
   u_long   time;          /* ctick of last state change */
   struct ip_hostentry * dnsip;
   char     outbuf[ sizeof(qstring) + MAXURL + 4];
   char     inbuf[ SA_INSIZE ];
};

struct sa_session * sa_sessions;  /* list link */

/* table of text scraps->picture mappings. Scraps are testing in 
 * table order; first match is the one used. No match loads no 
 * code.
 */

struct replycode {
   char *   matchText;  /* match this text in html reply */
   char *   imgname;    /* disply this image is found */
   int      block;      /* nonzero to block matching site */
} replycodes[] = 
{
    {"Classification code=\"UNKNOWN\"",  "qmark.gif", 0},
    {"color=\"green\"",                  "greenface.gif", 0}, 
    {"color=\"red\"",                    "redface.gif", -1},
    {"color=\"yellow\"",                 "yellowface.gif", 0},
};

#define  NUM_REPLYCODES ( sizeof(replycodes)/sizeof(struct replycode) )

ip_addr  sa_ip = 0xD88F466B; /* 216.143.70.107 - dss2.siteadvisor.com */

struct sa_session *
new_sasess(struct ip_hostentry * dnsipe)
{
   struct sa_session * newsa;

   if(!sa_active)
      return NULL;

   newsa = (struct sa_session *)SA_ALLOC(sizeof(struct sa_session));
   if(newsa == NULL)
      return NULL;

   newsa->time = cticks;
   newsa->dnsip = dnsipe;

   newsa->next = sa_sessions;
   sa_sessions = newsa;
   return newsa;
}


struct sa_session *
del_sasess(struct sa_session * to_del)
{
   struct sa_session * sa_sess;
   struct sa_session * last = NULL;

   for(sa_sess = sa_sessions; sa_sess; sa_sess = sa_sess->next)
   {
      if(sa_sess == to_del)
      {
         struct sa_session * next;
         if(last)
            last->next = sa_sess->next;
         else
            sa_sessions = sa_sess->next;
         next = sa_sess->next;      /* value to return */
         SA_FREE(sa_sess);
         return next;
      }
      last = sa_sess;
   }
   dtrap();    /* session to delete not in list */
   return NULL;
}

char *
br_extractDomain(char * fullname)
{
   char * cp;
   int   backup = 1; /* dots to back up */

   cp = strrchr(fullname, '.');   /* Find last dot in name */
   if(!cp)
   {
      return(fullname);   /* No dot? try anyway... */
   }

   /* if name ends in "uk" back up an extra level */
   if( (*(cp+1) == 'u') && ( *(cp+2) == 'k' ))
      backup++;

   /* else - back up to start of name before last dot */
   while(backup--)
   {
      cp--;
      while(*cp != '.' && *cp > ' ')
         cp--;
      cp++;
      if(cp < fullname)   /* don;t back up too far */
         return(fullname);
      if(backup)
         cp--;
   }
   return cp;
}


int
sa_connect(struct sa_session * newsa)
{
   int      err;
   int      length;
   char *   cp;
   struct sockaddr_in sa;

   newsa->sa_socket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);

   if(newsa->sa_socket == INVALID_SOCKET)
   {
      err = sock_errno(newsa->sa_socket);
      dprintf("sa_connect: Unable to get socket, error code %d\n",
         err);
      return ENP_RESOURCE;
   }

   sa.sin_family = AF_INET;
   sa.sin_addr.s_addr = htonl(sa_ip);
   sa.sin_port = htons(80);

   err = connect(newsa->sa_socket, (struct sockaddr*)&sa, sizeof(sa) );
   if(err)
   {
      err = sock_errno(newsa->sa_socket);
      dprintf("sa_connect: Unable to connect, error code %d\n",
         err);
      return ENP_RESOURCE;
   }
   err = br_nbiosocket(newsa->sa_socket);

   /* Format and send Web request to site advisor */
   cp = br_extractDomain(newsa->dnsip->dnsname);
   sprintf(newsa->outbuf, qstring, cp);
   length = strlen(newsa->outbuf);
   err = send(newsa->sa_socket, newsa->outbuf, length, 0);
   if(err != length)
   {
      err = sock_errno(newsa->sa_socket);
      dprintf("sa_connect: Unable to send, error code %d\n",
         err);
      return ENP_RESOURCE;
   }

   return 0;
}


/* sa_newhost() - Plugin per-object entry point */

void
sa_newhost(void * hostipe, struct plug_in * sapl)
{
   struct sa_session * newsess;

   newsess = new_sasess( (struct ip_hostentry *)hostipe );

   TH_WAKE( (long)&to_sathread );

   USE_ARG(sapl);

   return;
}

int
sa_getreply(struct sa_session * sa_sess)
{
   int   i, err;

   err = recv(sa_sess->sa_socket, sa_sess->inbuf, SA_INSIZE, 0);
   if(err < 0)
   {
      err = sock_errno(sa_sess->sa_socket);
      if(err == EWOULDBLOCK)
         return SA_REQSENT;

//      dtrap();    /* read failed? */
      return SA_DONE;
   }

   if(err == 0)      /* Nothing returned yet? */
      return SA_REQSENT;

   for(i = 0; i < NUM_REPLYCODES; i++)
   {
      if( strstr(sa_sess->inbuf, replycodes[i].matchText ))
      {
         char * domain;

         domain = br_extractDomain(sa_sess->dnsip->dnsname);

         /* add html to ip entry */
         sprintf(sa_sess->dnsip->ratingHtml, 
            "<a href=\"http://www.siteadvisor.com/sites/%s\" target=top >"
            "<img src=\"%s\" height=16 width=16 border=0 ></a>", 
               domain, replycodes[i].imgname );

         /* Sanity check html string length */
         if(strlen(sa_sess->dnsip->ratingHtml) >= 
            sizeof(sa_sess->dnsip->ratingHtml))
         {
            panic("sa_getreply");
         }

         /* See if we should block this site */
         if((replycodes[i].block) && sa_siteblock) 
         {
            /* Add both the IP address and DNS name to local block list */
//            dtrap();  // later
         }

         socketclose(sa_sess->sa_socket);
         return SA_DONE;
      }
   }

   return SA_REQSENT;
}

/* sa_thread() - the "forever" thread to drive the 
 *
 */

TH_ENTRY(sa_thread)
{
   int   active;  /* number of sesisons actual doing something */
   struct sa_session * sa_sess;
   int   err;

   for(;;)
   {
      active = 0;
      sa_sess = sa_sessions; 
      while(sa_sess)
      {
         switch(sa_sess->state)
         {
         case SA_NEW:
            active++;
            err = sa_connect(sa_sess);
            if(err)
            {
               sa_sess = del_sasess(sa_sess);
               continue;
            }
            else
               sa_sess->state = SA_REQSENT;
            /* Fall to send request */
         case SA_REQSENT:
            active++;
            sa_sess->state = sa_getreply(sa_sess);
            break;
         case SA_DONE:
            sa_sess = del_sasess(sa_sess);
            continue;
         default:
            dtrap();
            break;
         }
         sa_sess = sa_sess->next;
      }


      /* If no session were active, go back to sleep */
      if(active == 0)
      {
         TH_SLEEP(SECOND); /* long sleep - one second */
      }
      else
      {
         TH_SLEEP(1);   /* Short sleep - one time tick */
      }
   }
   TH_RETURN_OK();
}
TH_EXIT("sa_thread")


#endif /* PI_SITEADVISOR - whole file */

/*
Copyright (C) 2003-2008 by Praemio Inc. All rights reserved.

Author: John Bartas - jbartas@praemio.com

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/



