/* plugin.c
 *
 * Buster Plug-In module handling.
 *
 * Copyright 2003-2008 by Praemio Inc. All rights reserved.
 * Author: John Bartas - jbartas@praemio.com
 * This software is released under the terms of the LGPL.
 * See LGPL Notice at end of this file; Full license in ../license.txt
 *
 */

#include "busterport.h"
#include "brnet.h"
#include "ip.h"
#include "brdb.h"


/* Mask of all types registered helps accelerate lookups */
int   pl_mastermask = 0;

struct plug_in * plugins;  /* Master lisst of plugins */


struct plug_in *
add_plugin(
    void (*callback)(void * object, struct plug_in *), 
    int imask)
{
   struct plug_in * newpl;

   newpl = (struct plug_in *)PL_ALLOC( sizeof(struct plug_in) );
   if(!newpl)
      return NULL;

   newpl->callback = callback;
   newpl->pl_type = imask;

   pl_mastermask |= imask;

   /* Add new plugin to master list */
   newpl->next = plugins;
   plugins = newpl;

   return newpl;
}


int  
del_plugin(struct plug_in * to_del)
{
   struct plug_in * pl;
   struct plug_in * last;

   last = NULL;
   for(pl = plugins; pl; pl = pl->next)
   {
      if(pl == to_del)
      {
         if(last)
            last->next = pl->next;
         else
            plugins = pl->next;

         PL_FREE(pl);
         return 0;
      }
   }
   dtrap();
   return -1;
}


void  
pl_newitem(void * object, int typecode)
{
   struct plug_in * pl;

   if((typecode & pl_mastermask) == 0)
      return; 

   /* call back all plug-ins which match the passed type */
   for(pl = plugins; pl; pl = pl->next)
   {
      if(pl->pl_type & typecode)
         pl->callback(object, pl);
   }
   return;
}

/*
Copyright (C) 2003-2008 by Praemio Inc. All rights reserved.

Author: John Bartas - jbartas@praemio.com

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/

